Package: IceSat2R
Type: Package
Title: 'ICESat-2' Altimeter Data using R
Version: 1.1.0
Date: 2026-06-12
Authors@R: c( person(given = "Lampros", family = "Mouselimis", email = "mouselimislampros@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "https://orcid.org/0000-0002-8024-1546")), person("OpenAltimetry.org", role = "cph"))
Description: Programmatic connection to the 'OpenAltimetry' API <https://openaltimetry.earthdatacloud.nasa.gov/data/openapi/swagger-ui/index.html/> to download and process 'ATL03' (Global 'Geolocated' Photon Data), 'ATL06' (Land Ice Height), 'ATL07' (Sea Ice Height), 'ATL08' (Land and Vegetation Height), 'ATL10' (Sea Ice 'Freeboard'), 'ATL12' (Ocean Surface Height) and 'ATL13' (Inland Water Surface Height) 'ICESat-2' Altimeter Data. The user has the option to download the data by selecting a bounding box from a 1- or 5-degree grid globally utilizing a shiny application. The 'ICESat-2' mission collects 'altimetry' data of the Earth's surface. The sole instrument on 'ICESat-2' is the Advanced Topographic Laser Altimeter System (ATLAS) instrument that measures ice sheet elevation change and sea ice thickness, while also generating an estimate of global vegetation biomass. 'ICESat-2' continues the important observations of ice-sheet elevation change, sea-ice 'freeboard', and vegetation canopy height begun by 'ICESat' in 2003.
License: GPL-3
Encoding: UTF-8
URL: https://github.com/mlampros/IceSat2R,
        https://mlampros.github.io/IceSat2R/
BugReports: https://github.com/mlampros/IceSat2R/issues
Depends: R(>= 4.1.0)
Imports: glue, sf, lwgeom, units, data.table, httr, utils, foreach,
        tools, doParallel, magrittr, leaflet, leafgl, htmltools,
        htmlwidgets, leafsync, miniUI, shiny, rnaturalearth, lubridate,
        rvest, withr
Suggests: rmarkdown, knitr, DT, mapview, grDevices, stargazer,
        reshape2, plotly, geodist, CopernicusDEM, terra, testthat (>=
        3.0.0)
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-12 12:56:55 UTC; lampros
Author: Lampros Mouselimis [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8024-1546>),
  OpenAltimetry.org [cph]
Maintainer: Lampros Mouselimis <mouselimislampros@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 14:00:02 UTC
Built: R 4.5.2; ; 2026-06-12 16:40:27 UTC; unix

# Generated by roxygen2: do not edit by hand

export(available_RGTs)
export(available_nominal_orbits)
export(degrees_to_global_grid)
export(download_file)
export(getTracks)
export(get_atlas_data)
export(get_level3a_data)
export(latest_orbits)
export(overall_mission_orbits)
export(revisit_time_RGTs)
export(select_aoi_global_grid)
export(time_specific_orbits)
export(verify_RGTs)
export(vsi_kml_from_zip)
export(vsi_nominal_orbits_wkt)
export(vsi_time_specific_orbits_wkt)
import(foreach)
import(lwgeom)
import(magrittr)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,rbindlist)
importFrom(data.table,setDT)
importFrom(doParallel,registerDoParallel)
importFrom(glue,glue)
importFrom(htmlwidgets,JS)
importFrom(httr,GET)
importFrom(httr,content)
importFrom(httr,http_error)
importFrom(leafgl,addGlPolygons)
importFrom(leafgl,addGlPolylines)
importFrom(leaflet,addLayersControl)
importFrom(leaflet,addProviderTiles)
importFrom(leaflet,filterNULL)
importFrom(leaflet,getMapData)
importFrom(leaflet,invokeMethod)
importFrom(leaflet,layersControlOptions)
importFrom(leaflet,leaflet)
importFrom(leaflet,leafletDependencies)
importFrom(leaflet,providers)
importFrom(leafsync,sync)
importFrom(lubridate,yday)
importFrom(lubridate,ymd)
importFrom(miniUI,gadgetTitleBar)
importFrom(miniUI,miniContentPanel)
importFrom(miniUI,miniPage)
importFrom(miniUI,miniTitleBarButton)
importFrom(rnaturalearth,ne_countries)
importFrom(rvest,html_elements)
importFrom(rvest,read_html)
importFrom(sf,gdal_utils)
importFrom(sf,sf_use_s2)
importFrom(sf,st_area)
importFrom(sf,st_as_sf)
importFrom(sf,st_as_sfc)
importFrom(sf,st_bbox)
importFrom(sf,st_cast)
importFrom(sf,st_crs)
importFrom(sf,st_geometry)
importFrom(sf,st_intersects)
importFrom(sf,st_layers)
importFrom(sf,st_make_grid)
importFrom(sf,st_read)
importFrom(sf,st_zm)
importFrom(shiny,dialogViewer)
importFrom(shiny,isTruthy)
importFrom(shiny,observeEvent)
importFrom(shiny,runGadget)
importFrom(shiny,stopApp)
importFrom(tools,file_ext)
importFrom(units,set_units)
importFrom(utils,download.file)
importFrom(utils,flush.console)
importFrom(utils,globalVariables)
importFrom(utils,unzip)
importFrom(withr,with_options)

<img align="right" width="200" src="man/figures/hex_sticker.png"></img>


## IceSat2R

<br>

Programmatic connection to the [OpenAltimetry EARTHDATA API](https://openaltimetry.earthdatacloud.nasa.gov/data/) to download and process the following *ICESat-2 Altimeter Data*,

* *'ATL03'* (Global Geolocated Photon Data)
* *'ATL06'* (Land Ice Height)
* *'ATL07'* (Sea Ice Height)
* *'ATL08'* (Land and Vegetation Height)
* *'ATL10'* (Sea Ice Freeboard)
* *'ATL12'* (Ocean Surface Height) 
* *'ATL13'* (Inland Water Surface Height) 

The user has the option to download the data by selecting a bounding box from a 1- or 5-degree grid globally utilizing a shiny application. The [Documentation](https://mlampros.github.io/IceSat2R/reference/index.html), the two package Vignettes ([first](https://mlampros.github.io/IceSat2R/articles/IceSat-2_Mission_Orbits_HTML.html), [second](https://mlampros.github.io/IceSat2R/articles/IceSat-2_Atlas_products_HTML.html)) and the [blog post](http://mlampros.github.io/2022/02/12/IceSat2R_Altimetry_data/) explain the functionality in detail.

<br>

The ICESat-2 mission collects altimetry data of the Earth's surface. The sole instrument on ICESat-2 is the Advanced Topographic Laser Altimeter System (ATLAS) instrument that measures ice sheet elevation change and sea ice thickness, while also generating an estimate of global vegetation biomass. 

ICESat-2 continues the important observations of 

* *ice-sheet elevation change*
* *sea-ice freeboard*, and 
* *vegetation canopy height* 

begun by ICESat in 2003.

<br>

#### **System Requirements**

The usage of the *IceSat2R* package requires a geospatial setup as specified in the [sf](https://github.com/r-spatial/sf#installing) or [terra](https://github.com/rspatial/terra#from-source-code) README.md files.

<br>

#### **How the IceSat2R package can be used?**

<br>

The *IceSat2R* package includes the code, documentation, and examples so that,

* A user can select an area of interest (AOI) either programmatically or interactively
* If the Reference Ground Track (RGT) is not known, the user has the option to utilize either
  + one of the "overall_mission_orbits()" or "time_specific_orbits()" to compute the RGT(s) for a pre-specified global area or for a time period, or
  + one of the "vsi_nominal_orbits_wkt()" or "vsi_time_specific_orbits_wkt()" to compute the RGT(s) for a specific AOI
* Once the RGT is computed it can be verified with the "getTracks()" function of the OpenAltimetry EARTHDATA Web API
* Finally the user can utilize one of the "get_atlas_data()" or "get_level3a_data()" functions to retrieve the data for specific product(s), Date(s) and Beam(s)

This work-flow is illustrated also in the following diagram,

<br>

<img src="man/figures/icesat_2_diagram.png" ></img>

<br>

### Shiny application to select an area of interest (AOI) from a 1- or 5-degree global grid

<br>

The [OpenAltimetry EARTHDATA API](https://openaltimetry.earthdatacloud.nasa.gov/data/) restricts the requests to a *1x1* or *5x5* degree spatial bounding box, unless the *"sampling"* parameter is set to *TRUE*. The shiny application of the *IceSat2R* package allows the user to create a spatial grid of an AOI, preferably a 1- or 5-degree grid so that the selection can be within limits. An alternative would be to create a grid of smaller grid cells than required (for instance a 4-degree grid) and then to select multiple grid cells,

<br>

<img src="man/figures/shiny_app_grid.gif" ></img>

<br>

### **Example Use Case-1**: 3-Dimensional Line Plot by combining ICESat-2 and Copernicus DEM (Digital Elevation Model) Data

<br>

The following 3-dimensional interactive line plot (which appears in the ['IceSat-2_Atlas_products' Vignette](https://mlampros.github.io/IceSat2R/articles/IceSat-2_Atlas_products_HTML.html)) shows,

* in *blue* color the *elevation* based on the *DEM* compared to the two ICESat-2 beams ('gt1r' and 'gt2l'), as these are separated by a 3-km distance
* in *orange* color the land-ice-height measurements of the *summer* period (separately for 'gt1r' and 'gt2l')
* in *green* color the land-ice-height measurements of the *winter* period (separately for 'gt1r' and 'gt2l')

<br>

<img src="man/figures/3_dim_plot.gif" ></img>

<br>

### **Example Use Case-2**: Multi-Plot displaying the Ice, Land, Canopy and Copernicus DEM (30-meter) of each beam separately for a specific ICESat-2 Track and area of interest (Himalayas mountain range)

<br>

<img src="man/figures/ice_land_canopy_dem30.gif" ></img>

<br>

#### **Binder**

The user of the **IceSat2R** R package can reproduce the examples of the documentation using the available [binder Rstudio image](https://mybinder.org/v2/gh/mlampros/IceSat2R/HEAD?urlpath=rstudio). Once launched the cloud instance will take a few minutes to be ready. You can read more [about binder on the web](https://mybinder.org/). In short, binder allows to make "your code immediately reproducible by anyone, anywhere". Limitations:

* up to [maximum 100 concurrent users](https://mybinder.readthedocs.io/en/latest/about/user-guidelines.html#maximum-concurrent-users-for-a-repository)
* [1 CPU](https://discourse.jupyter.org/t/mybinder-and-multiprocessing/3238/2)
* [1 to 2 GB of memory](https://mybinder.readthedocs.io/en/latest/about/user-guidelines.html#how-much-memory-am-i-given-when-using-binder)
* [up to six hours of session time per user session](https://mybinder.readthedocs.io/en/latest/about/user-guidelines.html#how-long-will-my-binder-session-last), or up to one cpu-hour for more computationally intensive sessions

<br>

#### **Docker Image**

<br>

**Docker images** of the *IceSat2R* package are available to download from my [dockerhub](https://hub.docker.com/r/mlampros/icesat2r) account. The images come with *Rstudio* and the *R-development* version (latest) installed. The whole process was tested on Ubuntu 18.04. To **pull** & **run** the image do the following,

<br>

```R

docker pull mlampros/icesat2r:rstudiodev

docker run -d --name rstudio_dev -e USER=rstudio -e PASSWORD=give_here_your_password --rm -p 8787:8787 mlampros/icesat2r:rstudiodev

```

<br>

The user can also **bind** a home directory / folder to the image to use its files by specifying the **-v** command,

<br>

```R

docker run -d --name rstudio_dev -e USER=rstudio -e PASSWORD=give_here_your_password --rm -p 8787:8787 -v /home/YOUR_DIR:/home/rstudio/YOUR_DIR mlampros/icesat2r:rstudiodev


```

<br>

The **USER** defaults to *rstudio* but you have to give your **PASSWORD** of preference (see [https://rocker-project.org/](https://rocker-project.org/) for more information).

<br>

Open your web-browser and depending where the docker image was *build / run* give, 

<br>

**1st. Option** on your personal computer,

<br>

```R
http://0.0.0.0:8787 

```

<br>

**2nd. Option** on a cloud instance, 

<br>

```R
http://Public DNS:8787

```

<br>

to access the Rstudio console in order to give your username and password.

<br>

### Installation:

<br>

To install the package from CRAN use, 

```R
install.packages("IceSat2R")

```
<br>

and to download the latest version of the package from Github,

```R
remotes::install_github('mlampros/IceSat2R')

```

<br>

### R package tests:

<br>

To execute the package tests (all or a specific file) use the following code snippet:

```R

# first download the latest version of the package

url_pkg = 'https://github.com/mlampros/IceSat2R/archive/refs/heads/master.zip'
temp_pkg_file = tempfile(fileext = '.zip')
print(temp_pkg_file)

options(timeout = 600)
downl_f = IceSat2R::download_file(url = url_pkg,
                                  destfile = temp_pkg_file,
                                  download_method = 'curl',
                                  verbose = TRUE)

dir_pkg_save = dirname(temp_pkg_file)
utils::unzip(zipfile = temp_pkg_file, exdir = dir_pkg_save, junkpaths = FALSE)

# build and install the latest version of the package

require(glue)

setwd(dir_pkg_save)
system('R CMD build --compact-vignettes="gs+qpdf" --resave-data IceSat2R-master')
gz_file = which(gregexpr(pattern = "^IceSat2R+_+[0-9]+.+[0-9]+.+[0-9]+.tar.gz", text = list.files()) != -1)
system(glue::glue("R CMD INSTALL {list.files()[gz_file]}"))

# load the package

require(IceSat2R)

# run all tests

testthat::test_local(path = file.path(dirname(temp_pkg_file), 'IceSat2R-master'),
                     reporter = testthat::default_reporter())

# run a specific test file from the 'testthat' directory of the package 
# https://github.com/mlampros/IceSat2R/tree/master/tests/testthat

test_specific_file = file.path(dirname(temp_pkg_file), 
                               'IceSat2R-master', 
                               'tests', 
                               'testthat', 
                               'test-mission_orbits.R')

Sys.setenv(NOT_CRAN = "true")       # run all tests (including the ones skipped on CRAN)
testthat::test_file(path = test_specific_file, reporter = testthat::default_reporter())
Sys.unsetenv("NOT_CRAN")            # unset the previously modified environment variable

```

The previous code snippet allows a user to test if the package works as expected in any Operating System.

<br>

### The Beam Pattern

<br>

The ATLAS beam pattern on the ground changes depending on the orientation of the ICESat-2 observatory. The pattern on top (of the following Figure) corresponds to traveling in the forward (+x) orientation, while the pattern on the bottom corresponds to traveling in the backward (-x) orientation. The numbers indicate the corresponding ATLAS beam, while the L/R mapping are used on the ATL03 and higher-level data products. The two strong beams with the TEP are ATLAS beams 1 and 3 (Fig.8, Neumann et al., 2019, https://doi.org/10.1016/j.rse.2019.111325)

<br>

<img src="man/figures/ATLAS_beam_pattern.jpg" ></img>

<br>

Using a table to map the **strong** and **weak** beams (Reference: [sliderule-python documentation](https://docs.slideruleearth.io/background/ICESat-2.html)

<br>

<center> <h1>ATLAS oriented forward (+x)</h1> </center>

| ATLAS Spot Number      | Ground track Designation     | Beam Strength      |
|:----------------------:|:----------------------------:|:------------------:|
| 1                      | gt3r                         | Strong             |
| 2                      | gt3l                         | Weak               |
| 3                      | gt2r                         | Strong             |
| 4                      | gt2l                         | Weak               |
| 5                      | gt1r                         | Strong             |
| 6                      | gt1l                         | Weak               |

<br>

<center> <h1>ATLAS oriented backwards (-x)</h1> </center>

| ATLAS Spot Number      | Ground track Designation     | Beam Strength      |
|:----------------------:|:----------------------------:|:------------------:|
| 1                      | gt3r                         | Weak               |
| 2                      | gt3l                         | Strong             |
| 3                      | gt2r                         | Weak               |
| 4                      | gt2l                         | Strong             |
| 5                      | gt1r                         | Weak               |
| 6                      | gt1l                         | Strong             |

<br>


### Citation:

<br>

If you use the code of this repository in your paper or research please cite both **IceSat2R** and the **original articles** (see CITATION) `https://CRAN.R-project.org/package=IceSat2R`:

<br>

```R
@Manual{,
  title = {{IceSat2R}: ICESat-2 Altimeter Data using R},
  author = {Lampros Mouselimis},
  year = {2026},
  doi = 10.32614/CRAN.package.IceSat2R,
  note = {R package version 1.0.9},
  url = {https://CRAN.R-project.org/package=IceSat2R},
}
```

<br>

## Code of Conduct
  
Please note that the IceSat2R project is released with a [Contributor Code of Conduct](https://github.com/mlampros/IceSat2R/blob/master/CODE_OF_CONDUCT.md). By contributing to this project, you agree to abide by its terms.


## Acknowledgment

This project received financial [support](https://r-consortium.org/all-projects/) from the

<a href="https://r-consortium.org/all-projects/">
<img src="https://r-consortium.org/images/RConsortium_Horizontal_Pantone.webp" width="300">
</a>

<br>

<img align="right" width="200" src="man/figures/hex_sticker.png"></img>


## IceSat2R

<br>

Programmatic connection to the [OpenAltimetry EARTHDATA API](https://openaltimetry.earthdatacloud.nasa.gov/data/) to download and process the following *ICESat-2 Altimeter Data*,

* *'ATL03'* (Global Geolocated Photon Data)
* *'ATL06'* (Land Ice Height)
* *'ATL07'* (Sea Ice Height)
* *'ATL08'* (Land and Vegetation Height)
* *'ATL10'* (Sea Ice Freeboard)
* *'ATL12'* (Ocean Surface Height) 
* *'ATL13'* (Inland Water Surface Height) 

The user has the option to download the data by selecting a bounding box from a 1- or 5-degree grid globally utilizing a shiny application. The [Documentation](https://mlampros.github.io/IceSat2R/reference/index.html), the two package Vignettes ([first](https://mlampros.github.io/IceSat2R/articles/IceSat-2_Mission_Orbits_HTML.html), [second](https://mlampros.github.io/IceSat2R/articles/IceSat-2_Atlas_products_HTML.html)) and the [blog post](http://mlampros.github.io/2022/02/12/IceSat2R_Altimetry_data/) explain the functionality in detail.

<br>

The ICESat-2 mission collects altimetry data of the Earth's surface. The sole instrument on ICESat-2 is the Advanced Topographic Laser Altimeter System (ATLAS) instrument that measures ice sheet elevation change and sea ice thickness, while also generating an estimate of global vegetation biomass. 

ICESat-2 continues the important observations of 

* *ice-sheet elevation change*
* *sea-ice freeboard*, and 
* *vegetation canopy height* 

begun by ICESat in 2003.

<br>

#### **System Requirements**

The usage of the *IceSat2R* package requires a geospatial setup as specified in the [sf](https://github.com/r-spatial/sf#installing) or [terra](https://github.com/rspatial/terra#from-source-code) README.md files.

<br>

#### **How the IceSat2R package can be used?**

<br>

The *IceSat2R* package includes the code, documentation, and examples so that,

* A user can select an area of interest (AOI) either programmatically or interactively
* If the Reference Ground Track (RGT) is not known, the user has the option to utilize either
  + one of the "overall_mission_orbits()" or "time_specific_orbits()" to compute the RGT(s) for a pre-specified global area or for a time period, or
  + one of the "vsi_nominal_orbits_wkt()" or "vsi_time_specific_orbits_wkt()" to compute the RGT(s) for a specific AOI
* Once the RGT is computed it can be verified with the "getTracks()" function of the OpenAltimetry EARTHDATA Web API
* Finally the user can utilize one of the "get_atlas_data()" or "get_level3a_data()" functions to retrieve the data for specific product(s), Date(s) and Beam(s)

This work-flow is illustrated also in the following diagram,

<br>

<img src="man/figures/icesat_2_diagram.png" ></img>

<br>

### Shiny application to select an area of interest (AOI) from a 1- or 5-degree global grid

<br>

The [OpenAltimetry EARTHDATA API](https://openaltimetry.earthdatacloud.nasa.gov/data/) restricts the requests to a *1x1* or *5x5* degree spatial bounding box, unless the *"sampling"* parameter is set to *TRUE*. The shiny application of the *IceSat2R* package allows the user to create a spatial grid of an AOI, preferably a 1- or 5-degree grid so that the selection can be within limits. An alternative would be to create a grid of smaller grid cells than required (for instance a 4-degree grid) and then to select multiple grid cells,

<br>

<img src="man/figures/shiny_app_grid.gif" ></img>

<br>

### **Example Use Case-1**: 3-Dimensional Line Plot by combining ICESat-2 and Copernicus DEM (Digital Elevation Model) Data

<br>

The following 3-dimensional interactive line plot (which appears in the ['IceSat-2_Atlas_products' Vignette](https://mlampros.github.io/IceSat2R/articles/IceSat-2_Atlas_products_HTML.html)) shows,

* in *blue* color the *elevation* based on the *DEM* compared to the two ICESat-2 beams ('gt1r' and 'gt2l'), as these are separated by a 3-km distance
* in *orange* color the land-ice-height measurements of the *summer* period (separately for 'gt1r' and 'gt2l')
* in *green* color the land-ice-height measurements of the *winter* period (separately for 'gt1r' and 'gt2l')

<br>

<img src="man/figures/3_dim_plot.gif" ></img>

<br>

### **Example Use Case-2**: Multi-Plot displaying the Ice, Land, Canopy and Copernicus DEM (30-meter) of each beam separately for a specific ICESat-2 Track and area of interest (Himalayas mountain range)

<br>

<img src="man/figures/ice_land_canopy_dem30.gif" ></img>

<br>

#### **Binder**

The user of the **IceSat2R** R package can reproduce the examples of the documentation using the available [binder Rstudio image](https://mybinder.org/v2/gh/mlampros/IceSat2R/HEAD?urlpath=rstudio). Once launched the cloud instance will take a few minutes to be ready. You can read more [about binder on the web](https://mybinder.org/). In short, binder allows to make "your code immediately reproducible by anyone, anywhere". Limitations:

* up to [maximum 100 concurrent users](https://mybinder.readthedocs.io/en/latest/about/user-guidelines.html#maximum-concurrent-users-for-a-repository)
* [1 CPU](https://discourse.jupyter.org/t/mybinder-and-multiprocessing/3238/2)
* [1 to 2 GB of memory](https://mybinder.readthedocs.io/en/latest/about/user-guidelines.html#how-much-memory-am-i-given-when-using-binder)
* [up to six hours of session time per user session](https://mybinder.readthedocs.io/en/latest/about/user-guidelines.html#how-long-will-my-binder-session-last), or up to one cpu-hour for more computationally intensive sessions

<br>

#### **Docker Image**

<br>

**Docker images** of the *IceSat2R* package are available to download from my [dockerhub](https://hub.docker.com/r/mlampros/icesat2r) account. The images come with *Rstudio* and the *R-development* version (latest) installed. The whole process was tested on Ubuntu 18.04. To **pull** & **run** the image do the following,

<br>

```R

docker pull mlampros/icesat2r:rstudiodev

docker run -d --name rstudio_dev -e USER=rstudio -e PASSWORD=give_here_your_password --rm -p 8787:8787 mlampros/icesat2r:rstudiodev

```

<br>

The user can also **bind** a home directory / folder to the image to use its files by specifying the **-v** command,

<br>

```R

docker run -d --name rstudio_dev -e USER=rstudio -e PASSWORD=give_here_your_password --rm -p 8787:8787 -v /home/YOUR_DIR:/home/rstudio/YOUR_DIR mlampros/icesat2r:rstudiodev


```

<br>

The **USER** defaults to *rstudio* but you have to give your **PASSWORD** of preference (see [https://rocker-project.org/](https://rocker-project.org/) for more information).

<br>

Open your web-browser and depending where the docker image was *build / run* give, 

<br>

**1st. Option** on your personal computer,

<br>

```R
http://0.0.0.0:8787 

```

<br>

**2nd. Option** on a cloud instance, 

<br>

```R
http://Public DNS:8787

```

<br>

to access the Rstudio console in order to give your username and password.

<br>

### Installation:

<br>

To install the package from CRAN use, 

```R
install.packages("IceSat2R")

```
<br>

and to download the latest version of the package from Github,

```R
remotes::install_github('mlampros/IceSat2R')

```

<br>

### R package tests:

<br>

To execute the package tests (all or a specific file) use the following code snippet:

```R

# first download the latest version of the package

url_pkg = 'https://github.com/mlampros/IceSat2R/archive/refs/heads/master.zip'
temp_pkg_file = tempfile(fileext = '.zip')
print(temp_pkg_file)

options(timeout = 600)
downl_f = IceSat2R::download_file(url = url_pkg,
                                  destfile = temp_pkg_file,
                                  download_method = 'curl',
                                  verbose = TRUE)

dir_pkg_save = dirname(temp_pkg_file)
utils::unzip(zipfile = temp_pkg_file, exdir = dir_pkg_save, junkpaths = FALSE)

# build and install the latest version of the package

require(glue)

setwd(dir_pkg_save)
system('R CMD build --compact-vignettes="gs+qpdf" --resave-data IceSat2R-master')
gz_file = which(gregexpr(pattern = "^IceSat2R+_+[0-9]+.+[0-9]+.+[0-9]+.tar.gz", text = list.files()) != -1)
system(glue::glue("R CMD INSTALL {list.files()[gz_file]}"))

# load the package

require(IceSat2R)

# run all tests

testthat::test_local(path = file.path(dirname(temp_pkg_file), 'IceSat2R-master'),
                     reporter = testthat::default_reporter())

# run a specific test file from the 'testthat' directory of the package 
# https://github.com/mlampros/IceSat2R/tree/master/tests/testthat

test_specific_file = file.path(dirname(temp_pkg_file), 
                               'IceSat2R-master', 
                               'tests', 
                               'testthat', 
                               'test-mission_orbits.R')

Sys.setenv(NOT_CRAN = "true")       # run all tests (including the ones skipped on CRAN)
testthat::test_file(path = test_specific_file, reporter = testthat::default_reporter())
Sys.unsetenv("NOT_CRAN")            # unset the previously modified environment variable

```

The previous code snippet allows a user to test if the package works as expected in any Operating System.

<br>

### The Beam Pattern

<br>

The ATLAS beam pattern on the ground changes depending on the orientation of the ICESat-2 observatory. The pattern on top (of the following Figure) corresponds to traveling in the forward (+x) orientation, while the pattern on the bottom corresponds to traveling in the backward (-x) orientation. The numbers indicate the corresponding ATLAS beam, while the L/R mapping are used on the ATL03 and higher-level data products. The two strong beams with the TEP are ATLAS beams 1 and 3 (Fig.8, Neumann et al., 2019, https://doi.org/10.1016/j.rse.2019.111325)

<br>

<img src="man/figures/ATLAS_beam_pattern.jpg" ></img>

<br>

Using a table to map the **strong** and **weak** beams (Reference: [sliderule-python documentation](https://docs.slideruleearth.io/background/ICESat-2.html)

<br>

<center> <h1>ATLAS oriented forward (+x)</h1> </center>

| ATLAS Spot Number      | Ground track Designation     | Beam Strength      |
|:----------------------:|:----------------------------:|:------------------:|
| 1                      | gt3r                         | Strong             |
| 2                      | gt3l                         | Weak               |
| 3                      | gt2r                         | Strong             |
| 4                      | gt2l                         | Weak               |
| 5                      | gt1r                         | Strong             |
| 6                      | gt1l                         | Weak               |

<br>

<center> <h1>ATLAS oriented backwards (-x)</h1> </center>

| ATLAS Spot Number      | Ground track Designation     | Beam Strength      |
|:----------------------:|:----------------------------:|:------------------:|
| 1                      | gt3r                         | Weak               |
| 2                      | gt3l                         | Strong             |
| 3                      | gt2r                         | Weak               |
| 4                      | gt2l                         | Strong             |
| 5                      | gt1r                         | Weak               |
| 6                      | gt1l                         | Strong             |

<br>


### Citation:

<br>

If you use the code of this repository in your paper or research please cite both **IceSat2R** and the **original articles** (see CITATION) `https://CRAN.R-project.org/package=IceSat2R`:

<br>

```R
@Manual{,
  title = {{IceSat2R}: ICESat-2 Altimeter Data using R},
  author = {Lampros Mouselimis},
  year = {2026},
  doi = 10.32614/CRAN.package.IceSat2R,
  note = {R package version 1.1.0},
  url = {https://CRAN.R-project.org/package=IceSat2R},
}
```

<br>

## Code of Conduct
  
Please note that the IceSat2R project is released with a [Contributor Code of Conduct](https://github.com/mlampros/IceSat2R/blob/master/CODE_OF_CONDUCT.md). By contributing to this project, you agree to abide by its terms.


## Acknowledgment

This project received financial [support](https://r-consortium.org/all-projects/) from the

<a href="https://r-consortium.org/all-projects/">
<img src="https://r-consortium.org/images/RConsortium_Horizontal_Pantone.webp" width="300">
</a>

<br>

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(fig.width = 4,
                      fig.height = 6, 
                      fig.align = "center",
                      fig.pos = "!H",
                      warning = FALSE, 
                      message = FALSE,
                      echo = TRUE,
                      eval = TRUE)

## ----himalayas-aoi, fig.pos='h', out.width = "105%", out.height = "100%", fig.align = 'center', fig.cap = "Area of Interest", fig.alt="Display the area of interest in Himalayas", echo = F----

knitr::include_graphics("himalayas_aoi.png")

# mp1 = mapview::mapview(geoms_himal[2, ], legend = F)      # see next code snippet for the "geoms_himal" variable
# mp2 = mapview::mapview(geoms_himal[2, ], legend = F)
# 
# 
# leafsync::sync(mp1, mp2, ncol = 2)

## ----reference-plt-1, echo = F------------------------------------------------
# #......................................................... mapview visualization
# 
# # make the sf-objects valid
# nams = names(lst_out)
# lst_out = lapply(lst_out, function(x) sf::st_make_valid(x))
# names(lst_out) <- nams
# 
# sf_wkt = sf::st_make_valid(sf_wkt)
# 
# # Plot the two sf-objects
# RGTs = mapview::mapview(lst_out, legend = F)
# AOI_wkt = mapview::mapview(sf_wkt, legend = F)
# 
# lft = RGTs + AOI_wkt
# 
# require(magrittr)
# 
# lft = lft@map %>% leaflet::setView(lng = as.numeric(centr_wkt[, 'X']),
#                                    lat = as.numeric(centr_wkt[, 'Y']),
#                                    zoom = 7)
# lft
# #.......................................................


## ----himalayas-rgts, fig.pos='h', out.width = "80%", out.height = "80%", fig.align = 'center', fig.cap = "Intersected RGTs", fig.alt="Intersected Reference Ground Tracks", echo = F----

knitr::include_graphics("himalayas_rgts.png")

#...............................................
# R script using Github Actions on per day basis
# to update the 'technical_specs_urls.csv' file
#...............................................

pth_orb = file.path('inst', 'data_files', 'technical_specs_urls.csv')
upd_orbs = IceSat2R::latest_orbits(verbose = TRUE)
data.table::fwrite(x = upd_orbs, file = pth_orb, row.names = F)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  warning = FALSE,
  message = FALSE,
  fig.pos = "!H",
  echo = TRUE,
  eval = TRUE,
  screenshot.force = FALSE
) # this vignette by default does not evaluate the code snippets

## -----------------------------------------------------------------------------
pkgs <- c(
  "IceSat2R", "sf", "data.table", "stargazer", "glue", "utils",
  "reshape2", "plotly", "magrittr", "geodist", "CopernicusDEM", "terra"
)
load_pkgs <- lapply(pkgs, require, character.only = TRUE) # load required R packages

sf::sf_use_s2(use_s2 = FALSE) # disable 's2' in this vignette
if (requireNamespace("mapview", quietly = TRUE)) {
  mapview::mapviewOptions(
    leafletHeight = "600px",
    leafletWidth = "700px"
  ) # applies to all leaflet maps
}

## ----echo = F-----------------------------------------------------------------
# .........................................................................................
# For reproducibility purposes save only the 'date' and 'RGT' matches of winter and summer
# .........................................................................................

# data.table::fwrite(x = rgts_ws, file = system.file('data_files', 'vignette_data', 'winter_summer_inters.csv', package = "IceSat2R"), row.names = F)

## ----echo = FALSE-------------------------------------------------------------
subs_join_s <- readRDS(file = system.file("data_files", "vignette_data", "Greenland_grid_cells.RDS", package = "IceSat2R"))

## -----------------------------------------------------------------------------
if (requireNamespace("mapview", quietly = TRUE)) {
  mapview::mapview(subs_join_s, legend = F)
}

## -----------------------------------------------------------------------------
join_geoms <- 1:5
subs_join_reduced <- subs_join_s[join_geoms]

if (requireNamespace("mapview", quietly = TRUE)) {
  mapview::mapview(subs_join_reduced, legend = F)
}

## ----echo = F, results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)----
pth_subs_RGTs <- system.file("data_files", "vignette_data", "Greenland_RGTs_grid_cells.csv", package = "IceSat2R")
dtbl_RGTs <- data.table::fread(pth_subs_RGTs, header = T, stringsAsFactors = F)
dtbl_RGTs <- dtbl_RGTs[order(dtbl_RGTs$RGT, decreasing = F), ]

stargazer::stargazer(dtbl_RGTs,
  summary = FALSE,
  rownames = FALSE,
  header = FALSE,
  float = FALSE,
  table.placement = "h",
  title = "Greenland Grid Cells"
)


# #.................................................................
# # I came to these RGT's after processing the next for-loop for
# #   - all 9 Greenland geoms
# #   - all RGT's (of the 'rgts_ws' variable)
# #.................................................................
#
# require(magrittr)
# logs_out_dtbl = data.table::rbindlist(logs_out)         # the output of the 'logs_out' list is required!
# logs_out_dtbl$index = names(dat_out_w)                  # the output of the 'dat_out_w' list is required!
# logs_out_dtbl = logs_out_dtbl[order(logs_out_dtbl$N_rows_winter, decreasing = T), ]
# logs_out_dtbl_subs = subset(logs_out_dtbl, N_rows_winter > 0 & N_rows_summer > 0)
# vec_grid = as.vector(unlist(lapply(strsplit(logs_out_dtbl_subs$index, '_'), function(x) x[3])))
# logs_out_dtbl_subs$greenland_cell = vec_grid
# spl_rgt = split(logs_out_dtbl_subs, by = 'greenland_cell')
# spl_subs = spl_rgt[as.character(1:5)] %>%
#   lapply(function(x) x[1:4, ]) %>%
#   data.table::rbindlist()

## ----echo = FALSE-------------------------------------------------------------
pth_logs <- system.file("data_files", "vignette_data", "LOGs.csv", package = "IceSat2R")
logs_out_dtbl <- data.table::fread(file = pth_logs, stringsAsFactors = F, header = T)

## ----results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)----
logs_out_dtbl <- logs_out_dtbl[order(logs_out_dtbl$N_rows_winter, decreasing = T), ]

stargazer::stargazer(logs_out_dtbl,
  summary = FALSE,
  rownames = FALSE,
  header = FALSE,
  float = FALSE,
  table.placement = "h",
  title = "LOGs"
)

## ----echo = FALSE-------------------------------------------------------------
ws_vis <- readRDS(file = system.file("data_files", "vignette_data", "plotly_beams.RDS", package = "IceSat2R"))

## ----eval = requireNamespace("reshape2", quietly = TRUE)----------------------
ws_vis_mlt <- reshape2::melt(ws_vis, id.vars = c("segment_id_winter", "beam_winter"))
ws_vis_mlt <- data.table::data.table(ws_vis_mlt, stringsAsFactors = F)
ws_vis_mlt_spl <- split(ws_vis_mlt, by = "beam_winter")
# BEAMS = names(ws_vis_mlt_spl)       # plot all beams

## -----------------------------------------------------------------------------
# ...................................
# function to plot each subplot beam
# ...................................

plotly_beams <- function(spl_data,
                         beam,
                         left_width,
                         left_height,
                         right_width,
                         right_height) {
  subs_iter <- spl_data[[beam]]

  cat(glue::glue("Plot for Beam '{beam}' will be created ..."), "\n")

  # ......................
  # plot for all segments
  # ......................

  fig_lns <- plot_ly(
    data = subs_iter,
    x = ~segment_id_winter,
    y = ~value,
    color = ~variable,
    colors = c("blue", "red"),
    line = list(width = 2),
    text = ~ glue::glue("land-ice-height: {value} Segment-id: {segment_id_winter}"),
    hoverinfo = "text",
    width = left_width,
    height = left_height
  ) %>%
    plotly::layout(
      xaxis = list(gridcolor = "grey", showgrid = T),
      yaxis = list(gridcolor = "grey", showgrid = T)
    ) %>%
    plotly::add_lines()

  # ..............................
  # plot for a subset of segments
  # ..............................

  segm_ids <- 588326:588908 # this subset of segments show a big difference betw. summer and winter
  plt_title <- glue::glue("Beam: '{beam}' ( Segments: from {min(segm_ids)} to {max(segm_ids)} )")
  subs_iter_segm <- subset(subs_iter, segment_id_winter %in% segm_ids)

  fig_spl <- plot_ly(
    data = subs_iter_segm,
    x = ~segment_id_winter,
    y = ~value,
    color = ~variable,
    colors = c("blue", "red"),
    line = list(width = 2),
    text = ~ glue::glue("land-ice-height: {value} Segment-id: {segment_id_winter}"),
    hoverinfo = "text",
    width = right_width,
    height = right_height
  ) %>%
    plotly::layout(
      xaxis = list(gridcolor = "grey", showgrid = T),
      yaxis = list(gridcolor = "grey", showgrid = T)
    ) %>%
    plotly::add_lines(showlegend = FALSE)

  both_plt <- plotly::subplot(list(fig_lns, fig_spl), nrows = 1, margin = 0.03, widths = c(0.7, 0.3)) %>%
    plotly::layout(title = plt_title)
  # plotly::export(p = both_plt, file = glue::glue('{beam}.png'))

  return(both_plt)
}

## ----eval = requireNamespace("plotly", quietly = TRUE) && requireNamespace("reshape2", quietly = TRUE)----
plt_gt1l <- plotly_beams(
  spl_data = ws_vis_mlt_spl,
  beam = "gt1l",
  left_width = 1800,
  left_height = 800,
  right_width = 900,
  right_height = 400
)

plt_gt1l

## ----eval = requireNamespace("plotly", quietly = TRUE) && requireNamespace("reshape2", quietly = TRUE)----
plt_gt1r <- plotly_beams(
  spl_data = ws_vis_mlt_spl,
  beam = "gt1r",
  left_width = 1800,
  left_height = 800,
  right_width = 900,
  right_height = 400
)
plt_gt1r

## ----echo = F-----------------------------------------------------------------
# #......................................................
# # save all images for all beams in a separate directory
# #......................................................
#
# nams_ws = names(dat_out_w)
# save_summary = save_dat = list()
#
# for (nam_iter in nams_ws) {
#
#   cat("-----------------\n")
#   cat(nam_iter, '\n')
#   cat("-----------------\n")
#
#   w_subs = dat_out_w[[nam_iter]]
#   s_subs = dat_out_s[[nam_iter]]
#
#   cols_keep = c('date', 'segment_id', 'longitude', 'latitude', 'h_li', 'beam')
#
#   w_subs_hq = subset(w_subs, atl06_quality_summary == 0)
#   w_subs_hq = w_subs_hq[, ..cols_keep]
#   colnames(w_subs_hq) = glue::glue("{cols_keep}_winter")
#
#   s_subs_hq = subset(s_subs, atl06_quality_summary == 0)
#   s_subs_hq = s_subs_hq[, ..cols_keep]
#   colnames(s_subs_hq) = glue::glue("{cols_keep}_summer")
#
#   sw_hq_merg = merge(x = w_subs_hq,
#                      y = s_subs_hq,
#                      by.x = c('segment_id_winter', 'beam_winter'),
#                      by.y = c('segment_id_summer', 'beam_summer'))
#
#   if (nrow(sw_hq_merg) > 0) {
#     sw_hq_merg$dif_height = sw_hq_merg$h_li_winter - sw_hq_merg$h_li_summer
#
#     save_dat[[nam_iter]] = sw_hq_merg
#     save_summary[[nam_iter]] = data.table::setDT(list(name_iter = nam_iter,
#                                                       min = min(sw_hq_merg$dif_height),
#                                                       mean = mean(sw_hq_merg$dif_height),
#                                                       median = median(sw_hq_merg$dif_height),
#                                                       max = max(sw_hq_merg$dif_height),
#                                                       N_rows = nrow(sw_hq_merg)))
#     #.......................................
#     # save the plots for visual verification
#     #.......................................
#
#     cols_viz = c('segment_id_winter', 'beam_winter', 'h_li_winter', 'h_li_summer')
#     ws_vis = sw_hq_merg[, ..cols_viz]
#
#     ws_vis_mlt = reshape2::melt(ws_vis, id.vars = c('segment_id_winter', 'beam_winter'))
#     ws_vis_mlt = data.table::data.table(ws_vis_mlt, stringsAsFactors = F)
#     ws_vis_mlt_spl = split(ws_vis_mlt, by = 'beam_winter')
#
#     dir_save = file.path('all_beams_all_RGTs', nam_iter)           # !! create the 'all_beams_all_RGTs' directory first
#     if (!dir.exists(dir_save)) dir.create(dir_save)
#
#     BEAMS = names(ws_vis_mlt_spl)       # plot all beams
#
#     for (beam in BEAMS) {
#
#       subs_iter = ws_vis_mlt_spl[[beam]]
#
#       cat(glue::glue("Plot for Beam '{beam}' will be saved ..."), '\n')
#
#       #......................
#       # plot for all segments
#       #......................
#
#       fig_lines = plotly::plot_ly(data = subs_iter,
#                                   x = ~segment_id_winter,
#                                   y = ~value,
#                                   color = ~variable,
#                                   colors = c("blue", "red"),
#                                   line = list(width = 2),
#                                   text = ~glue::glue("land-ice-height: {value}  Segment-id: {segment_id_winter}"),
#                                   hoverinfo = "text",
#                                   width = 1800,
#                                   height = 1000) %>%
#
#         plotly::layout(xaxis = list(gridcolor = "grey", showgrid = T),
#                        yaxis = list(gridcolor = "grey", showgrid = T)) %>%
#
#         plotly::add_lines()
#
#       plotly::export(p = fig_lines, file = file.path(dir_save, glue::glue('{beam}.png')))
#     }
#   }
#   else {
#    message(glue::glue("Empty data table after merging for idx and RGT: '{nam_iter}'"))
#   }
# }
#
# save_summary = data.table::rbindlist(save_summary)
# save_summary = save_summary[order(save_summary$max, decreasing = T), ]
# save_summary

## ----echo = FALSE-------------------------------------------------------------
# merg_cells_viz = readRDS(file = system.file('data_files', 'vignette_data', '3d_plot.RDS', package = "IceSat2R"))

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  fig.width = 4,
  fig.height = 4,
  fig.align = "center",
  fig.pos = "!H",
  warning = FALSE,
  message = FALSE,
  echo = TRUE,
  eval = TRUE
)

## -----------------------------------------------------------------------------
pkgs <- c(
  "IceSat2R", "magrittr", "sf", "rnaturalearth",
  "data.table", "DT", "stargazer"
)
load_pkgs <- lapply(pkgs, require, character.only = TRUE) # load required R packages

sf::sf_use_s2(use_s2 = FALSE) # disable 's2' in this vignette
if (requireNamespace("mapview", quietly = TRUE)) {
  mapview::mapviewOptions(
    leafletHeight = "600px",
    leafletWidth = "700px"
  ) # applies to all leaflet maps
}

# .............................
# load the 'RGT_cycle_14' data
# .............................

data(RGT_cycle_14)

res_rgt_many <- sf::st_as_sf(x = RGT_cycle_14, coords = c("longitude", "latitude"), crs = 4326)
res_rgt_many

## -----------------------------------------------------------------------------
cntr <- rnaturalearth::ne_countries(scale = 110, type = "countries", returnclass = "sf")
cntr <- cntr[, c("sovereignt", "sov_a3")]
cntr

## -----------------------------------------------------------------------------
dat_both <- suppressMessages(sf::st_join(
  x = res_rgt_many,
  y = cntr,
  join = sf::st_intersects,
  left = TRUE
))
dat_both

## -----------------------------------------------------------------------------
length(unique(dat_both$RGT))

## -----------------------------------------------------------------------------
df_tbl <- data.frame(table(dat_both$sovereignt), stringsAsFactors = F)
colnames(df_tbl) <- c("country", "Num_IceSat2_points")

df_subs <- dat_both[, c("RGT", "sovereignt")]
df_subs$geometry <- NULL
df_subs <- data.table::data.table(df_subs, stringsAsFactors = F)
colnames(df_subs) <- c("RGT", "country")
df_subs <- split(df_subs, by = "country")
df_subs <- lapply(df_subs, function(x) {
  unq_rgt <- sort(unique(x$RGT))
  items <- ifelse(length(unq_rgt) < 5, length(unq_rgt), 5)
  concat <- paste(unq_rgt[1:items], collapse = "-")
  iter_dat <- data.table::setDT(list(
    country = unique(x$country),
    Num_RGTs = length(unq_rgt),
    first_5_RGTs = concat
  ))
  iter_dat
})

df_subs <- data.table::rbindlist(df_subs)

df_tbl <- merge(df_tbl, df_subs, by = "country")
df_tbl <- df_tbl[order(df_tbl$Num_IceSat2_points, decreasing = T), ]

## ----eval = requireNamespace("DT", quietly = TRUE)----------------------------
DT_dtbl <- DT::datatable(df_tbl, rownames = FALSE)

## ----echo = FALSE, eval = requireNamespace("DT", quietly = TRUE)--------------
DT_dtbl

## -----------------------------------------------------------------------------
num_sea <- sum(is.na(dat_both$sovereignt))
num_land <- sum(!is.na(dat_both$sovereignt))

perc_sea <- round(num_sea / nrow(dat_both), digits = 4) * 100.0
perc_land <- round(num_land / nrow(dat_both), digits = 4) * 100.0

dtbl_land_sea <- data.frame(list(
  percentage = c(perc_sea, perc_land),
  Num_Icesat2_points = c(num_sea, num_land)
))

row.names(dtbl_land_sea) <- c("sea", "land")

## ----results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)----
stargazer::stargazer(dtbl_land_sea,
  summary = FALSE,
  rownames = TRUE,
  header = FALSE,
  float = FALSE,
  table.placement = "!h",
  title = "Land and Sea Proportions"
)

## -----------------------------------------------------------------------------
data(ne_10m_glaciated_areas)

## -----------------------------------------------------------------------------
ne_obj_subs <- subset(ne_10m_glaciated_areas, !is.na(name))
ne_obj_subs <- sf::st_make_valid(x = ne_obj_subs) # check validity of geometries
ne_obj_subs

## -----------------------------------------------------------------------------
if (requireNamespace("mapview", quietly = TRUE)) {
  mpv <- mapview::mapview(ne_obj_subs,
    color = "cyan",
    col.regions = "blue",
    alpha.regions = 0.5,
    legend = FALSE
  )
  mpv
}

## -----------------------------------------------------------------------------
res_rgt_many$id_rgt <- 1:nrow(res_rgt_many) # include 'id' for fast subsetting

dat_glac_sf <- suppressMessages(sf::st_join(
  x = ne_obj_subs,
  y = res_rgt_many,
  join = sf::st_intersects
))

dat_glac <- data.table::data.table(sf::st_drop_geometry(dat_glac_sf), stringsAsFactors = F)
dat_glac <- dat_glac[complete.cases(dat_glac), ] # keep non-NA observations
dat_glac

## -----------------------------------------------------------------------------
dat_glac_name <- split(x = dat_glac, by = "name")

sum_stats_glac <- lapply(dat_glac_name, function(x) {
  dtbl_glac <- x[, .(
    name_glacier = unique(name),
    Num_unique_Dates = length(unique(Date)),
    Num_unique_RGTs = length(unique(RGT))
  )]
  dtbl_glac
})

sum_stats_glac <- data.table::rbindlist(sum_stats_glac)
sum_stats_glac <- sum_stats_glac[order(sum_stats_glac$Num_unique_RGTs, decreasing = T), ]

## ----results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)----
stargazer::stargazer(sum_stats_glac,
  summary = FALSE,
  rownames = FALSE,
  header = FALSE,
  float = FALSE,
  table.placement = "h",
  title = "Days and RGTs"
)

## -----------------------------------------------------------------------------
sample_glacier <- "Southern Patagonian Ice Field"
dat_glac_smpl <- dat_glac_name[[sample_glacier]]

## ----results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)----
cols_display <- c("name", "day_of_year", "Date", "hour", "minute", "second", "RGT")

stargazer::stargazer(dat_glac_smpl[, ..cols_display],
  summary = FALSE,
  rownames = FALSE,
  header = FALSE,
  float = FALSE,
  table.placement = "h",
  title = "Southern Patagonian Ice Field"
)

## -----------------------------------------------------------------------------
subs_rgts <- subset(res_rgt_many, id_rgt %in% dat_glac_smpl$id_rgt)

set.seed(1)
samp_colrs <- sample(
  x = grDevices::colors(distinct = TRUE),
  size = nrow(subs_rgts)
)
subs_rgts$color <- samp_colrs

## -----------------------------------------------------------------------------
ne_obj_subs_smpl <- subset(ne_obj_subs, name == sample_glacier)

if (requireNamespace("mapview", quietly = TRUE)) {
  mpv_glacier <- mapview::mapview(ne_obj_subs_smpl,
    color = "cyan",
    col.regions = "blue",
    alpha.regions = 0.5,
    legend = FALSE
  )

  mpv_RGTs <- mapview::mapview(subs_rgts,
    color = subs_rgts$color,
    alpha.regions = 0.0,
    lwd = 6,
    legend = FALSE
  )
}

## -----------------------------------------------------------------------------
if (requireNamespace("mapview", quietly = TRUE)) {
  lft <- mpv_glacier + mpv_RGTs
  lft
}

utils::globalVariables(c(
  "i", # for the foreach for-loop
  "Type"
)) # for the Type column when I use the base R subset() function


#' Extraction of the url from the Technical Specification Website
#'
#' This function allows the user to view the latest 'Nominal' and 'Time Specfic' orbit metadata (Url, Reference Ground Track Names, Dates and Types)
#'
#' @param technical_specs_url a character string specifying the technical specs website
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return a 'data.table' object
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' @importFrom rvest read_html html_elements
#' @importFrom data.table data.table
#' @importFrom lubridate ymd
#' @import magrittr
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' orbs <- latest_orbits(verbose = TRUE)
#' orbs
#' }
latest_orbits <- function(technical_specs_url = "https://icesat-2.gsfc.nasa.gov/science/specs",
                          verbose = FALSE) {
  if (verbose) {
    t_start <- proc.time()
    cat("Access the data of the technical specs website ...\n")
  }

  pg <- rvest::read_html(technical_specs_url)

  if (verbose) cat("Extract the .zip files and the corresponding titles ...\n")
  url_zipfiles <- pg %>%
    rvest::html_elements("a[href$='.zip']") %>% # see: https://stackoverflow.com/a/48146333
    as.character()
  # rvest::html_text2()                               # returns only the url-titles

  if (verbose) cat("Keep the relevant data from the url's and titles ...\n")
  df_out <- lapply(seq_along(url_zipfiles), function(x) {
    item <- url_zipfiles[x]
    item <- gsub(pattern = '<a href=\"', replacement = "", x = item)
    item <- gsub(pattern = "</a>", replacement = "", x = item)

    idx_str <- which(gregexpr(pattern = "<strong>", text = item) != -1)
    if (length(idx_str) > 0) {
      item <- gsub(pattern = "<strong>", replacement = "", x = item)
      item <- gsub(pattern = "</strong>", replacement = "", x = item)
    }

    item <- strsplit(x = item, split = '[\">]')[[1]]
    item <- item[item != ""]
    item <- trimws(x = item, which = "both")
    item
  })

  df_out <- do.call(rbind, df_out)

  if (verbose) cat("Process the nominal and time specific orbits separately ...\n")
  idx_time_spec <- which(gregexpr(pattern = "^Cycle", text = df_out[, 2]) != -1)
  nominal_orbits <- df_out[-idx_time_spec, ]
  time_specific <- df_out[idx_time_spec, ]

  if (verbose) cat("Adjust the Dates of the time specific orbits ...\n")
  dates_spec <- as.vector(unlist(lapply(strsplit(x = time_specific[, 2], split = ":"), function(x) trimws(x[2], which = "both"))))
  cycles_spec <- as.vector(unlist(lapply(strsplit(x = time_specific[, 2], split = ":"), function(x) trimws(x[1], which = "both"))))
  cycles_spec <- gsub(pattern = " ", replacement = "_", x = cycles_spec)

  dates_spec <- gsub("[(),]", "", dates_spec) %>%
    strsplit(split = "-") %>%
    lapply(function(x) trimws(x, which = "both")) %>%
    lapply(function(x) {
      y <- strsplit(x, split = " ")
      y1 <- y[[1]]
      y2 <- y[[2]]
      if (length(y2) != 3) stop("The second sublist must consist of 3 items!", call. = F)
      if (length(y1) == 2) {
        y1 <- c(y1, y2[3])
      }
      y1 <- paste(c(y1[3], y1[1], y1[2]), collapse = "-")
      y2 <- paste(c(y2[3], y2[1], y2[2]), collapse = "-")
      c(as.character(lubridate::ymd(y1)), as.character(lubridate::ymd(y2)))
    })

  any_nas <- as.vector(unlist(lapply(dates_spec, function(x) any(is.na(x)))))
  if (any(any_nas)) stop("The conversion to dates returned missing values!", call. = F)

  dates_spec <- do.call(rbind, dates_spec)

  if (verbose) cat("Create the nominal orbits data.table ...\n")
  nominal_orbits <- data.table::data.table(nominal_orbits, stringsAsFactors = F)
  colnames(nominal_orbits) <- c("Url", "RGT")
  nominal_orbits$Date_from <- NA_character_
  nominal_orbits$Date_to <- NA_character_
  nominal_orbits$Type <- "Nominal"

  if (verbose) cat("Create the time specific orbits data.table ...\n")
  time_specific <- data.table::data.table(time_specific[, 1, drop = F], stringsAsFactors = F)
  colnames(time_specific) <- "Url"
  time_specific$RGT <- trimws(gsub(pattern = ".zip", "", basename(path = time_specific$Url)), which = "both")
  time_specific$Date_from <- dates_spec[, 1]
  time_specific$Date_to <- dates_spec[, 2]
  time_specific$Type <- "Time_Specific"

  if (verbose) cat("Return a single data.table ...\n")
  res_both <- rbind(nominal_orbits, time_specific)
  if (verbose) compute_elapsed_time(time_start = t_start)

  return(res_both)
}


#' Nominal mission orbits
#'
#' This function allows the user to view the nominal orbits (all or a selection)
#'
#' @param orbit_area either NULL or a character string specifying the earth partition to use, it can be one of 'antarctic', 'arctic', 'western_hemisphere' and 'eastern_hemisphere'
#' @param technical_specs_url a character string specifying the technical specs website
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return a list object with the available nominal orbits
#'
#' @importFrom glue glue
#' @importFrom data.table fread
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' @export
#'
#' @examples
#'
#' require(IceSat2R)
#'
#' # .............................
#' # all available nominal orbits
#' # .............................
#'
#' nomin_orb <- available_nominal_orbits(verbose = TRUE)
#' nomin_orb
#'
#' # ........................
#' # specific nominal orbits
#' # ........................
#'
#' nomin_orb <- available_nominal_orbits(
#'   orbit_area = "arctic",
#'   verbose = TRUE
#' )
#' nomin_orb
#'
available_nominal_orbits <- function(orbit_area = NULL,
                                     technical_specs_url = "https://icesat-2.gsfc.nasa.gov/science/specs",
                                     verbose = FALSE) {
  if (verbose) cat(glue::glue("The available Icesat-2 orbits will be red from '{technical_specs_url}' ..."), "\n")
  avail_urls <- tryCatch(latest_orbits(technical_specs_url = technical_specs_url, verbose = verbose), error = function(e) e)
  if (inherits(avail_urls, "error")) {
    if (verbose) message(glue::glue("The 'latest_orbits()' function gave the following error: '{avail_urls$message}'! The existing orbits file will be uploaded!"))
    avail_urls <- data.table::fread(file = system.file("data_files", "technical_specs_urls.csv", package = "IceSat2R"), stringsAsFactors = F, header = T)
  }

  nom_orb <- subset(avail_urls, Type == "Nominal")

  rgts <- as.vector(unlist(lapply(strsplit(x = nom_orb$RGT, split = " "), function(x) {
    if (length(x) == 2) {
      x <- x[1]
    } else {
      x <- x[1:2]
    }
    x <- paste(x, collapse = "_")
    tolower(x)
  })))

  nom_orb$RGT <- rgts
  intern_orbits <- list()
  for (i in 1:nrow(nom_orb)) {
    intern_orbits[[nom_orb$RGT[i]]] <- nom_orb$Url[i]
  }

  if (!is.null(orbit_area)) {
    if (!all(orbit_area %in% names(intern_orbits))) stop(glue::glue("The 'orbit_area' parameter can be one of {paste(rgts, collapse = ', ')}!"), call. = F)
    intern_orbits <- intern_orbits[orbit_area]
  }

  return(intern_orbits)
}



#' Overall Mission Orbits
#'
#' This function allows the user to view information of the nominal mission orbits and beam locations: "The processed files have 7 tracks per
#' orbit: one for each of the six beams of ICESat-2, and the seventh for the Reference Ground Track (RGT). The RGT is an imaginary line
#' through the six-beam pattern that is handy for getting a sense of where the orbits fall on Earth, and which the mission uses to point the
#' observatory. However, the six tracks for the six beams are our best estimate of where the beams will actually fall on Earth's surface."
#'
#' @param orbit_area a character string specifying the nominal mission orbits and beam locations. It can be one of 'antarctic', 'arctic', 'western_hemisphere' or 'eastern_hemisphere'
#' @param download_method a character string specifying the download method. Corresponds to the 'method' parameter of the 'utils::download.file()' function. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'
#' @param threads an integer that specifies the number of threads to use in parallel when processing the data
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return an 'sf' object of multiple tracks (see the 'LAYER' column of the output object)
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' @importFrom glue glue
#' @importFrom utils download.file unzip
#' @importFrom tools file_ext
#' @importFrom doParallel registerDoParallel
#' @importFrom sf st_layers st_read st_as_sf st_zm
#' @importFrom data.table rbindlist
#' @import foreach
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' res_orb <- overall_mission_orbits(
#'   orbit_area = "antarctic",
#'   download_method = "curl",
#'   threads = 1,
#'   verbose = TRUE
#' )
#' str(res_orb)
#' }
overall_mission_orbits <- function(orbit_area,
                                   download_method = "curl",
                                   threads = 1,
                                   verbose = FALSE) {
  if (verbose) {
    t_start <- t_downl <- proc.time()
  }

  orbits <- available_nominal_orbits(orbit_area = orbit_area, verbose = verbose)

  tmp_orbit_file <- tempfile(fileext = ".zip")
  if (verbose) cat(glue::glue("The orbits of '{orbit_area}' will be downloaded (temporarily) in the '{tmp_orbit_file}' file ..."), "\n")

  downl_u <- download_file(
    url = orbits[[orbit_area]],
    destfile = tmp_orbit_file,
    download_method = download_method,
    verbose = verbose
  )

  tmp_orbits_dir <- file.path(tempdir(), glue::glue("orbits_{orbit_area}")) # create a second directory inside the temporary directory to extract the .zip file
  if (!dir.exists(tmp_orbits_dir)) dir.create(tmp_orbits_dir)

  if (verbose) cat(glue::glue("The downloaded .zip file will be extracted in the '{tmp_orbits_dir}' directory ..."), "\n")
  utils::unzip(zipfile = tmp_orbit_file, exdir = tmp_orbits_dir, junkpaths = T)

  lst_kmz <- list.files(path = tmp_orbits_dir, full.names = F, pattern = ".kmz")
  LEN <- length(lst_kmz)
  if (LEN == 0) stop(glue::glue("The directory '{tmp_orbits_dir}' does not include any .kmz files!"), call. = F)
  EXT <- unique(tools::file_ext(lst_kmz))[1]

  if (verbose) {
    cat(glue::glue("Download and unzip the {LEN} '.{EXT}' files: "))
    compute_elapsed_time(time_start = t_downl)
    t_proc <- proc.time()
    cat(glue::glue("An 'sf' object will be created from the {LEN} .kmz files."), "\n")
  }

  if (threads > 1 & LEN > 1) {
    if (verbose) cat(glue::glue("Parallel processing of {LEN} '.{EXT}' files using  {threads}  threads starts ..."), "\n")

    if (.Platform$OS.type == "unix") {
      doParallel::registerDoParallel(cores = threads)
    }

    if (.Platform$OS.type == "windows") {
      cl <- parallel::makePSOCKcluster(threads)
      doParallel::registerDoParallel(cl = cl) # compared to unix, ".. if not specified, on Windows a three worker cluster is created and used .." [ see also: https://stackoverflow.com/a/45122448/8302386 ]
    }

    res_out <- foreach::foreach(i = 1:LEN) %dopar% { # parallelize the 'OUTER' loop because I have few layers
      FILE <- lst_kmz[i]
      iter_file <- file.path(tmp_orbits_dir, FILE)
      lrs <- sf::st_layers(dsn = iter_file) # fist get the layers
      NAMS <- lrs$name
      if (length(NAMS) == 0) stop(glue::glue("The file '{FILE}' does not include any layers!"), call. = F)
      nam_file <- gsub(".kmz", "", FILE)
      sf_objs <- list()

      for (LAYER in NAMS) {
        lr_dat <- sf::st_read(dsn = iter_file, layer = LAYER, quiet = T)
        nam_layer <- gsub(".kml", "", LAYER)
        lr_dat$LAYER <- rep(nam_layer, nrow = lr_dat)
        sf_objs[[glue::glue("{nam_file}_{nam_layer}")]] <- lr_dat
      }

      sf_objs
    }

    res_out <- unlist(res_out, recursive = F) # in case of nested lists return a single list object removing sublists
  } else {
    if (verbose) cat(glue::glue("A single thread will be used to process the '.{EXT}' files ..."), "\n")
    res_out <- list()

    for (FILE in lst_kmz) {
      iter_file <- file.path(tmp_orbits_dir, FILE)
      lrs <- sf::st_layers(dsn = iter_file)
      NAMS <- lrs$name
      if (length(NAMS) == 0) stop(glue::glue("The file '{FILE}' does not include any layers!"), call. = F)

      nam_file <- gsub(".kmz", "", FILE)

      for (LAYER in NAMS) {
        if (verbose) cat(glue::glue("File '{FILE}' and Layer '{LAYER}' will be processed ..."), "\n")
        lr_dat <- sf::st_read(dsn = iter_file, layer = LAYER, quiet = T)
        nam_layer <- gsub(".kml", "", LAYER)
        lr_dat$LAYER <- rep(nam_layer, nrow = lr_dat)
        res_out[[glue::glue("{nam_file}_{nam_layer}")]] <- lr_dat
      }
    }
  }

  if (verbose) {
    cat(glue::glue("Processing of the {LEN} '.{EXT}' files: "))
    compute_elapsed_time(time_start = t_proc)
  }

  if (verbose) cat("The temproary files will be removed ...\n")
  for (tmp_file in lst_kmz) { # remove the .kmz files
    pth_kmz <- file.path(tmp_orbits_dir, tmp_file)
    if (file.exists(pth_kmz)) file.remove(pth_kmz)
  }
  if (file.exists(tmp_orbit_file)) file.remove(tmp_orbit_file) # remove the .zip file

  if (verbose) cat(glue::glue("Create a single 'sf' object from the {length(res_out)} sublists and drop the Z dimension ..."), "\n")
  res_out <- sf::st_as_sf(data.table::rbindlist(res_out))
  res_out <- sf::st_zm(x = res_out, drop = T) # drop the Z dimension

  if (verbose) {
    cat("Total ")
    compute_elapsed_time(time_start = t_start)
  }
  return(res_out)
}



#' Reference Ground Tracks (RGTs)
#'
#' This function returns the url's of all Reference Ground Track (RGT) cycles. Moreover, it returns the dates that belong to each RGT cycle and the names of the RGT cycles.
#'
#' @param only_cycle_names a boolean. If TRUE then only the RGT (Reference Ground Track) cycle names will be returned. Otherwise all orbit files, dates and cycle names.
#' @param technical_specs_url a character string specifying the technical specs website
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return a list object with the available orbit files, dates and cycle names
#'
#' @importFrom glue glue
#' @importFrom data.table fread
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' # .................................................
#' # all available orbit files, dates and cycle names
#' # .................................................
#'
#' avail_dat <- available_RGTs(
#'   only_cycle_names = FALSE,
#'   verbose = TRUE
#' )
#' avail_dat
#'
#' # .............................
#' # receive only the cycle names
#' # .............................
#'
#' avail_cycles <- available_RGTs(
#'   only_cycle_names = TRUE,
#'   verbose = TRUE
#' )
#' avail_cycles
#' }
available_RGTs <- function(only_cycle_names = FALSE,
                           technical_specs_url = "https://icesat-2.gsfc.nasa.gov/science/specs",
                           verbose = FALSE) {
  if (verbose) cat(glue::glue("The available Icesat-2 orbits will be red from '{technical_specs_url}' ..."), "\n")
  avail_urls <- tryCatch(latest_orbits(technical_specs_url = technical_specs_url, verbose = verbose), error = function(e) e)
  if (inherits(avail_urls, "error")) {
    if (verbose) message(glue::glue("The 'latest_orbits()' function gave the following error: '{avail_urls$message}'! The existing orbits file will be uploaded!"))
    avail_urls <- data.table::fread(file = system.file("data_files", "technical_specs_urls.csv", package = "IceSat2R"), stringsAsFactors = F, header = T)
  }

  time_spec <- subset(avail_urls, Type == "Time_Specific")
  time_spec$Date_from <- as.Date(time_spec$Date_from)
  time_spec$Date_to <- as.Date(time_spec$Date_to)
  time_spec <- time_spec[order(time_spec$Date_from, decreasing = F), ] # sort by date to have the RGT-cycles ordered in increasing order

  orbit_files <- orbit_dates <- list()

  for (i in 1:nrow(time_spec)) {
    nam_i <- glue::glue("RGT_cycle_{i}")
    orbit_files[[nam_i]] <- time_spec$Url[i]
    orbit_dates[[nam_i]] <- seq(from = time_spec$Date_from[i], to = time_spec$Date_to[i], by = "1 day")
  }

  orb_nams <- names(orbit_files)

  if (only_cycle_names) {
    return(orb_nams)
  } else {
    return(list(
      orbit_files = orbit_files,
      orbit_dates = orbit_dates,
      names_cycles = orb_nams
    ))
  }
}



#' Revisit Time Reference Ground Tracks and Dates
#'
#' This function shows the information of the 'available_RGTs' function and additionally it returns the .zip (kmz files) of all laser
#' tracks over each 91-day repeat period (revisit time). Note that the locations and times are estimates, but should be correct to
#' within a few minutes in time and better than 100m in the predicted locations.
#'
#' @param RGT_cycle either NULL or a character string specifying a single RGT (Reference Ground Track) as determined by the output of the 'available_RGTs(only_cycle_names = TRUE)' function. If NULL then all available Data will be returned.
#' @param complete_date_sequence a boolean. If TRUE a complete sequence of Dates will be returned, otherwise only the 'minimum' and 'maximum' Dates.
#'
#' @return a list object with the available orbit files, dates and date sequence lengths
#'
#' @details
#'
#' ICESat-2 was in safe-hold from June 26 through July 9, 2019. ATLAS was off during this time, so data was not collected or pointed to the reference ground track.
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' @importFrom glue glue
#'
#' @export
#'
#' @examples
#'
#' require(IceSat2R)
#'
#' # .......................................................
#' # receive all orbit files, dates and length of sequences
#' # .......................................................
#'
#' rev_all <- revisit_time_RGTs(RGT_cycle = NULL, complete_date_sequence = TRUE)
#' rev_all
#'
#' # ...................................................
#' # observe and choose one of the available RGT-cycles
#' # ...................................................
#'
#' avail_cycles <- available_RGTs(
#'   only_cycle_names = TRUE,
#'   verbose = TRUE
#' )
#' avail_cycles
#'
#' # .....................................
#' # receive results for a specific cycle
#' # .....................................
#'
#' rev_cycle <- revisit_time_RGTs(RGT_cycle = "RGT_cycle_1", complete_date_sequence = FALSE)
#' rev_cycle
revisit_time_RGTs <- function(RGT_cycle = NULL,
                              complete_date_sequence = FALSE) {
  avail_rgts <- available_RGTs(only_cycle_names = FALSE)
  orbit_files <- avail_rgts$orbit_files
  orbit_dates <- avail_rgts$orbit_dates
  orb_nams <- names(orbit_files)

  if (!is.null(RGT_cycle)) {
    if (!RGT_cycle %in% orb_nams) {
      quote_nams <- as.vector(sapply(orb_nams, function(x) glue::glue("'{x}'")))
      concat_nams <- paste(quote_nams, collapse = ", ")
      stop(glue::glue("Available cycles are the following: {concat_nams}"), call. = F)
    }

    orbit_files <- orbit_files[[RGT_cycle]]
    orbit_dates <- orbit_dates[[RGT_cycle]]
    LEN_seq <- length(orbit_dates)

    if (!complete_date_sequence) {
      orbit_dates <- c(min(orbit_dates), max(orbit_dates))
    }
  } else {
    LEN_seq <- lapply(orbit_dates, function(x) length(x))

    if (!complete_date_sequence) {
      orbit_dates <- lapply(orbit_dates, function(x) {
        c(min(x), max(x))
      })
    }
  }

  return(list(
    orbit_files = orbit_files,
    orbit_dates = orbit_dates,
    length_dates = LEN_seq
  ))
}



#' Time Specific Orbits
#'
#' This function shows the reference ground track time and locations for specific date ranges. "Updated KML files have been posted to the 'tech-specs'
#' website (see the 'references' section for more details) containing individual files for each Reference Ground Track (RGT) with a date and time stamp
#' posted every 420 kilometers along-track (roughly 1 minute of flight time in between each point). The first RGT is 234; this is where the time series
#' begins. The date of each RGT is in the file name, so the user can easily ascertain where and when ICESat-2 will be on a particular day."
#'
#' @param date_from either NULL or a character string specifying the start date in the format 'yyyy-MM-dd' (such as '2020-01-01'). If this parameter is NULL then the 'RGT_cycle' parameter must be specified
#' @param date_to either NULL or a character string specifying the end date in the format 'yyyy-MM-dd' (such as '2020-01-01'). If this parameter is NULL then the 'RGT_cycle' parameter must be specified
#' @param RGT_cycle a character vector specifying the RGT (Reference Ground Track) cycle(s) (the specific revisit times of ICESAT-2). This parameter can be greater or equal to 1 with a maximum 'RGT-cycle' names as determined by the output of the 'available_RGTs(only_cycle_names = TRUE)' function. The computation time of a single 'RGT-cycle' might take approximately 15 minutes utilizing 8 threads (in parallel) and require approximately 2 GB of memory.
#' @param download_method a character string specifying the download method. Corresponds to the 'method' parameter of the 'utils::download.file()' function. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'
#' @param threads an integer that specifies the number of threads to use in parallel when processing the data
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return an 'sf' object that will include one or more Reference Ground Tracks  (see the 'RGT' column of the output object)
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' @importFrom glue glue
#' @importFrom utils download.file unzip flush.console
#' @importFrom doParallel registerDoParallel
#' @importFrom sf st_layers st_read st_as_sf st_zm
#' @importFrom data.table setDT rbindlist data.table
#' @import foreach
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' # ................................................
#' # RGTs (Reference Ground Tracks) for a single day
#' # ................................................
#'
#' res_rgt_one <- time_specific_orbits(
#'   date_from = "2019-06-01",
#'   date_to = "2019-06-01",
#'   download_method = "curl",
#'   threads = 1,
#'   verbose = TRUE
#' )
#' str(res_rgt_one)
#'
#' # ..........................................................
#' # RGTs (Reference Ground Tracks) for a specific time period
#' # ..........................................................
#'
#' res_rgt_many <- time_specific_orbits(
#'   date_from = "2019-06-01",
#'   date_to = "2019-06-03",
#'   download_method = "curl",
#'   threads = 1,
#'   verbose = TRUE
#' )
#' str(res_rgt_many)
#'
#'
#' # .........................................................
#' # processing more than one RGTs for a specified date range
#' # .........................................................
#'
#' res_rgt_inters <- time_specific_orbits(
#'   date_from = "2021-03-23",
#'   date_to = "2021-03-26",
#'   download_method = "curl",
#'   threads = 1,
#'   verbose = TRUE
#' )
#' str(res_rgt_inters)
#'
#' table(res_rgt_inters$cycle)
#' table(res_rgt_inters$day_of_year)
#' table(res_rgt_inters$RGT)
#'
#' # ...............................................................
#' # RGTs (Reference Ground Tracks) for a selected 'cycle'
#' # Observe the available RGT-cycles and use all available threads
#' # ...............................................................
#'
#' avail_cycles <- available_RGTs(
#'   only_cycle_names = TRUE,
#'   verbose = TRUE
#' )
#' avail_cycles
#'
#' choose_cycle <- avail_cycles[3]
#'
#' res_rgt_many <- time_specific_orbits(
#'   RGT_cycle = choose_cycle,
#'   download_method = "curl",
#'   threads = parallel::detectCores(),
#'   verbose = TRUE
#' )
#' }
time_specific_orbits <- function(date_from = NULL,
                                 date_to = NULL,
                                 RGT_cycle = NULL,
                                 download_method = "curl",
                                 threads = 1,
                                 verbose = FALSE) {
  if (verbose) t_start <- proc.time()
  if (all(c(!is.null(date_from), !is.null(date_to), !is.null(RGT_cycle)))) stop("You have to specify either an RGT cycle ('RGT_cycle' parameter) or the 'date_from' and 'date_to' parameters!", call. = F)

  avail_rgts <- available_RGTs(only_cycle_names = FALSE, verbose = verbose)
  orbit_files <- avail_rgts$orbit_files
  orbit_dates <- avail_rgts$orbit_dates
  cycle_names <- avail_rgts$names_cycles

  rgt_cycle_null <- is.null(RGT_cycle)

  if (rgt_cycle_null) {
    if (any(c(is.null(date_from), is.null(date_to)))) {
      stop("If the 'RGT_cycle' parameter is NULL then both 'date_from' and 'date_to' must be specified!", call. = F)
    } else {
      orbit_dates <- lapply(seq_along(cycle_names), function(x) {
        iter_lst <- list(date = orbit_dates[[x]])
        iter_dates <- data.table::setDT(iter_lst)
        iter_dates$cycle <- rep(cycle_names[x], nrow(iter_dates))
        iter_dates
      })

      orbit_dates <- data.table::rbindlist(orbit_dates)
      orbit_dates <- orbit_dates[order(orbit_dates$date, decreasing = F), ]

      complete_dates <- seq(from = orbit_dates$date[1], to = orbit_dates$date[length(orbit_dates$date)], by = "1 day")
      complete_dates <- data.table::setDT(list(date = complete_dates))
      orbit_dates <- merge(orbit_dates, complete_dates, by = "date", all = T)
      orbit_dates <- orbit_dates[order(orbit_dates$date, decreasing = F), ]
      if (verbose) cat(glue::glue("ICESAT-2 orbits: 'Earliest-Date' is '{orbit_dates$date[1]}'  'Latest-Date' is '{orbit_dates$date[length(orbit_dates$date)]}'"), "\n")

      if (!inherits(date_from, "Date")) date_from <- as.Date(date_from)
      if (!inherits(date_to, "Date")) date_to <- as.Date(date_to)
      if (date_from > date_to) stop("The 'date_from' parameter must not be greater than the 'date_to' parameter!", call. = F)

      idx_select <- which(orbit_dates$date >= date_from & orbit_dates$date <= date_to)
      if (length(idx_select) > 0) {
        orbit_dates <- orbit_dates[idx_select, , drop = F]

        idx_missing <- which(is.na(orbit_dates$cycle))
        if (length(idx_missing) > 0) {
          message(glue::glue("There are {length(idx_missing)} missing values (NA's) in the selected Date range from '{date_from}' to '{date_to}' and will be removed (for more information consult the 'https://icesat-2.gsfc.nasa.gov/science/specs' webpage)!"))
          orbit_dates <- orbit_dates[-idx_missing, , drop = F]
          if (nrow(orbit_dates) == 0) {
            stop("After removing the missing values an empty data.table was returned! Please consult the 'https://icesat-2.gsfc.nasa.gov/science/specs' webpage for more information!", call. = F)
          }
        }
      } else {
        stop("There are no intersected Dates between the ICESAT-2 orbit files and the input 'date_from' and 'date_to' parameters!", call. = F)
      }

      RGT_cycle <- unique(orbit_dates$cycle)
    }
  }

  all_cycles <- list()

  for (CYCLE in RGT_cycle) {
    if (verbose) {
      cat("-----------------------------------------------------\n")
      cat(glue::glue("The .zip file of '{CYCLE}' will be downloaded ..."), "\n")
      cat("-----------------------------------------------------\n")
      t_downl <- proc.time()
    }

    URL_zip <- orbit_files[[CYCLE]]

    tmp_orbit_file <- tempfile(fileext = ".zip")

    downl_u <- download_file(
      url = URL_zip,
      destfile = tmp_orbit_file,
      download_method = download_method,
      verbose = verbose
    )

    tmp_orbits_dir <- file.path(tempdir(), CYCLE) # create a second directory inside the temporary directory to extract the .zip file
    if (!dir.exists(tmp_orbits_dir)) dir.create(tmp_orbits_dir)

    if (verbose) cat(glue::glue("The downloaded .zip file will be extracted in the '{tmp_orbits_dir}' directory ..."), "\n")
    utils::unzip(zipfile = tmp_orbit_file, exdir = tmp_orbits_dir, junkpaths = T)

    if (verbose) {
      cat(glue::glue("Download and unzip the {CYCLE} .zip file: "))
      compute_elapsed_time(time_start = t_downl)
    }

    lst_kmz <- list.files(path = tmp_orbits_dir, full.names = F, pattern = ".kml") # there are 2 kind of files in the directory that begin with 'IS2_RGT_...' and '._IS2_RGT_..' [ the last type beginning with dot '.' is not a regular .kml file and won't be processed ]
    LEN <- length(lst_kmz)
    if (LEN == 0) stop(glue::glue("The directory '{tmp_orbits_dir}' does not include any .kmz files!"), call. = F)

    dtbl_files <- data.table::setDT(list(files = lst_kmz))
    vec_dates_files <- trimws(as.vector(unlist(lapply(strsplit(lst_kmz, "[_.]"), function(x) x[length(x) - 1]))), which = "both")
    # vec_dates_files = as.Date(strptime(x = vec_dates_files, format = "%d-%b-%Y"))                                                   # this command might not work for specific system locale
    vec_dates_files <- as.vector(unlist(lapply(strsplit(vec_dates_files, "-"), function(x) {
      iter_month <- switch_abb(x[2])
      as.character(glue::glue("{x[3]}-{iter_month}-{x[1]}"))
    })))
    vec_dates_files <- as.Date(vec_dates_files)
    dtbl_files$date <- vec_dates_files

    if (rgt_cycle_null) {
      merg_dat <- merge(orbit_dates, dtbl_files, by = "date")

      if (nrow(merg_dat) == 0) { # make sure that after merging I have files for processing
        print(orbit_dates)
        print("------------------")
        print(dtbl_files)
        stop("There is no intersection between the orbit file dates and the downloaded .zip file .kml files!", call. = F)
      }
    } else {
      merg_dat <- dtbl_files
    }

    LEN_subset <- nrow(merg_dat)
    if (verbose) {
      cat(glue::glue("{LEN_subset} .kml files will be processed ..."), "\n")
      t_proc <- proc.time()
    }

    sf_objs <- list()

    if (threads > 1 & LEN_subset > 1) {
      if (verbose) cat(glue::glue("Parallel processing of {LEN_subset} .kml files using  {threads}  threads starts ..."), "\n")

      if (.Platform$OS.type == "unix") {
        doParallel::registerDoParallel(cores = threads)
      }

      if (.Platform$OS.type == "windows") {
        cl <- parallel::makePSOCKcluster(threads)
        doParallel::registerDoParallel(cl = cl) # compared to unix, ".. if not specified, on Windows a three worker cluster is created and used .." [ see also: https://stackoverflow.com/a/45122448/8302386 ]
      }
    } else {
      if (verbose) cat(glue::glue("Sequential processing of {LEN_subset} .kml files using a single thread starts ..."), "\n")
    }

    for (idx in 1:LEN_subset) {
      if (verbose) {
        message("FILE: ", idx, "/", LEN_subset, "\r", appendLF = FALSE)
        utils::flush.console()
      }

      FILE <- merg_dat$files[idx]
      iter_file <- file.path(tmp_orbits_dir, FILE)
      lrs <- sf::st_layers(dsn = iter_file)
      NAMS <- lrs$name
      if (length(NAMS) == 0) stop(glue::glue("The file '{FILE}' does not include any layers!"), call. = F)
      nam_file <- gsub(".kml", "", FILE)
      LEN_nams <- length(NAMS)

      if (threads > 1 & LEN_subset > 1) {
        inner_obj <- foreach::foreach(i = 1:LEN_nams) %dopar% {
          LAYER <- NAMS[i]
          lr_dat <- sf::st_read(dsn = iter_file, layer = LAYER, quiet = T) # I expect each .kml file to consist of a single 'sfc_POINT'
          lr_dat
        }
      } else {
        inner_obj <- lapply(1:LEN_nams, function(x) {
          LAYER <- NAMS[x]
          lr_dat <- sf::st_read(dsn = iter_file, layer = LAYER, quiet = T) # I expect each .kml file to consist of a single 'sfc_POINT'
          lr_dat
        })
      }

      # class_obj = as.vector(unlist(lapply(inner_obj, function(x) class(sf::st_geometry(x))[1])))                      # I expect each sublist to be of type "sfc_POINT" and normally observations which are not (and can be for instance "sfc_LINESTRING") won't have a description column either. Therefore use the next line for removal
      descr_not_idx <- which(as.vector(unlist(lapply(inner_obj, function(x) {
        colnams <- colnames(x)
        if ("description" %in% colnams) {
          verify_descr <- (is.na(x$description) | x$description == "")
        } else if ("Description" %in% colnams) {
          verify_descr <- (is.na(x$Description) | x$Description == "") # In Macintosh (osx) the "description" column appears with an upper case "D" as "Description", see the following issue: https://github.com/mlampros/IceSat2R/issues/9#issuecomment-1152020607
        } else {
          stop("The 'D(d)escription' column must exist in every sublist!", call. = F)
        }
        verify_descr
      }))))

      LEN_exc <- length(descr_not_idx)
      if (LEN_exc > 0) {
        if (verbose) message(glue::glue("{LEN_exc} output sublist did not have a valid 'description' and will be removed!"))
        inner_obj <- inner_obj[-descr_not_idx]
      }

      inner_obj <- sf::st_as_sf(data.table::rbindlist(inner_obj))
      sf_objs[[glue::glue("{nam_file}")]] <- inner_obj
    }

    sf_objs <- sf::st_as_sf(data.table::rbindlist(sf_objs))

    if (verbose) cat("The 'description' column of the output data will be processed ...\n")
    # descr_proc = strsplit(x = sf_objs$description, split = '[ \n]')                           # split either by newline or by space  [ it's wrong because it splits the date and time too ]

    colnams_dtbl <- colnames(sf_objs)
    flag_upper_descr <- FALSE
    if ("description" %in% colnams_dtbl) { # see previous exception regarding "D(d)escription" (here I assume that only "description" or "Description" cases exist)
      sf_objs$description <- as.character(sf_objs$description)
      descr_proc <- strsplit(x = sf_objs$description, split = "\n")
    } else {
      sf_objs$Description <- as.character(sf_objs$Description)
      # ---- PREVIOUS CODE (commented out) ----
      # descr_proc <- strsplit(x = sf_objs$Description, split = " ") # in case that I have a 'Description' column, split by empty space
      # # ........................................................................... "start" error case with missing 'time' for midnight on Windows
      # len_desc <- lapply(descr_proc, function(x) length(x)) |>
      #   unlist() |>
      #   table()
      # len_desc <- as.integer(names(len_desc)[which.max(len_desc)])
      # # If the modal token count is odd, the expected even length is len_desc + 1.
      # # This handles the case where NASA changed the orbit-file Description format
      # # so that ALL rows now omit the time (i.e. the midnight-missing format
      # # became the majority rather than the exception).
      # len_desc_even <- if (len_desc %% 2 == 0) len_desc else len_desc + 1L
      # chk_row_items <- as.vector(unlist(lapply(descr_proc, function(x) (length(x) %% 2) == 0))) # check that I have an even number of columns (normally 8 but can be fewer too) then concatenate the columns by pairs of consecutive items
      # if (!all(chk_row_items)) {
      #   idx_not <- which(!chk_row_items)
      #   for (idx_i in idx_not) {
      #     item_i <- descr_proc[[idx_i]]
      #     if (length(item_i) == (len_desc_even - 1)) { # we expect that only the time is missing
      #       # This is the current exception:
      #       # RGT 1264 15-Dec-2020 DOY 350 Cycle 9                         # midnight time is missing, i.e. "00:00:00" (time)
      #       # RGT 1264 16-Dec-2020 00:01:00 DOY 351 Cycle 9                # after midnight case (as expected)
      #       item_i <- append(x = item_i, values = "00:00:00", after = 3) # add the time after the date
      #       descr_proc[[idx_i]] <- item_i
      #     } else { # throw an error in any other case
      #       len_dif <- len_desc_even - length(item_i)
      #       stop(glue::glue("We expect a difference of maximum one and received a difference of length {len_dif}, which means {len_dif} attributes are missing from the character string!"))
      #     }
      #   }
      # }
      # chk_row_items <- as.vector(unlist(lapply(descr_proc, function(x) (length(x) %% 2) == 0)))
      # if (!all(chk_row_items)) {
      #   stop("We expect after the code adjustments to receive equal length of vector (split) character strings!")
      # }
      # # ........................................................................... "end" error case with missing 'time' for midnight on Windows
      # descr_proc <- lapply(descr_proc, function(x) {
      #   seq_item <- seq(from = 1, to = length(x), by = 2)
      #   sapply(seq_item, function(y) paste(c(x[y], x[y + 1]), collapse = " "))
      # })
      # flag_upper_descr <- TRUE
      # ---- END PREVIOUS CODE ----
      # Detect whether Description entries use newline separators (new NASA KMZ format <U+2014>
      # CRLF on Windows) or space separators (classic Windows/macOS format).
      # When CRLF is embedded in space-split tokens (e.g. "1264\r\n15-Dec-2020"), both
      # idx_rgt and idx_date_time resolve to the same column and the second setnames call
      # silently overwrites the first, making descr_proc[["RGT"]] return NULL.
      non_empty_idx <- which(nchar(sf_objs$Description) > 0 & !is.na(sf_objs$Description))
      uses_newlines <- length(non_empty_idx) > 0 &&
        grepl("[\r\n]", sf_objs$Description[[non_empty_idx[[1]]]], perl = TRUE)
      if (uses_newlines) {
        # New format: Description uses CR/LF separators <U+2014> identical logical structure to
        # the lowercase "description" path, so split the same way.
        descr_proc <- strsplit(x = sf_objs$Description, split = "[\r\n]+", perl = TRUE)
      } else {
        # Classic format: Description uses space-separated tokens that must be paired.
        descr_proc <- strsplit(x = sf_objs$Description, split = " ") # in case that I have a 'Description' column, split by empty space
        # ........................................................................... "start" error case with missing 'time' for midnight on Windows
        len_desc <- lapply(descr_proc, function(x) length(x)) |>
          unlist() |>
          table()
        len_desc <- as.integer(names(len_desc)[which.max(len_desc)])
        # If the modal token count is odd, the expected even length is len_desc + 1.
        # This handles the case where NASA changed the orbit-file Description format
        # so that ALL rows now omit the time (i.e. the midnight-missing format
        # became the majority rather than the exception).
        len_desc_even <- if (len_desc %% 2 == 0) len_desc else len_desc + 1L
        chk_row_items <- as.vector(unlist(lapply(descr_proc, function(x) (length(x) %% 2) == 0))) # check that I have an even number of columns (normally 8 but can be fewer too) then concatenate the columns by pairs of consecutive items
        if (!all(chk_row_items)) {
          idx_not <- which(!chk_row_items)
          for (idx_i in idx_not) {
            item_i <- descr_proc[[idx_i]]
            if (length(item_i) == (len_desc_even - 1)) { # we expect that only the time is missing
              # This is the current exception:
              # RGT 1264 15-Dec-2020 DOY 350 Cycle 9                         # midnight time is missing, i.e. "00:00:00" (time)
              # RGT 1264 16-Dec-2020 00:01:00 DOY 351 Cycle 9                # after midnight case (as expected)
              item_i <- append(x = item_i, values = "00:00:00", after = 3) # add the time after the date
              descr_proc[[idx_i]] <- item_i
            } else { # throw an error in any other case
              len_dif <- len_desc_even - length(item_i)
              stop(glue::glue("We expect a difference of maximum one and received a difference of length {len_dif}, which means {len_dif} attributes are missing from the character string!"))
            }
          }
        }
        chk_row_items <- as.vector(unlist(lapply(descr_proc, function(x) (length(x) %% 2) == 0)))
        if (!all(chk_row_items)) {
          stop("We expect after the code adjustments to receive equal length of vector (split) character strings!")
        }
        # ........................................................................... "end" error case with missing 'time' for midnight on Windows
        descr_proc <- lapply(descr_proc, function(x) {
          seq_item <- seq(from = 1, to = length(x), by = 2)
          sapply(seq_item, function(y) paste(c(x[y], x[y + 1]), collapse = " "))
        })
      }
      flag_upper_descr <- TRUE
    }
    descr_proc <- data.table::as.data.table(do.call(rbind, descr_proc))

    # ............................................................................................................................................
    # !! IMPORTANT !! There was a case where only 'RGT' and 'Date_time' were returned. I have to include the other 2 columns too by using NA's
    #                 I have to guess the column-names based on the 1st row of the "descr_proc" data

    samp_rownames <- as.vector(unlist(descr_proc[1, , drop = T]))
    idx_rgt <- which(as.vector(unlist(lapply(gregexpr(pattern = "RGT", text = samp_rownames), function(x) all(x != -1)))))
    if (length(idx_rgt) == 0) stop("I expect that the RGT column exists!", call. = F)

    idx_date_time <- which(as.vector(unlist(lapply(gregexpr(pattern = "-", text = samp_rownames), function(x) all(x != -1)))))
    if (length(idx_date_time) == 0) stop("I expect that the Date-Time column exists!", call. = F)

    data.table::setnames(descr_proc, idx_rgt[1], "RGT")
    data.table::setnames(descr_proc, idx_date_time[1], "Date_time")

    idx_doy <- which(as.vector(unlist(lapply(gregexpr(pattern = "DOY", text = samp_rownames), function(x) all(x != -1)))))
    if (length(idx_doy) == 0) {
      descr_proc$DOY <- rep(NA_integer_, nrow(descr_proc))
    } else {
      data.table::setnames(descr_proc, idx_doy[1], "DOY")
    }

    idx_cycle <- which(as.vector(unlist(lapply(gregexpr(pattern = "Cycle", text = samp_rownames), function(x) all(x != -1)))))
    if (length(idx_cycle) == 0) {
      descr_proc$cycle <- rep(NA_integer_, nrow(descr_proc))
    } else {
      data.table::setnames(descr_proc, idx_cycle[1], "cycle")
    }

    # colnames(descr_proc) = c('RGT', 'Date_time', 'DOY', 'cycle')   # this is the order that normally appears in the output data
    # ............................................................................................................................................

    if (flag_upper_descr) {
      sf_objs$Description <- NULL
    } else {
      sf_objs$description <- NULL
    }

    sf_objs$RGT <- as.integer(gsub("RGT ", "", descr_proc[["RGT"]])) # keep the integer from the character string which corresponds to the 'RGT'
    sf_objs$Date_time <- descr_proc[["Date_time"]]
    sf_objs$day_of_year <- as.integer(gsub("DOY ", "", descr_proc[["DOY"]])) # keep the integer from the character string which corresponds to the 'DOY'
    sf_objs$cycle <- as.integer(gsub("Cycle ", "", descr_proc[["cycle"]])) # keep the integer from the character string which corresponds to the 'cycle'
    sf_objs <- sf::st_zm(x = sf_objs, drop = T) # drop the Z dimension

    all_cycles[[CYCLE]] <- sf_objs

    if (verbose) cat("The temproary files will be removed ...\n")
    for (tmp_file in lst_kmz) { # remove the .kmz files
      pth_kmz <- file.path(tmp_orbits_dir, tmp_file)
      if (file.exists(pth_kmz)) file.remove(pth_kmz)
    }
    if (file.exists(tmp_orbit_file)) file.remove(tmp_orbit_file) # remove the .zip file

    if (verbose) {
      cat(glue::glue("Processing of cycle '{CYCLE}': "))
      compute_elapsed_time(time_start = t_proc)
    }
  }

  all_cycles <- sf::st_as_sf(data.table::rbindlist(all_cycles))
  # all_cycles$Date_time = strptime(x = all_cycles$Date_time, format = "%d-%b-%Y %H:%M:%S")          # convert to Date-Time ('POSIXlt') format  [ !! I have to do that outside the for-loop, otherwise I might receive an error ]
  vec_dates_cycl <- as.vector(unlist(lapply(strsplit(all_cycles$Date_time, " "), function(x) {
    iter_date <- strsplit(x[1], "-")[[1]]
    iter_month <- switch_abb(iter_date[2])
    as.character(glue::glue("{iter_date[3]}-{iter_month}-{iter_date[1]} {x[2]}"))
  })))
  all_cycles$Date_time <- as.POSIXlt(vec_dates_cycl)

  if (verbose) {
    cat("Total ")
    compute_elapsed_time(time_start = t_start)
  }

  return(all_cycles)
}



#' Utilizing Virtual File Systems (vsi) to extract the .kml from the .zip file
#'
#' This function returns the '.kml' and '.kmz' files in form of virtual file paths. Moreover, the user has the option to download these files.
#'
#' @param icesat_rgt_url a character string specifying the input .zip URL
#' @param download_zip a boolean. If TRUE the .zip file will be first downloaded and then the .kml files will be returned, otherwise the 'gdalinfo' function will be used as input to the R 'system2()' function to read the .kml files without downloading the .zip file. The 'gdalinfo' command requires that the user has configured GDAL properly.
#' @param download_method a character string specifying the download method. Corresponds to the 'method' parameter of the 'utils::download.file()' function. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return an one column data.table of the output files
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' https://gdal.org/user/virtual_file_systems.html
#'
#' @importFrom glue glue
#' @importFrom utils download.file unzip
#' @importFrom tools file_ext
#' @importFrom sf gdal_utils
#' @importFrom data.table setDT
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' URL <- "https://icesat-2.gsfc.nasa.gov/sites/default/files/page_files/arcticallorbits.zip"
#'
#' # ..................................
#' # without downloading the .zip file
#' # ..................................
#'
#' res_out <- vsi_kml_from_zip(
#'   icesat_rgt_url = URL,
#'   download_zip = FALSE,
#'   download_method = "curl",
#'   verbose = TRUE
#' )
#' str(res_out)
#'
#'
#' # .............................
#' # by downloading the .zip file
#' # .............................
#'
#' res_out <- vsi_kml_from_zip(
#'   icesat_rgt_url = URL,
#'   download_zip = TRUE,
#'   download_method = "curl",
#'   verbose = TRUE
#' )
#' str(res_out)
#' }
vsi_kml_from_zip <- function(icesat_rgt_url,
                             download_zip = FALSE,
                             download_method = "curl",
                             verbose = FALSE) {
  if (verbose) t_start <- proc.time()
  EXT_url <- tools::file_ext(icesat_rgt_url)
  url_pth <- as.character(glue::glue("/vsizip/vsicurl/{icesat_rgt_url}"))

  if (download_zip) {
    tmp_url_file <- tempfile(fileext = as.character(glue::glue(".{EXT_url}")))

    if (verbose) cat(glue::glue("The '{icesat_rgt_url}' file will be downloaded (temporarily) in the '{tmp_url_file}' file ..."), "\n")

    downl_u <- download_file(
      url = icesat_rgt_url,
      destfile = tmp_url_file,
      download_method = download_method,
      verbose = verbose
    )

    info_url <- utils::unzip(zipfile = tmp_url_file, list = TRUE, junkpaths = T)
    info_url <- info_url$Name
    if (length(info_url) == 0) stop(glue::glue("The 'unzip -l' command didn't returned the required .kml url paths for the input url: {icesat_rgt_url}!"), call. = F)
  } else {
    info_url <- sf::gdal_utils(
      util = "info",
      source = url_pth,
      quiet = T
    )

    if (any(c(info_url == "", length(info_url) == 0))) {
      warning("The 'sf' gdalinfo returned an empty character string (or vector)! Attempt to read the url using the OS configured 'gdalinfo' function!  (As an alternative you can set the parameter 'download_zip' to TRUE and re-run the function)")

      info_url <- suppressWarnings(tryCatch(system2(command = "gdalinfo", args = url_pth, stdout = TRUE, stderr = FALSE), error = function(e) e))

      if (inherits(info_url, "error")) stop(glue::glue("The OS 'gdalinfo' gave the following error: {info_url$message}!"), call. = F)
      if (length(info_url) == 0) stop(glue::glue("The OS 'gdalinfo' didn't returned the required .kml url paths for the gdal input url: {url_pth}!"), call. = F)
    }
  }

  out_files <- tools::file_ext(info_url)
  tbl_files <- data.frame(table(out_files))
  tbl_files <- tbl_files[order(tbl_files$Freq, decreasing = T), ]
  type_files <- as.character(tbl_files$out_files[1])
  if (verbose) cat(glue::glue("The internal type of the .zip file is '{type_files}'"), "\n")

  idx_kml <- which(out_files == type_files)
  if (length(idx_kml) == 0) stop("The .zip file (of the specified 'icesat_rgt_url' parameter) does not include any .kml files!", call. = F)
  info_url <- info_url[idx_kml]
  info_url <- trimws(info_url, which = "both")

  macosx_idx <- which(gregexpr(pattern = "MACOSX", text = info_url) != -1)
  LEN_mc <- length(macosx_idx)
  if (LEN_mc > 0) {
    if (verbose) message(glue::glue("{LEN_mc} files will be removed because they are hidden MACOSX files!"))
    info_url <- info_url[-macosx_idx]
  }

  LEN_url <- length(info_url)
  flag_error <- F

  if (type_files == "kml") {
    if (!LEN_url %in% c(1221, 1387)) {
      flag_error <- T
      msg <- "Except for the 'IS2_RGTs_cycle4_date_time_rev2_0.zip' file which has 1221, I expect that the remaining have 1387 ground track .kml files in every cycle (every cycle consists of approximately 91 days)!"
    }
  }

  if (type_files == "kmz") {
    if (!LEN_url %in% c(7, 56)) {
      flag_error <- T
      msg <- "I expect that the 'antarcticaallorbits' and 'arcticallorbits' consist of 7 URLs ( 6 beams + 1 RGT ), whereas the 'ICESat2groundtracksWesternHem' and 'ICESat2groundtracksEasternHem' consist of 56 URLs ( 8 repeats * [6 beams + 1 RGT] )!"
    }
  }

  if (flag_error) stop(msg, call. = F)

  if (verbose) cat(glue::glue("The '{icesat_rgt_url}'  '{EXT_url}'  file includes  {LEN_url}  '{type_files}' files."), "\n")
  if (download_zip) info_url <- glue::glue("{url_pth}/{info_url}")
  info_url <- data.table::setDT(list(file = info_url))

  if (verbose) compute_elapsed_time(time_start = t_start)

  return(info_url)
}



#' Utilizing Virtual File Systems (vsi) and Well Known Text (WKT) to access the 'nominal orbits'
#'
#' @param orbit_area a character string specifying the earth partition to use, it can be one of 'antarctic', 'arctic', 'western_hemisphere' and 'eastern_hemisphere'
#' @param track a character string specifying the orbit track. Can be one of 'GT1L','GT1R','GT2L','GT2R','GT3L','GT3R' or 'GT7'
#' @param rgt_repeat an integer specifying the orbit repeat. This parameter defaults to 1 and it is relevant if a user chooses one of the 'western_hemisphere' or 'eastern_hemisphere' where there are 8 orbit repeats, whereas for the 'antarctic' and 'arctic' there is only 1 repeat
#' @param wkt_filter either NULL, or a Well Known Text (WKT) character string to allow a user to restrict to an area of interest rather than processing all data (this parameter will be used as input to the 'sf::st_read()' function)
#' @param download_zip a boolean. If TRUE the .zip file will be first downloaded and then the .kml files will be returned, otherwise the 'gdalinfo' function will be used as input to the R 'system2()' function to read the .kml files without downloading the .zip file. The 'gdalinfo' command requires that the user has configured GDAL properly. Set the parameter 'download_zip' to TRUE if GDAL is not (properly) installed.
#' @param download_method a character string specifying the download method. Corresponds to the 'method' parameter of the 'utils::download.file()' function. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return an 'sf' object
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' https://gdal.org/user/virtual_file_systems.html
#'
#' @importFrom glue glue
#' @importFrom sf st_layers st_read st_as_sf
#' @importFrom data.table rbindlist
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' require(IceSat2R)
#' require(magrittr)
#'
#' # ......................................
#' # processing all data of the orbit area
#' # ......................................
#'
#' dat_rgt <- vsi_nominal_orbits_wkt(
#'   orbit_area = "eastern_hemisphere",
#'   track = "GT7",
#'   rgt_repeat = 1,
#'   wkt_filter = NULL,
#'   download_method = "curl",
#'   download_zip = FALSE,
#'   verbose = TRUE
#' )
#' str(dat_rgt)
#'
#'
#' # ...........................................
#' # extracting nominal orbits only for the WKT
#' # ...........................................
#'
#' WKT <- "POLYGON ((-14.765 18.979, -11.25 18.979, -11.25 21.943, -14.765 21.943, -14.765 18.979))"
#'
#' dat_rgt <- vsi_nominal_orbits_wkt(
#'   orbit_area = "western_hemisphere",
#'   track = "GT3R",
#'   rgt_repeat = 8,
#'   wkt_filter = WKT,
#'   download_method = "curl",
#'   download_zip = FALSE,
#'   verbose = TRUE
#' )
#' str(dat_rgt)
#' dat_rgt[[1]]$RGT # Reference Ground Tracks of input WKT
#'
#' # .............................
#' # Visualize the results
#' # (first compute the centroid)
#' # .............................
#'
#' wkt_sf <- sf::st_as_sfc(WKT, crs = 4326)
#' centr_wkt <- sf::st_coordinates(sf::st_centroid(wkt_sf))
#'
#' RGTs <- mapview::mapview(dat_rgt, legend = F)
#' AOI_wkt <- mapview::mapview(wkt_sf, legend = F)
#'
#' lft <- RGTs + AOI_wkt
#' lft@map %>% leaflet::setView(
#'   lng = as.numeric(centr_wkt[, "X"]),
#'   lat = as.numeric(centr_wkt[, "Y"]),
#'   zoom = 7
#' )
#' }
vsi_nominal_orbits_wkt <- function(orbit_area,
                                   track = "GT7",
                                   rgt_repeat = 1,
                                   wkt_filter = NULL,
                                   download_method = "curl",
                                   download_zip = FALSE,
                                   verbose = FALSE) {
  if (verbose) t_start <- proc.time()
  nomin_orb <- available_nominal_orbits(orbit_area = orbit_area, verbose = verbose)
  lst <- list()

  for (url_pth in nomin_orb) {
    dat_out <- vsi_kml_from_zip(
      icesat_rgt_url = url_pth,
      download_zip = download_zip,
      download_method = download_method,
      verbose = verbose
    )
    lst[[url_pth]] <- dat_out
  }

  if (verbose) cat("Data based on repeat and track will be kept ...\n")
  dat_item <- lapply(lst, function(x) {
    idx <- which(gregexpr(pattern = as.character(glue::glue("repeat{rgt_repeat}_{track}")), text = x$file) != -1)
    if (length(idx) > 0) {
      x[idx, , drop = F]
    } else {
      NULL
    }
  })

  all_null <- all(as.vector(unlist(lapply(dat_item, function(x) is.null(x)))))
  if (all_null) stop("The specified parameter setting (track, rgt_repeat) returned only NULL objects! Be aware: for the 'western_hemisphere' or 'eastern_hemisphere' there is max. 8 repeats, whereas for the 'antarctic' and 'arctic' is only 1 repeat!", call. = F)

  dat_item <- data.table::rbindlist(dat_item)
  NROW <- nrow(dat_item)

  if (verbose) cat("Data based on repeat and track will be kept ...\n")
  sublists <- lapply(1:NROW, function(idx_row) {
    if (verbose) cat(glue::glue("The file '{basename(dat_item$file[idx_row])}' will be processed ..."), "\n")
    lrs <- sf::st_layers(dsn = dat_item$file[idx_row])

    NAMS <- lrs$name
    if (length(NAMS) == 0) stop(glue::glue("The '{dat_item$file[idx_row]}' does not include any layers!"), call. = F)
    LEN_nams <- length(NAMS)

    dat_all <- lapply(1:LEN_nams, function(x) {
      LAYER <- NAMS[x]
      wkt_inp <- character(0)
      if (!is.null(wkt_filter)) wkt_inp <- wkt_filter
      lr_dat <- sf::st_read(dsn = dat_item$file[idx_row], wkt_filter = wkt_inp, layer = LAYER, quiet = TRUE)
      lr_dat
    })

    dat_all <- data.table::rbindlist(dat_all)
    dat_all <- sf::st_as_sf(dat_all, crs = 4326)
    dat_all$orbit_area <- glue::glue("{rep(basename(dat_item$file[idx_row]), nrow(dat_all))}")
    dat_all <- dat_all[, c("Name", "orbit_area")]
    colnames(dat_all) <- c("RGT", "orbit_area", "geometry")
    dat_all
  })

  if (length(sublists) > 0) names(sublists) <- basename(dat_item$file)

  if (verbose) {
    cat("Total ")
    compute_elapsed_time(time_start = t_start)
  }

  return(sublists)
}



#' Utilizing Virtual File Systems (vsi) and Well Known Text (WKT) to access the 'time specific orbits'
#'
#' @param date_from a character string specifying the 'start' date in the format 'yyyy-MM-dd' (such as '2020-01-01')
#' @param date_to a character string specifying the 'end' date in the format 'yyyy-MM-dd' (such as '2020-01-01')
#' @param RGTs a character vector (consisting of one or more) Reference Ground Track (RGT). See the Examples section on how to come to these RGTs based on the "vsi_nominal_orbits_wkt()" function
#' @param wkt_filter either NULL, or a Well Known Text (WKT) character string to allow a user to restrict to an area of interest rather than processing all data. It is possible that the WKT won't intersect with any of the available time specific orbits due to the sparsity of the coordinates (the output in that case will be an empty list)
#' @param download_zip a boolean. If TRUE the .zip file will be first downloaded and then the .kml files will be returned, otherwise the 'gdalinfo' function will be used as input to the R 'system2()' function to read the .kml files without downloading the .zip file. The 'gdalinfo' command requires that the user has configured GDAL properly. Set the parameter 'download_zip' to TRUE if GDAL is not (properly) installed.
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return a list of 'sf' objects where each sublist will represent a different RGT cycle
#'
#' @details
#'
#' In case that this function does not return any results (empty list object) for a specified 'wkt_filter' parameter, then use a bigger Well Known Text (WKT) area. This is required because the 'time specific orbits' (points) are quite sparse.
#'
#' Moreover, set the parameter 'download_zip' to TRUE if the 'gdalinfo' function returns internally an empty character string. In that case also a warning will be shown in the R session.
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' https://gdal.org/user/virtual_file_systems.html
#'
#' @importFrom glue glue
#' @importFrom sf st_layers st_read st_as_sf st_zm
#' @importFrom data.table rbindlist data.table
#' @importFrom utils flush.console
#' @importFrom tools file_ext
#' @importFrom lubridate yday
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' require(IceSat2R)
#' require(magrittr)
#'
#' # ...........................................
#' # extracting nominal orbits only for the WKT
#' # ...........................................
#'
#' WKT <- "POLYGON ((-14.765 18.979, -11.25 18.979, -11.25 21.943, -14.765 21.943, -14.765 18.979))"
#'
#' dat_rgt <- vsi_nominal_orbits_wkt(
#'   orbit_area = "western_hemisphere",
#'   track = "GT3R",
#'   rgt_repeat = 8,
#'   wkt_filter = WKT,
#'   download_method = "curl",
#'   download_zip = FALSE,
#'   verbose = TRUE
#' )
#' str(dat_rgt)
#'
#' out_rgt <- dat_rgt[[1]]$RGT
#'
#' # .........................................
#' # time specific RGTs (for a time interval)
#' # request using a single RGT cycle
#' # .........................................
#'
#' date_start <- "2020-01-01"
#' date_end <- "2020-02-01"
#'
#' orb_cyc_single <- vsi_time_specific_orbits_wkt(
#'   date_from = date_start,
#'   date_to = date_end,
#'   RGTs = out_rgt,
#'   wkt_filter = WKT,
#'   verbose = TRUE
#' )
#' str(orb_cyc_single)
#'
#' # .........................................
#' # time specific RGTs (for a time interval)
#' # request using more than one RGT cycles
#' # .........................................
#'
#' date_start <- "2019-11-01"
#' date_end <- "2020-01-01"
#'
#' orb_cyc_multi <- vsi_time_specific_orbits_wkt(
#'   date_from = date_start,
#'   date_to = date_end,
#'   RGTs = out_rgt,
#'   wkt_filter = WKT,
#'   verbose = TRUE
#' )
#' str(orb_cyc_multi)
#' table(orb_cyc_multi$cycle)
#'
#'
#' # .......................................................
#' # visualization of the output cycles (including the WKT)
#' # .......................................................
#'
#' orb_cyc_multi$cycle <- as.factor(orb_cyc_multi$cycle)
#' mp_orb <- mapview::mapview(orb_cyc_multi, legend = TRUE, zcol = "cycle")
#'
#' sf_aoi <- sf::st_as_sfc(WKT, crs = 4326)
#' mp_aoi <- mapview::mapview(sf_aoi, alpha.regions = 0.3, legend = F)
#'
#' mp_orb + mp_aoi
#' }
vsi_time_specific_orbits_wkt <- function(date_from,
                                         date_to,
                                         RGTs,
                                         wkt_filter = NULL,
                                         download_zip = FALSE,
                                         verbose = FALSE) {
  if (verbose) t_start <- proc.time()

  RGTs <- as.vector(sapply(RGTs, function(x) { # make sure to add the required digits
    if (nchar(x) == 1) {
      x <- glue::glue("000{x}")
    } else if (nchar(x) == 2) {
      x <- glue::glue("00{x}")
    } else if (nchar(x) == 3) {
      x <- glue::glue("0{x}")
    } else {
      x
    }

    as.character(x)
  }))

  avail_rgts <- available_RGTs(only_cycle_names = FALSE, verbose = verbose)
  orbit_files <- avail_rgts$orbit_files
  orbit_dates <- avail_rgts$orbit_dates

  seq_dates <- seq(from = as.Date(date_from), to = as.Date(date_to), by = "1 day")
  inters_dates <- as.vector(unlist(lapply(orbit_dates, function(x) length(intersect(x = x, y = seq_dates)))))

  res_lst <- list()
  idx_inters <- which(inters_dates > 0)
  LEN_inters <- length(idx_inters)

  if (LEN_inters > 0) {
    cycles_subs <- names(orbit_files)[idx_inters]
    zip_subs <- as.vector(unlist(orbit_files))[idx_inters]

    if (verbose) {
      cat(glue::glue("In total there are {LEN_inters} intersected dates for which data will be processed!"), "\n")
      cat(glue::glue("The RGT cycles from which data will be processed are: {paste(cycles_subs, collapse = ', ')}"), "\n")
    }

    for (item_cy in seq_along(zip_subs)) {
      iter_cycle <- cycles_subs[item_cy]
      if (verbose) {
        cat("-------------------------------------------------\n")
        cat(glue::glue("RGTs of cycle '{iter_cycle}' will be processed ..."), "\n")
        cat("-------------------------------------------------\n")
      }

      iter_url <- zip_subs[item_cy]
      zip_dat <- vsi_kml_from_zip(
        icesat_rgt_url = iter_url,
        download_zip = download_zip,
        download_method = "curl",
        verbose = verbose
      )
      if (nrow(zip_dat) > 0) {
        greg_expr <- as.character(glue::glue("IS2_RGT_{RGTs}"))
        rows_inters <- as.vector(unlist(sapply(greg_expr, function(x) which(gregexpr(pattern = x, text = zip_dat$file) != -1))))
        if (length(rows_inters)) {
          zip_dat_subs <- zip_dat[rows_inters, , drop = F]
          NROW <- nrow(zip_dat_subs)

          sublists <- lapply(1:NROW, function(idx_row) {
            if (verbose) {
              message(glue::glue("{iter_cycle} FILE: "), idx_row, "/", NROW, "\r", appendLF = FALSE)
              utils::flush.console()
            }

            lrs <- sf::st_layers(dsn = zip_dat_subs$file[idx_row])

            NAMS <- lrs$name
            if (length(NAMS) == 0) stop(glue::glue("The '{zip_dat_subs$file[idx_row]}' does not include any layers!"), call. = F)
            LEN_nams <- length(NAMS)

            dat_all <- lapply(1:LEN_nams, function(x) {
              LAYER <- NAMS[x]
              wkt_inp <- character(0)
              if (!is.null(wkt_filter)) wkt_inp <- wkt_filter
              lr_dat <- sf::st_read(dsn = zip_dat_subs$file[idx_row], wkt_filter = wkt_inp, layer = LAYER, quiet = TRUE)
              # lr_dat = sf::st_read(dsn = zip_dat_subs$file[idx_row], layer = LAYER, quiet = TRUE)                           # keep this line for debugging in case the 'wkt_filter' does not work
              lr_dat
            })

            n_rows <- as.vector(unlist(lapply(dat_all, nrow)))
            idx_relev <- which(n_rows > 0)

            if (length(idx_relev) > 0) {
              dat_all <- dat_all[idx_relev]

              # I expect each sublist to be of type "sfc_POINT" and normally observations which are not (and can be for instance "sfc_LINESTRING") won't have a description column either. Therefore use the next line for removal
              descr_not_idx <- which(as.vector(unlist(lapply(dat_all, function(x) {
                colnams <- colnames(x)
                if ("description" %in% colnams) {
                  verify_descr <- (is.na(x$description) | x$description == "")
                } else if ("Description" %in% colnams) {
                  verify_descr <- (is.na(x$Description) | x$Description == "") # In Macintosh (osx) the "description" column appears with an upper case "D" as "Description", see the following issue: https://github.com/mlampros/IceSat2R/issues/9#issuecomment-1152020607
                } else {
                  stop("The 'D(d)escription' column must exist in every sublist!", call. = F)
                }
                verify_descr
              }))))

              LEN_exc <- length(descr_not_idx)
              if (LEN_exc > 0) {
                if (verbose) message(glue::glue("{LEN_exc} output sublist did not have a valid 'description' and will be removed!"))
                dat_all <- dat_all[-descr_not_idx]
              }

              dat_all <- data.table::rbindlist(dat_all)

              if (nrow(dat_all) > 0) {
                dat_all
              } else {
                NULL
              }
            }
          })

          nams_init <- basename(zip_dat_subs$file) # 'nams_init' will be modified if there are NULL sublists
          idx_null <- which(as.vector(unlist(lapply(sublists, is.null))))
          if (length(idx_null) > 0) {
            if (verbose) message(glue::glue("{length(idx_null)} out of {length(sublists)} sublists were empty and will be removed!"))
            sublists <- sublists[-idx_null]
            nams_init <- nams_init[-idx_null]
          }

          if (length(sublists) > 0) {
            names(sublists) <- nams_init
            res_lst[[iter_cycle]] <- sublists
          }
        }
      }
    }
  }

  LEN_lst <- length(res_lst)

  if (LEN_lst > 0) {
    nams_out <- names(res_lst)
    if (verbose) cat(glue::glue("In total {LEN_lst} RGT cycles will be included in the output 'sf' object ({paste(nams_out, collapse = ', ')})!"), "\n")

    res_lst <- lapply(1:LEN_lst, function(y) {
      x <- res_lst[[y]]

      nams_lst <- names(x)
      unq_ext <- unique(tools::file_ext(nams_lst))
      if (length(unq_ext) > 1) stop("I expect either a '.kml' or '.kmz' file extension!", call. = F)

      if (verbose) cat(glue::glue("output of '{nams_out[y]}' will be re-formatted ..."), "\n")

      x <- data.table::rbindlist(x)
      x <- sf::st_as_sf(x, crs = 4326) # by default use 'EPSG:4326'

      colnams_sf <- colnames(x)
      flag_upper_descr <- FALSE

      if ("description" %in% colnams_sf) {
        descr_x <- x$description
      } else if ("Description" %in% colnams_sf) {
        descr_x <- x$Description
        flag_upper_descr <- TRUE
      } else {
        stop("The 'D(d)escription' column must exist in the data!", call. = F)
      }

      descr_x_invalid <- (any(descr_x == "") | any(is.na(descr_x)))

      if (descr_x_invalid) {
        dat_tim <- gsub(glue::glue(".{unq_ext}"), "", nams_lst) # remove the file extension from the file names
        dat_tim <- lapply(strsplit(dat_tim, "_"), function(k) {
          iter_date <- k[length(k)]
          iter_cycle <- k[length(k) - 1]
          iter_cycle <- as.integer(gsub("[a-zA-Z]", "", iter_cycle))
          list(date = iter_date, cycle = iter_cycle)
        })
        dat_tim <- data.table::rbindlist(dat_tim)
        # yday = strptime(x = dat_tim$date, format = "%d-%b-%Y")                                                    # this command might not work for specific system locale
        yday <- as.vector(unlist(lapply(strsplit(dat_tim$date, "-"), function(x) {
          iter_month <- switch_abb(x[2])
          as.character(glue::glue("{x[3]}-{iter_month}-{x[1]}"))
        })))

        dat_tim$date <- as.character(glue::glue("{dat_tim$date} 00:00:00")) # add time so that it matches the next "else" condition time-format

        if (flag_upper_descr) {
          x$Description <- NULL
        } else {
          x$description <- NULL
        }

        x$RGT <- as.integer(gsub("RGT ", "", x$Name))
        x$Date_time <- dat_tim$date # add the date
        x$day_of_year <- lubridate::yday(x = yday)
        x$cycle <- dat_tim$cycle
      } else {
        if (verbose) cat("The 'description' column of the output data will be processed ...\n")

        colnams_dtbl <- colnames(x)
        if ("description" %in% colnams_dtbl) { # see previous exception regarding "D(d)escription" (here I assume that only "description" or "Description" cases exist)
          x$description <- as.character(x$description)
          descr_proc <- strsplit(x = x$description, split = "\n")
        } else {
          x$Description <- as.character(x$Description)
          # ---- PREVIOUS CODE (commented out) ----
          # descr_proc <- strsplit(x = x$Description, split = " ") # in case that I have a 'Description' column, split by empty space
          # chk_row_items <- as.vector(unlist(lapply(descr_proc, function(x) (length(x) %% 2) == 0))) # check that I have an even number of columns (normally 8 but can be fewer too) then concatenate the columns by pairs of consecutive items
          # if (!all(chk_row_items)) stop("It seems that after splitting the observations by empty space the number of columns (per row) are not an even number (vsi function)!", call. = F)
          # descr_proc <- lapply(descr_proc, function(x) {
          #   seq_item <- seq(from = 1, to = length(x), by = 2)
          #   sapply(seq_item, function(y) paste(c(x[y], x[y + 1]), collapse = " "))
          # })
          # ---- END PREVIOUS CODE ----
          # Same newline detection as the time_specific_orbits path.
          non_empty_idx_x <- which(nchar(x$Description) > 0 & !is.na(x$Description))
          uses_newlines_x <- length(non_empty_idx_x) > 0 &&
            grepl("[\r\n]", x$Description[[non_empty_idx_x[[1]]]], perl = TRUE)
          if (uses_newlines_x) {
            descr_proc <- strsplit(x = x$Description, split = "[\r\n]+", perl = TRUE)
          } else {
            descr_proc <- strsplit(x = x$Description, split = " ") # in case that I have a 'Description' column, split by empty space
            chk_row_items <- as.vector(unlist(lapply(descr_proc, function(x) (length(x) %% 2) == 0))) # check that I have an even number of columns (normally 8 but can be fewer too) then concatenate the columns by pairs of consecutive items
            if (!all(chk_row_items)) stop("It seems that after splitting the observations by empty space the number of columns (per row) are not an even number (vsi function)!", call. = F)
            descr_proc <- lapply(descr_proc, function(x) {
              seq_item <- seq(from = 1, to = length(x), by = 2)
              sapply(seq_item, function(y) paste(c(x[y], x[y + 1]), collapse = " "))
            })
          }
        }

        descr_proc <- data.table::as.data.table(do.call(rbind, descr_proc))
        data.table::setnames(descr_proc, 1:ncol(descr_proc), c("RGT", "Date_time", "DOY", "cycle")[seq_len(ncol(descr_proc))])

        if (flag_upper_descr) {
          x$Description <- NULL
        } else {
          x$description <- NULL
        }

        x$RGT <- as.integer(gsub("RGT ", "", descr_proc[["RGT"]])) # keep the integer from the character string which corresponds to the 'RGT'
        x$Date_time <- descr_proc[["Date_time"]]
        x$day_of_year <- as.integer(gsub("DOY ", "", descr_proc[["DOY"]])) # keep the integer from the character string which corresponds to the 'DOY'
        x$cycle <- as.integer(gsub("Cycle ", "", descr_proc[["cycle"]])) # keep the integer from the character string which corresponds to the 'cycle'
      }

      x <- sf::st_zm(x = x, drop = T) # drop the Z dimension
      x
    })

    res_lst <- sf::st_as_sf(data.table::rbindlist(res_lst))
    # res_lst$Date_time = strptime(x = res_lst$Date_time, format = "%d-%b-%Y %H:%M:%S")          # convert to Date-Time ('POSIXlt') format  [ !! I have to do that outside the for-loop, otherwise I might receive an error ]

    vec_lst_cycl <- as.vector(unlist(lapply(strsplit(res_lst$Date_time, " "), function(x) {
      iter_date <- strsplit(x[1], "-")[[1]]
      iter_month <- switch_abb(iter_date[2])
      as.character(glue::glue("{iter_date[3]}-{iter_month}-{iter_date[1]} {x[2]}"))
    })))
    res_lst$Date_time <- as.POSIXlt(vec_lst_cycl)
  } else {
    if (verbose) message(glue::glue("The specified parameter setting returned an empty list for all {length(idx_inters)} processed cycles!"))
  }

  if (verbose) {
    cat("Total ")
    compute_elapsed_time(time_start = t_start)
  }

  return(res_lst)
}

testthat::test_that("the function 'revisit_time_RGTs()' gives an error if the 'RGT_cycle' parameter is invalid!", {
  testthat::expect_error(revisit_time_RGTs(RGT_cycle = "INVALID"))
})


testthat::test_that("the function 'time_specific_orbits()' gives an error if all 3 parameters ('date_from', 'date_to', 'RGT_cycle') are specified!", {
  testthat::expect_error(time_specific_orbits(
    date_from = "2020-01-01",
    date_to = "2020-01-15",
    RGT_cycle = "RGT_cycle_6"
  ))
})


testthat::test_that("the function 'time_specific_orbits()' gives an error in case that the 'RGT_cycle' parameter is NULL and also any of ('date_from', 'date_to') parameters are NULL too!", {
  testthat::expect_error(time_specific_orbits(
    date_from = NULL,
    date_to = "2020-01-15",
    RGT_cycle = NULL
  ))
})


testthat::test_that("the function 'time_specific_orbits()' gives an error if the 'date_from' is greater than the 'date_to' parameter!", {
  testthat::expect_error(time_specific_orbits(
    date_from = "2020-01-15",
    date_to = "2020-01-01",
    RGT_cycle = NULL
  ))
})


testthat::test_that("the function 'latest_orbits()' returns the expected output!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_if(
    isTRUE(tryCatch(httr::GET("https://icesat-2.gsfc.nasa.gov", httr::timeout(10))$status_code != 200, error = function(e) TRUE)),
    "icesat-2.gsfc.nasa.gov is not reachable"
  )

  df_orbs <- latest_orbits(technical_specs_url = "https://icesat-2.gsfc.nasa.gov/science/specs", verbose = FALSE)
  df_orbs_spl <- split(df_orbs, by = "Type")

  testthat::expect_true(ncol(df_orbs) == 5 & nrow(df_orbs) >= 16 & !any(is.na(c(df_orbs_spl$Time_Specific$Date_from, df_orbs_spl$Time_Specific$Date_to))))
})


testthat::test_that("the function 'available_nominal_orbits()' returns the expected output (orbit_area is NULL)!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  nomin_orb <- available_nominal_orbits(verbose = FALSE)

  testthat::expect_true(inherits(nomin_orb, "list") & length(nomin_orb) > 0)
})


testthat::test_that("the function 'available_nominal_orbits()' returns the expected output (orbit_area is 'arctic')!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  nomin_orb <- available_nominal_orbits(orbit_area = "arctic", verbose = FALSE)

  testthat::expect_true(inherits(nomin_orb, "list") & length(nomin_orb) == 1)
})


# #................................................................................................. it takes approximately 1 minute using 1 thread
testthat::test_that("the function 'overall_mission_orbits()' returns the expected output!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  if (Sys.info()["sysname"] == "Linux") { # skip for Windows, Mac OSX because it seems that the gdal version can not read .kmz files and throws an error

    testthat::skip_if(
      isTRUE(tryCatch(httr::GET("https://icesat-2.gsfc.nasa.gov", httr::timeout(10))$status_code != 200, error = function(e) TRUE)),
      "icesat-2.gsfc.nasa.gov is not reachable"
    )
    res_orb <- overall_mission_orbits(
      orbit_area = "eastern_hemisphere",
      download_method = "curl",
      threads = 1,
      verbose = FALSE
    )

    testthat::expect_true(inherits(res_orb, c("sf", "data.table", "data.frame")) & nrow(res_orb) > 1 & any(c("Description", "description") %in% colnames(res_orb)))
  }
})
# #.................................................................................................


testthat::test_that("the function 'available_RGTs()' returns the expected output (only_cycle_names is FALSE)!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  rgt_lst <- available_RGTs(only_cycle_names = FALSE, verbose = FALSE)

  testthat::expect_true(inherits(rgt_lst, "list") & length(rgt_lst) == 3 & all(names(rgt_lst) %in% c("orbit_files", "orbit_dates", "names_cycles")) & all(unlist(lapply(rgt_lst, function(x) length(x) > 0))))
})


testthat::test_that("the function 'available_RGTs()' returns the expected output (only_cycle_names is TRUE)!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  rgt_lst <- available_RGTs(only_cycle_names = TRUE, verbose = FALSE)

  testthat::expect_true(inherits(rgt_lst, "character") & length(rgt_lst) > 0)
})


testthat::test_that("the function 'revisit_time_RGTs()' returns the expected output (RGT_cycle is NULL and complete_date_sequence is TRUE)!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  rev_all <- revisit_time_RGTs(RGT_cycle = NULL, complete_date_sequence = TRUE)

  testthat::expect_true(inherits(rev_all, "list") & length(rev_all) == 3 & all(names(rev_all) %in% c("orbit_files", "orbit_dates", "length_dates")) & all(unlist(lapply(rev_all, function(x) length(x) >= 1))))
})


testthat::test_that("the function 'revisit_time_RGTs()' returns the expected output (RGT_cycle is 'RGT_cycle_1' and complete_date_sequence is FALSE)!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  rev_all <- revisit_time_RGTs(RGT_cycle = "RGT_cycle_1", complete_date_sequence = FALSE)

  testthat::expect_true(inherits(rev_all, "list") & length(rev_all) == 3 & all(names(rev_all) %in% c("orbit_files", "orbit_dates", "length_dates")) & all(unlist(lapply(rev_all, function(x) length(x) >= 1))))
})


# #................................................................................................. it takes approximately 2 minutes using 1 thread
testthat::test_that("the function 'time_specific_orbits()' returns the expected output!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_if(
    isTRUE(tryCatch(httr::GET("https://icesat-2.gsfc.nasa.gov", httr::timeout(10))$status_code != 200, error = function(e) TRUE)),
    "icesat-2.gsfc.nasa.gov is not reachable"
  )

  res_rgt_one <- time_specific_orbits(
    date_from = "2019-06-01",
    date_to = "2019-06-01",
    download_method = "curl",
    threads = 1,
    verbose = FALSE
  )

  testthat::expect_true(inherits(res_rgt_one, c("sf", "data.table", "data.frame")) & nrow(res_rgt_one) > 0 & all(c("Name", "RGT", "Date_time", "day_of_year", "cycle", "geometry") %in% colnames(res_rgt_one)) & !any(is.na(res_rgt_one$Date_time)))
})
# #.................................................................................................


testthat::test_that("the function 'vsi_kml_from_zip()' returns the expected output!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_on_ci() # skip on Github Actions which throws an error when using the '/vsi../' url  [ for the 'skip_on_ci()' see: https://github.com/r-lib/testthat/issues/970#issuecomment-611176719 AND https://github.com/r-lib/actions/issues/81 ]

  res_out <- vsi_kml_from_zip(
    icesat_rgt_url = ORBITS_URL,
    download_zip = FALSE, # Don't download the file which takes too long
    download_method = "curl",
    verbose = FALSE
  )

  testthat::expect_true(inherits(res_out, c("data.table", "data.frame")) & nrow(res_out) > 0 & ncol(res_out) == 1)
})


testthat::test_that("the function 'vsi_nominal_orbits_wkt()' returns the expected output!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_on_ci() # skip on Github Actions which throws an error when using the '/vsi../' url  [ for the 'skip_on_ci()' see: https://github.com/r-lib/testthat/issues/970#issuecomment-611176719 AND https://github.com/r-lib/actions/issues/81 ]

  dat_rgt <- vsi_nominal_orbits_wkt(
    orbit_area = "eastern_hemisphere",
    track = "GT7",
    rgt_repeat = 1,
    wkt_filter = NULL,
    download_method = "curl",
    download_zip = FALSE,
    verbose = FALSE
  )

  testthat::expect_true(length(dat_rgt) == 1 & inherits(dat_rgt[[1]], c("sf", "data.table", "data.frame")) & nrow(dat_rgt[[1]]) == 1478 & ncol(dat_rgt[[1]]) == 3)
})


testthat::test_that("the function 'vsi_nominal_orbits_wkt()' returns the expected output (extracting nominal orbits only for a WKT)!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_on_ci() # skip on Github Actions which throws an error when using the '/vsi../' url  [ for the 'skip_on_ci()' see: https://github.com/r-lib/testthat/issues/970#issuecomment-611176719 AND https://github.com/r-lib/actions/issues/81 ]

  dat_rgt <- vsi_nominal_orbits_wkt(
    orbit_area = "western_hemisphere",
    track = "GT3R",
    rgt_repeat = 8,
    wkt_filter = WKT,
    download_method = "curl",
    download_zip = FALSE,
    verbose = FALSE
  )

  testthat::expect_true(length(dat_rgt) == 1 & inherits(dat_rgt[[1]], c("sf", "data.table", "data.frame")) & nrow(dat_rgt[[1]]) == 30 & ncol(dat_rgt[[1]]) == 3)
})


testthat::test_that("the function 'vsi_time_specific_orbits_wkt()' returns the expected output for an input WKT!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_on_ci() # skip on Github Actions which throws an error when using the '/vsi../' url  [ for the 'skip_on_ci()' see: https://github.com/r-lib/testthat/issues/970#issuecomment-611176719 AND https://github.com/r-lib/actions/issues/81 ]

  orb_cyc_single <- vsi_time_specific_orbits_wkt(
    date_from = date_start,
    date_to = date_end,
    RGTs = sample_rgts,
    wkt_filter = WKT,
    verbose = FALSE
  )

  testthat::expect_true(inherits(orb_cyc_single, c("sf", "data.table", "data.frame")) & nrow(orb_cyc_single) == 2 & all(c("Name", "RGT", "Date_time", "day_of_year", "cycle", "geometry") %in% colnames(orb_cyc_single)))
})


testthat::test_that("the function 'time_specific_orbits()' returns the expected output for an input date range!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_if(
    isTRUE(tryCatch(httr::GET("https://icesat-2.gsfc.nasa.gov", httr::timeout(10))$status_code != 200, error = function(e) TRUE)),
    "icesat-2.gsfc.nasa.gov is not reachable"
  )

  approx_date_start <- "2021-02-01"
  approx_date_end <- "2021-02-15"

  res_rgt_many <- time_specific_orbits(
    date_from = approx_date_start,
    date_to = approx_date_end,
    RGT_cycle = NULL,
    download_method = "curl",
    threads = 1,
    verbose = FALSE
  )

  testthat::expect_true(inherits(res_rgt_many, c("sf", "data.table", "data.frame")) & nrow(res_rgt_many) > 0 & all(c("Name", "RGT", "Date_time", "day_of_year", "cycle", "geometry") %in% colnames(res_rgt_many)))
})

# ..............................................................
# reference: https://github.com/trafficonese/leaflet.extras
#
# "leaflet.extras" was archived at 2026-02-19 and is no
#  longer available on CRAN, thus I had to include the
#  required code snippets (I used previously) in this package
# ..............................................................

#' Options for drawn shapes
#' @param stroke	Whether to draw stroke along the path. Set it to false to disable borders on polygons or circles.
#' @param color	Stroke color.
#' @param weight	Stroke width in pixels.
#' @param opacity	Stroke opacity.
#' @param fill	Whether to fill the path with color. Set it to false to disable filling on polygons or circles.
#' @param fillColor	same as color	Fill color.
#' @param fillOpacity	Fill opacity.
#' @param dashArray	A string that defines the stroke dash pattern. Doesn't work on canvas-powered layers (e.g. Android 2).
#' @param lineCap	A string that defines shape to be used at the end of the stroke.
#' @param lineJoin	A string that defines shape to be used at the corners of the stroke.
#' @param clickable	If false, the vector will not emit mouse events and will act as a part of the underlying map.
#' @param pointerEvents	Sets the pointer-events attribute on the path if SVG backend is used.
#' @param smoothFactor	How much to simplify the polyline on each zoom level. More means better performance and smoother look, and less means more accurate representation.
#' @param noClip	Disabled polyline clipping.
#' @importFrom leaflet filterNULL
#' @noRd
drawShapeOptions <- function(
    stroke = TRUE,
    color = "#03f",
    weight = 1,
    opacity = 1,
    fill = TRUE,
    fillColor = "#03f",
    fillOpacity = 0.4,
    dashArray = NULL,
    lineCap = NULL,
    lineJoin = NULL,
    clickable = TRUE,
    pointerEvents = NULL,
    smoothFactor = 1.0,
    noClip = TRUE) {
  leaflet::filterNULL(list(
    stroke = stroke,
    color = color,
    weight = weight,
    opacity = opacity,
    fill = fill,
    fillColor = fillColor,
    fillOpacity = fillOpacity,
    dashArray = dashArray,
    lineCap = lineCap,
    lineJoin = lineJoin,
    clickable = clickable,
    pointerEvents = pointerEvents,
    smoothFactor = smoothFactor,
    noClip = noClip
  ))
}

#' Options for drawing polylines
#' @param allowIntersection	Determines if line segments can cross.
#' @param drawError	Configuration options for the error that displays if an intersection is detected.
#' @param guidelineDistance	Distance in pixels between each guide dash.
#' @param maxGuideLineLength Maximum length of the guide lines.
#' @param showLength Whether to display the distance in the tooltip.
#' @param metric	Determines which measurement system (metric or imperial) is used.
#' @param feet When not metric, use feet instead of yards for display.
#' @param nautic When not metric, not feet, use nautic mile for display.
#' @param zIndexOffset	This should be a high number to ensure that you can draw over all other layers on the map.
#' @param shapeOptions	Leaflet Polyline options
#' @param repeatMode	Determines if the draw tool remains enabled after drawing a shape.
#' @importFrom leaflet filterNULL
#' @noRd
drawPolylineOptions <- function(
    allowIntersection = TRUE,
    drawError = list(color = "#b00b00", timeout = 2500),
    guidelineDistance = 20,
    maxGuideLineLength = 4000,
    showLength = TRUE,
    metric = TRUE,
    feet = TRUE,
    nautic = FALSE,
    zIndexOffset = 2000,
    shapeOptions = drawShapeOptions(fill = FALSE),
    repeatMode = FALSE) {
  leaflet::filterNULL(list(
    allowIntersection = allowIntersection,
    drawError = drawError,
    guidelineDistance = guidelineDistance,
    maxGuideLineLength = maxGuideLineLength,
    showLength = showLength,
    metric = metric,
    feet = feet,
    nautic = nautic,
    zIndexOffset = zIndexOffset,
    shapeOptions = shapeOptions,
    repeatMode = repeatMode
  ))
}

#' Options for drawing polygons
#' @param showArea Show the area of the drawn polygon in m??, ha or km??. The area is only approximate and become less accurate the larger the polygon is.
#' @importFrom leaflet filterNULL
#' @noRd
drawPolygonOptions <- function(
    showArea = FALSE,
    metric = TRUE,
    shapeOptions = drawShapeOptions(),
    repeatMode = FALSE) {
  leaflet::filterNULL(list(
    showArea = showArea,
    metric = metric,
    shapeOptions = shapeOptions,
    repeatMode = repeatMode
  ))
}

#' Options for drawing rectangles
#' @importFrom leaflet filterNULL
#' @noRd
drawRectangleOptions <- function(
    showArea = TRUE,
    metric = TRUE,
    shapeOptions = drawShapeOptions(),
    repeatMode = FALSE) {
  leaflet::filterNULL(list(
    showArea = showArea,
    metric = metric,
    shapeOptions = shapeOptions,
    repeatMode = repeatMode
  ))
}

#' Options for drawing Circles
#' @param showRadius Show the radius of the drawn circle in m, km, ft (feet), or nm (nautical mile).
#' @importFrom leaflet filterNULL
#' @noRd
drawCircleOptions <- function(
    showRadius = TRUE,
    metric = TRUE,
    feet = TRUE,
    nautic = FALSE,
    shapeOptions = drawShapeOptions(),
    repeatMode = FALSE) {
  leaflet::filterNULL(list(
    shapeOptions = shapeOptions,
    repeatMode = repeatMode,
    showRadius = showRadius,
    metric = metric,
    feet = feet,
    nautic = nautic
  ))
}

#' Options for drawing markers
#' @importFrom leaflet filterNULL
#' @noRd
drawMarkerOptions <- function(
    markerIcon = NULL,
    zIndexOffset = 2000,
    repeatMode = FALSE) {
  leaflet::filterNULL(list(
    markerIcon = markerIcon,
    zIndexOffset = zIndexOffset,
    repeatMode = repeatMode
  ))
}

#' Options for drawing markers
#' @importFrom leaflet filterNULL
#' @noRd
drawCircleMarkerOptions <- function(
    stroke = TRUE,
    color = "#3388ff",
    weight = 4,
    opacity = 0.5,
    fill = TRUE,
    fillColor = NULL, # same as color by default
    fillOpacity = 0.2,
    clickable = TRUE,
    zIndexOffset = 2000,
    repeatMode = FALSE) {
  leaflet::filterNULL(list(
    stroke = stroke,
    color = color,
    weight = weight,
    opacity = opacity,
    fill = fill,
    fillColor = fillColor, # same as color by default
    fillOpacity = fillOpacity,
    clickable = clickable,
    zIndexOffset = zIndexOffset,
    repeatMode = repeatMode
  ))
}

#' Options for path when in editMode
#' @param maintainColor Whether to maintain shape's original color
#' @importFrom leaflet filterNULL
#' @noRd
selectedPathOptions <- function(
    dashArray = c("10, 10"),
    weight = 2,
    color = "black",
    fill = TRUE,
    fillColor = "black",
    fillOpacity = 0.6,
    maintainColor = FALSE) {
  leaflet::filterNULL(list(
    dashArray = dashArray,
    weight = weight,
    color = color,
    fill = fill,
    fillColor = fillColor,
    fillOpacity = fillOpacity,
    maintainColor = maintainColor
  ))
}

#' Options for editing shapes
#' @param edit Editing enabled by default. Set to false do disable editing.
#' @param selectedPathOptions To customize shapes in editing mode
#' @param remove Set to false to disable removing.
#' @importFrom leaflet filterNULL
#' @noRd
editToolbarOptions <- function(
    edit = TRUE,
    remove = TRUE,
    selectedPathOptions = NULL,
    allowIntersection = TRUE) {
  leaflet::filterNULL(list(
    edit = edit,
    remove = remove,
    selectedPathOptions = selectedPathOptions,
    allowIntersection = allowIntersection
  ))
}


#' Options for editing handlers
#' @description Customize tooltips
#' @param polyline List of options for polyline tooltips.
#' @param polygon List of options for polygon tooltips.
#' @param rectangle List of options for rectangle tooltips.
#' @param circle List of options for circle tooltips.
#' @param marker List of options for marker tooltips.
#' @param circlemarker List of options for circlemarker tooltips.
#' @param simpleshape List of options for simpleshape tooltips.
#' @noRd
handlersOptions <- function(
    polyline = list(
      error = "<strong>Error:</strong> shape edges cannot cross!",
      tooltipStart = "Click to start drawing line.",
      tooltipCont = "Click to start drawing line.",
      tooltipEnd = "Click to start drawing line."
    ),
    polygon = list(
      tooltipStart = "Click to start drawing shape.",
      tooltipCont = "Click to start drawing shape.",
      tooltipEnd = "Click to start drawing shape."
    ),
    rectangle = list(
      tooltipStart = "Click and drag to draw rectangle."
    ),
    circle = list(
      tooltipStart = "Click map to place circle marker.",
      radius = "Radius"
    ),
    marker = list(
      tooltipStart = "Click map to place marker."
    ),
    circlemarker = list(
      tooltipStart = "Click and drag to draw circle."
    ),
    simpleshape = list(
      tooltipEnd = "Release mouse to finish drawing."
    )) {
  leaflet::filterNULL(list(
    polyline = list(
      error = polyline$error,
      tooltip = list(
        start = polyline$tooltipStart,
        cont = polyline$tooltipCont,
        end = polyline$tooltipEnd
      )
    ),
    polygon = list(
      tooltip = list(
        start = polygon$tooltipStart,
        cont = polygon$tooltipCont,
        end = polygon$tooltipEnd
      )
    ),
    rectangle = list(tooltip = list(start = rectangle$tooltipStart)),
    circle = list(
      radius = circle$radius,
      tooltip = list(start = circle$tooltipStart)
    ),
    marker = list(tooltip = list(start = marker$tooltipStart)),
    circlemarker = list(tooltip = list(start = circlemarker$tooltipStart)),
    simpleshape = list(tooltip = list(end = simpleshape$tooltipEnd))
  ))
}


#' Options for editing the toolbar
#' @description Customize the toolbar
#' @param actions List of options for actions toolbar button.
#' @param finish List of options for finish toolbar button.
#' @param undo List of options for undo toolbar button.
#' @param buttons List of options for buttons toolbar button.
#' @importFrom leaflet filterNULL
#' @noRd
toolbarOptions <- function(
    actions = list(
      title = "Cancel drawing",
      text = "Cancel"
    ),
    finish = list(
      title = "Finish drawing",
      text = "Finish"
    ),
    undo = list(
      title = "Delete last point drawn",
      text = "Delete last point"
    ),
    buttons = list(
      polyline = "Draw a polyline",
      polygon = "Draw a polygon",
      rectangle = "Draw a rectangle",
      circle = "Draw a circle",
      marker = "Draw a marker",
      circlemarker = "Draw a circlemarker"
    )) {
  leaflet::filterNULL(list(
    actions = list(
      title = actions$title,
      text = actions$text
    ),
    finish = list(
      title = finish$title,
      text = finish$text
    ),
    undo = list(
      title = undo$title,
      text = undo$text
    ),
    buttons = list(
      polyline = buttons$polyline,
      polygon = buttons$polygon,
      rectangle = buttons$rectangle,
      circle = buttons$circle,
      marker = buttons$marker,
      circlemarker = buttons$circlemarker
    )
  ))
}


#' Options for editing edit handlers
#' @description Customize edit handlers
#' @param edit List of options for editing tooltips.
#' @param remove List of options for removing tooltips.
#' @importFrom leaflet filterNULL
#' @noRd
edithandlersOptions <- function(
    edit = list(
      tooltipText = "Drag handles or markers to edit features.",
      tooltipSubtext = "Click cancel to undo changes."
    ),
    remove = list(
      tooltipText = "Click on a feature to remove."
    )) {
  leaflet::filterNULL(list(
    edit = list(
      tooltip = list(
        text = edit$tooltipText,
        subtext = edit$tooltipSubtext
      )
    ),
    remove = list(
      tooltip = list(
        text = remove$tooltipText
      )
    )
  ))
}


#' Options for editing the toolbar
#' @description Customize the edit toolbar
#' @param actions List of options for edit action tooltips.
#' @param buttons List of options for edit button tooltips.
#' @importFrom leaflet filterNULL
#' @noRd
edittoolbarOptions <- function(
    actions = list(
      save = list(
        title = "Save changes",
        text = "Save"
      ),
      cancel = list(
        title = "Cancel editing, discards all changes",
        text = "Cancel"
      ),
      clearAll = list(
        title = "Clear all layers",
        text = "Clear All"
      )
    ),
    buttons = list(
      edit = "Edit layers",
      editDisabled = "No layers to edit",
      remove = "Delete layers",
      removeDisabled = "No layers to delete"
    )) {
  leaflet::filterNULL(list(
    actions = actions,
    buttons = buttons
  ))
}

# ..............................................................
# reference: https://github.com/trafficonese/leaflet.extras
#
# "leaflet.extras" was archived at 2026-02-19 and is no
#  longer available on CRAN, thus I had to include the
#  required code snippets (I used previously) in this package
# ..............................................................

drawDependencies <- function(drag = TRUE) {
  if (drag) {
    list(
      html_dep_prod("lfx-draw", "1.0.4", has_style = TRUE, has_binding = TRUE),
      html_dep_prod("lfx-draw-drag", "0.4.8")
    )
  } else {
    list(
      html_dep_prod("lfx-draw", "1.0.4", has_style = TRUE, has_binding = TRUE)
    )
  }
}

#' Adds a Toolbar to draw shapes/points on the map.
#' @param map The map widget.
#' @param targetLayerId An optional layerId of a GeoJSON/TopoJSON layer whose features need to be editable.
#'  Used for adding  a GeoJSON/TopoJSON layer and then editing the features using the draw plugin.
#' @param targetGroup An optional group name of a Feature Group whose features need to be editable.
#'  Used for adding shapes(markers, lines, polygons) and then editing them using the draw plugin.
#'  You can either set layerId or group or none but not both.
#' @param position The position where the toolbar should appear.
#' @param polylineOptions Set to FALSE to disable polyline drawing.
#' @param polygonOptions Set to FALSE to disable polygon drawing.
#' @param circleOptions Set to FALSE to disable circle drawing.
#' @param rectangleOptions Set to FALSE to disable rectangle drawing.
#' @param markerOptions Set to FALSE to disable marker drawing.
#' @param circleMarkerOptions Set to FALSE to disable circle marker drawing.
#' @param editOptions By default editing is disable.
#' @param singleFeature When set to TRUE, only one feature can be drawn at a time, the previous ones being removed.
#' @param toolbar Set to \code{NULL} to take Leaflets default values.
#' @param handlers Set to \code{NULL} to take Leaflets default values.
#' @param edittoolbar Set to \code{NULL} to take Leaflets default values.
#' @param edithandlers Set to \code{NULL} to take Leaflets default values.
#' @param drag When set to \code{TRUE}, the drawn features will be draggable during editing, utilizing
#'    the \code{Leaflet.Draw.Drag} plugin. Otherwise, this library will not be included.
#' @importFrom leaflet invokeMethod getMapData filterNULL
#' @noRd
addDrawToolbar <- function(
    map, targetLayerId = NULL, targetGroup = NULL,
    position = c("topleft", "topright", "bottomleft", "bottomright"),
    polylineOptions = drawPolylineOptions(),
    polygonOptions = drawPolygonOptions(),
    circleOptions = drawCircleOptions(),
    rectangleOptions = drawRectangleOptions(),
    markerOptions = drawMarkerOptions(),
    circleMarkerOptions = drawCircleMarkerOptions(),
    editOptions = FALSE,
    singleFeature = FALSE,
    toolbar = NULL,
    handlers = NULL,
    edittoolbar = NULL,
    edithandlers = NULL,
    drag = TRUE) {
  if (!is.null(targetGroup) && !is.null(targetLayerId)) {
    stop("To edit existing features either specify a targetGroup or a targetLayerId, but not both")
  }

  if (!inherits(toolbar, "list")) toolbar <- NULL
  if (!inherits(handlers, "list")) handlers <- NULL
  if (!inherits(edittoolbar, "list")) edittoolbar <- NULL
  if (!inherits(edithandlers, "list")) edithandlers <- NULL

  map$dependencies <- c(map$dependencies, drawDependencies(drag))

  markerIconFunction <- NULL
  if (inherits(markerOptions, "list") && !is.null(markerOptions$markerIcon)) {
    if (inherits(markerOptions$markerIcon, "leaflet_icon")) {
      markerIconFunction <- defIconFunction
    } else if (inherits(markerOptions$markerIcon, "leaflet_awesome_icon")) {
      map <- addAwesomeMarkersDependencies(
        map, markerOptions$markerIcon$library
      )
      markerIconFunction <- awesomeIconFunction
    } else {
      stop("markerIcon should be created using either leaflet::makeIcon() or leaflet::makeAwesomeIcon()")
    }
    markerOptions$markerIconFunction <- markerIconFunction
  }

  position <- match.arg(position)

  options <- list(
    position = position,
    draw = leaflet::filterNULL(list(
      polyline = polylineOptions,
      polygon = polygonOptions,
      circle = circleOptions,
      rectangle = rectangleOptions,
      marker = markerOptions,
      circlemarker = circleMarkerOptions,
      singleFeature = singleFeature
    )),
    edit = editOptions,
    toolbar = toolbar,
    handlers = handlers,
    edittoolbar = edittoolbar,
    edithandlers = edithandlers
  )

  leaflet::invokeMethod(
    map, leaflet::getMapData(map), "addDrawToolbar",
    targetLayerId, targetGroup, options
  )
}

#' Removes the draw toolbar
#' @param clearFeatures whether to clear the map of drawn features.
#' @importFrom leaflet invokeMethod getMapData
#' @noRd
removeDrawToolbar <- function(map, clearFeatures = FALSE) {
  leaflet::invokeMethod(map, leaflet::getMapData(map), "removeDrawToolbar", clearFeatures)
}

utils::globalVariables(c('..re_order_cols'))                        # used in the "verify_RGTs()" function


#' Customized function to download files
#' 
#' @param url a character string specifying a valid url
#' @param destfile a character string specifying a valid path where the output file will be saved
#' @param download_method a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function
#' @param verbose a boolean. If TRUE then information will appear in the console
#' 
#' @importFrom withr with_options
#' @importFrom utils download.file
#' 
#' @export
#' 
#' @references 
#' 
#' https://github.com/mlverse/torchdatasets/blob/master/R/utils.R#L20
#' 
#' @examples
#' 
#' \dontrun{
#'
#' require(IceSat2R)
#' 
#' # the default timeout is 60 and we set it to 600
#' options(timeout = 600)
#' 
#' # we specify a URL and a temporary file
#' default_url = "https://icesat-2.gsfc.nasa.gov/sites/default/"
#' url_downl = glue::glue('{default_url}files/page_files/IS2RGTscycle12datetime.zip')
#' tmp_f = tempfile(fileext = '.zip')
#'
#' # then we download the file
#' downl_f = download_file(url = url_downl,
#'                         destfile = tmp_f, 
#'                         download_method = 'curl',
#'                         verbose = TRUE)
#' }

download_file = function(url, 
                         destfile, 
                         download_method,
                         verbose = FALSE) {
  
  withr::with_options(new = list(timeout = max(120, getOption("timeout"))), {
    
    utils::download.file(url = url,
                         destfile = destfile,
                         method = download_method, 
                         quiet = !verbose)
  })
  
  return(destfile)
}



#' Get the data based on the API URL
#'
#' @param URL a character string specifying the API command
#' @param outputFormat a character string specifying the output format of the downloaded data. One of 'csv', 'json' or 'zip'
#' @param download_method a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function
#' @param file_path_zip either NULL or a character string specifying a valid path to the output .zip file. This parameter will normally be a valid path if the 'outputFormat' parameter is set to 'zip'. If it's NULL and the 'outputFormat' parameter is 'zip' then the downloaded '.zip' file will be converted and returned as a data.table object
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return if the 'file_path_zip' parameter is NULL it returns either a data.table (if outputFormat is 'csv') or a nested list (if outputFormat is 'json') else the file path where the .zip file is saved. In case that the 'outputFormat' is set to 'zip' and the 'file_path_zip' parameter to NULL then a data.table will be returned.
#'
#' @importFrom glue glue
#' @importFrom data.table fread
#' @importFrom httr content GET http_error
#' @importFrom utils download.file
#' @importFrom tools file_ext
#'
#' @keywords internal

get_URL_data = function(URL,
                        outputFormat = 'csv',
                        download_method = 'curl',
                        file_path_zip = NULL,
                        verbose = FALSE) {

  if (verbose) t_start = proc.time()

  # Error code mapping for informative messages (per CRAN graceful-failure policy)
  openaltimetry_error_codes = list(
    '400' = 'Bad request',
    '401' = 'Unauthorized',
    '404' = 'Not found',
    '500' = 'Internal error'
  )

  # Wrap the HTTP request in tryCatch to handle network/DNS/timeout errors gracefully
  query_response = tryCatch(
    httr::GET(url = URL),
    error = function(e) {
      message(glue::glue("The OpenAltimetry API request failed: {conditionMessage(e)} (URL: {URL})"))
      return(NULL)
    }
  )

  # If a network error occurred, return NULL (no error thrown)
  if (is.null(query_response)) {
    return(NULL)
  }

  # Check HTTP status using httr::http_error() - fail gracefully per CRAN policy
  if (httr::http_error(query_response)) {
    status_code <- query_response$status_code
    url_error_code <- openaltimetry_error_codes[[as.character(status_code)]]
    if (is.null(url_error_code)) {
      msg <- glue::glue("The OpenAltimetry API request failed: HTTP {status_code} (URL: {URL})")
    } else {
      msg <- glue::glue("The OpenAltimetry API request failed: HTTP {status_code} - {url_error_code} (URL: {URL})")
    }
    message(msg)
    return(NULL)
  }

  obj_out_flag = TRUE

  if (outputFormat == 'csv') {
    data_out = data.table::fread(URL, stringsAsFactors = F, header = T, verbose = F, showProgress = verbose)
  }
  else if (outputFormat == 'json') {
    data_out = httr::content(x = query_response, as = "parsed")
  }
  else if (outputFormat == 'zip') {
    if (is.null(file_path_zip)) {
      message("The 'file_path_zip' parameter is NULL therefore the downloaded .zip file will be converted to a data.table object!")
      file_path_zip = tempfile(fileext = '.zip')
    }
    else {
      obj_out_flag = FALSE
      ext_zip = tools::file_ext(file_path_zip)
      if (ext_zip != 'zip') stop("The extension of the 'file_path_zip' parameter must be '.zip'!", call. = F)
    }

    downl_u = download_file(url = URL,
                            destfile = file_path_zip,
                            download_method = download_method,
                            verbose = verbose)
    
    if (obj_out_flag) {
      data_out = data.table::fread(cmd = glue::glue('unzip -p {file_path_zip} *.csv'), stringsAsFactors = F, header = T, verbose = F, showProgress = verbose)
      if (file.exists(file_path_zip)) file.remove(file_path_zip)
    }
  }
  else {
    stop("Available output formats are 'csv', 'json' or 'zip'!", call. = F)
  }

  if (verbose) {
    cat('\n')
    compute_elapsed_time(time_start = t_start)
  }

  if (obj_out_flag) {
    return(data_out)
  }
  else {
    return(file_path_zip)
  }
}


#' Convert an abbreviated month name to a numeric value
#'
#' @param month a character string specifying one of the twelve months, i.e. ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")
#' @return a character string which will correspond to the numeric value of the input month, i.e. ("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12")
#'
#' @keywords internal

switch_abb = function(month) {

  month = tolower(month)

  switch(month,
         jan = {mnth = '01'},
         feb = {mnth = '02'},
         mar = {mnth = '03'},
         apr = {mnth = '04'},
         may = {mnth = '05'},
         jun = {mnth = '06'},
         jul = {mnth = '07'},
         aug = {mnth = '08'},
         sep = {mnth = '09'},
         oct = {mnth = '10'},
         nov = {mnth = '11'},
         dec = {mnth = '12'},
  )
  return(mnth)
}


#' Convert a full month name to a numeric value
#'
#' @param month a character string specifying one of the twelve months, i.e. ("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December")
#' @return a character string which will correspond to the numeric value of the input month, i.e. ("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12")
#'
#' @keywords internal

switch_full = function(month) {

  month = tolower(month)

  switch(month,
         january = {mnth = '01'},
         february = {mnth = '02'},
         march = {mnth = '03'},
         april = {mnth = '04'},
         may = {mnth = '05'},
         june = {mnth = '06'},
         july = {mnth = '07'},
         august = {mnth = '08'},
         september = {mnth = '09'},
         october = {mnth = '10'},
         november = {mnth = '11'},
         december = {mnth = '12'},
  )
  return(mnth)
}


#' Verification of the Reference Ground Tracks (RGTs)
#'
#' This function allows the user to verify the NSIDC extracted RGTs with the corresponding OpenAltimetry using the same Dates
#'
#' @param nsidc_rgts a data.frame, data.table or tibble object that includes the columns 'Date_time' and 'RGT'
#' @param bbx_aoi a named numeric vector or an sf-bbox object with names 'xmin', 'ymin', 'xmax', 'ymax'
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#' @param ... further parameters for the getTracks function
#'
#' @return a 'data.table' object where it is possible that the number of the OpenAltimetry RGTs is higher compared to the NSIDC RGTs
#'
#' @importFrom utils flush.console
#' @importFrom data.table setDT rbindlist
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' rgts = data.table::setDT(list(RGT = c(1251L, 1252L, 1260L, 1267L, 1275L),
#'                               Date_time = c("2020-12-15", "2020-12-15",
#'                               "2020-12-15", "2020-12-16", "2020-12-16")))
#' bbx = c(xmin = -53.108876, ymin = 60.119614, xmax = -19.203521, ymax = 80.793117)
#'
#' dtbl = verify_RGTs(nsidc_rgts = rgts, bbx_aoi = bbx, verbose = TRUE)
#' dtbl
#'
#' # split by Date to observe RGTs by date
#'
#' split(dtbl, by = 'Date_time')
#'
#' }

verify_RGTs = function(nsidc_rgts, bbx_aoi, verbose = FALSE, ...) {

  if (verbose) t_start = proc.time()
  if (!all(names(bbx_aoi) %in% c("xmin", "ymin", "xmax", "ymax"))) stop("The input parameter 'bbx_aoi' must be a named numeric vector or an sf-bbox object with names 'xmin', 'ymin', 'xmax', 'ymax'!", call. = F)

  tracks_dates_RGT = list()
  NROW = nrow(nsidc_rgts)

  for (item in 1:NROW) {

    if (verbose) {
      message("Row: ", item, "/", NROW, "\r", appendLF = FALSE)
      utils::flush.console()
    }

    dat_item = nsidc_rgts[item, , drop = F]
    Date = as.Date(dat_item$Date_time)
    char_date = as.character(Date)

    op_tra = getTracks(minx = as.numeric(bbx_aoi['xmin']),
                       miny = as.numeric(bbx_aoi['ymin']),
                       maxx = as.numeric(bbx_aoi['xmax']),
                       maxy = as.numeric(bbx_aoi['ymax']),
                       date = char_date,
                       verbose = FALSE, ...)

    if (nrow(op_tra) > 0) {
      colnames(op_tra) = 'RGT_OpenAlt'
      op_tra$Date_time = char_date
      op_tra$RGT_NSIDC = dat_item$RGT
    }
    else {
      op_tra = data.table::setDT(list(RGT_NSIDC = dat_item$RGT,
                                      RGT_OpenAlt = NA_integer_,
                                      Date_time = char_date))
    }

    tracks_dates_RGT[[char_date]] = op_tra
  }

  tracks_dates_RGT = data.table::rbindlist(tracks_dates_RGT, use.names = TRUE)
  re_order_cols = c('Date_time', 'RGT_OpenAlt', 'RGT_NSIDC')
  tracks_dates_RGT = tracks_dates_RGT[, ..re_order_cols]
  if (verbose) compute_elapsed_time(time_start = t_start)

  return(tracks_dates_RGT)
}

testthat::test_that("the function 'get_URL_data()' returns NULL for an invalid URL (graceful failure)", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_if_offline()

  res <- get_URL_data(
    URL = INVALID_URL,
    outputFormat = "csv",
    download_method = "curl",
    file_path_zip = NULL,
    verbose = FALSE
  )
  testthat::expect_null(res)
})


testthat::test_that("the function 'get_URL_data()' returns a data.frame if the input url is valid!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  expected_COLS <- c(
    "segment_id_beg", "segment_id_end", "longitude", "latitude",
    "h_te_best_fit", "h_te_uncertainty", "h_canopy", "h_canopy_uncertainty",
    "track_id", "beam", "file_name"
  )

  res_df <- get_URL_data(
    URL = VALID_URL_remaining,
    outputFormat = "csv",
    download_method = "curl",
    file_path_zip = NULL,
    verbose = FALSE
  )

  cols_df <- colnames(res_df)

  testthat::expect_true(all(cols_df %in% expected_COLS) & ncol(res_df) == length(cols_df))
})


testthat::test_that("the function 'verify_RGTs()' gives an error if the input bounding box is invalid!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  testthat::expect_error(verify_RGTs(nsidc_rgts = rgts, bbx_aoi = c(
    xmin = -53.108876,
    ymin = 60.119614,
    xmax = -19.203521,
    INVALID = 80.793117
  ), verbose = TRUE))
})


testthat::test_that("the function 'verify_RGTs()' returns a data.table if the input parameters are valid!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  dtbl <- tryCatch(
    verify_RGTs(nsidc_rgts = rgts, bbx_aoi = bbx, verbose = FALSE),
    error = function(e) NULL
  )

  testthat::skip_if(
    is.null(dtbl) || nrow(dtbl) == 0 || nrow(dtbl) < 5 ||
      ncol(dtbl) != 3 || !all(c("Date_time", "RGT_OpenAlt", "RGT_NSIDC") %in% colnames(dtbl)),
    "API returned no data, an error, or unexpected structure; skipping value check"
  )
  testthat::expect_true(nrow(dtbl) >= 5 & ncol(dtbl) == 3 & all(colnames(dtbl) %in% c("Date_time", "RGT_OpenAlt", "RGT_NSIDC")))
})

#' Get the ICESAT-2 Tracks
#'
#' Get a list of ICESat-2 tracks using a bounding box as input
#'
#' @param minx the 'minx' parameter of the bounding box
#' @param maxx the 'maxx' parameter of the bounding box
#' @param miny the 'miny' parameter of the bounding box
#' @param maxy the 'maxy' parameter of the bounding box
#' @param date a character string specifying the Data collection date in the format 'yyyy-MM-dd' (such as '2020-01-01')
#' @param outputFormat a character string specifying the output format of the downloaded data. One of 'csv' or 'json'
#' @param download_method a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return either a data.table (if outputFormat is 'csv') or a nested list (if outputFormat is 'json')
#'
#' @references
#'
#' https://openaltimetry.earthdatacloud.nasa.gov
#'
#' https://nsidc.org/data/icesat-2
#'
#' @importFrom glue glue
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#'
#' require(IceSat2R)
#' require(magrittr)
#' require(sf)
#'
#' sf::sf_use_s2(use_s2 = FALSE)
#'
#' #..........................................................
#' # In case we are interested to find the ICESat-2 tracks for
#' #     - a specific area (or bounding box)
#' #     - a specific time interval (close to September 2021)
#' #..........................................................
#'
#' #...............
#' # Ice sheet area
#' #...............
#'
#' data(ne_10m_glaciated_areas)
#'
#' ice_sheet = 'Kluane Ice Cap'
#'
#' ice_sheet_geom = ne_10m_glaciated_areas %>%
#'   subset(!is.na(name)) %>%
#'   subset(name == ice_sheet)
#'
#' ice_sheet_geom
#'
#' #..............
#' # time interval
#' #..............
#'
#' approx_date_start = "2021-09-05"
#' approx_date_end = "2021-09-20"
#'
#' res_rgt_many = time_specific_orbits(date_from = approx_date_start,
#'                                     date_to = approx_date_end,
#'                                     RGT_cycle = NULL,
#'                                     download_method = 'curl',
#'                                     threads = parallel::detectCores(),
#'                                     verbose = TRUE)
#'
#' res_rgt_many
#'
#' # table(as.Date(res_rgt_many$Date_time))
#'
#' #..............................................
#' # create the bounding box of the ice
#' # sheet so that I use the same area in the
#' # 'st_intersects()' and 'getTracks()' functions
#' # ('getTracks()' takes a bounding box as input)
#' #..............................................
#'
#' bbx_ice_sheet = sf::st_bbox(obj = ice_sheet_geom)
#' sf_obj_ice_sheet = sf::st_as_sfc(bbx_ice_sheet)
#'
#' #.........................................
#' # intersection of the Ice Sheet with
#' # the tracks for the specified time period
#' #.........................................
#'
#' res_inters = sf::st_intersects(x = sf::st_geometry(sf_obj_ice_sheet),
#'                                y = sf::st_geometry(res_rgt_many),
#'                                sparse = TRUE)
#'
#' #...........................................
#' # one or more intersected RGT's for the area
#' #...........................................
#'
#' df_inters = data.frame(res_inters)
#' rgt_subs = res_rgt_many[df_inters$col.id, , drop = FALSE]
#'
#'
#' #.......................................................
#' # Keep the Date and the bounding box to
#' # verify with the 'getTracks()' function
#' # an alternative is to use the "verify_RGTs()" function
#' #.......................................................
#'
#' for (item in 1:nrow(rgt_subs)) {
#'
#'   dat_item = rgt_subs[item, , drop = F]
#'   Date = as.Date(dat_item$Date_time)
#'
#'   op_tra = getTracks(minx = as.numeric(bbx_ice_sheet['xmin']),
#'                                miny = as.numeric(bbx_ice_sheet['ymin']),
#'                                maxx = as.numeric(bbx_ice_sheet['xmax']),
#'                                maxy = as.numeric(bbx_ice_sheet['ymax']),
#'                                date = as.character(Date),
#'                                outputFormat = 'csv',
#'                                download_method = 'curl',
#'                                verbose = FALSE)
#'
#'   date_obj = dat_item$Date_time
#'   tim_rgt = glue::glue("Date: {date_obj} Time specific RGT: '{dat_item$RGT}'")
#'
#'   if (nrow(op_tra) > 0) {
#'     iter_op_trac = paste(op_tra$track, collapse = ', ')
#'     cat(glue::glue("{tim_rgt}  OpenAltimetry: '{iter_op_trac}'"), '\n')
#'   }
#'   else {
#'     cat(glue::glue("{tim_rgt} without an OpenAltimetry match!"), '\n')
#'   }
#' }
#'
#' }

getTracks = function(minx,
                     miny,
                     maxx,
                     maxy,
                     date,
                     outputFormat = 'csv',
                     download_method = 'curl',
                     verbose = FALSE) {

  if (!outputFormat %in% c('csv', 'json')) stop("The 'outputFormat' parameter must be one of 'csv' or 'json'!", call. = F)
  URL = glue::glue("https://openaltimetry.earthdatacloud.nasa.gov/data/api/icesat2/getTracks?date={date}&minx={minx}&miny={miny}&maxx={maxx}&maxy={maxy}&outputFormat={outputFormat}")

  res_out = get_URL_data(URL = URL,
                         outputFormat = outputFormat,
                         download_method = download_method,
                         file_path_zip = NULL,                    # there isn't a '.zip' option for the 'ICESAT-2 Tracks'
                         verbose = verbose)
  return(res_out)
}

testthat::test_that("the function 'getTracks()' gives an error if the 'outputFormat' parameter is not one of 'csv' or 'json'!", {
  testthat::expect_error(getTracks(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    date = "2020-01-01",
    outputFormat = "zip",
    download_method = "curl",
    verbose = FALSE
  ))
})


testthat::test_that("the function 'getTracks()' returns the correct output!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  res_df <- tryCatch(
    getTracks(
      minx = minx,
      miny = miny,
      maxx = maxx,
      maxy = maxy,
      date = "2021-02-15",
      outputFormat = "csv",
      download_method = "curl",
      verbose = FALSE
    ),
    error = function(e) NULL
  )

  testthat::skip_if(
    is.null(res_df) || !is.data.frame(res_df) || nrow(res_df) == 0 ||
      !"track" %in% colnames(res_df),
    "API returned no data, an error, or unexpected structure; skipping value check"
  )
  testthat::expect_true(is.data.frame(res_df) & "track" %in% colnames(res_df))
})

#' Get ICESat-2 ATLAS data for a specific Date
#'
#' This function allows the user to download ICESat-2 ATLAS Product data for a specific date, bounding box, track and beam.
#'
#' @param minx the 'minx' parameter of the bounding box
#' @param maxx the 'maxx' parameter of the bounding box
#' @param miny the 'miny' parameter of the bounding box
#' @param maxy the 'maxy' parameter of the bounding box
#' @param date a character string specifying the Data collection date in the format 'yyyy-MM-dd' (such as '2020-01-01')
#' @param trackId an integer specifying the 'Reference ground track ID' (see the examples section on how to come to the 'trackId' and to a bounding box of 1 or 5 degrees globally)
#' @param beamName either NULL (if the 'product' parameter is not equal to 'atl03' which means data for all 6 beams will be returned) or a character string specifying the Beam Name, which can be one of the 'gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l' or 'gt3r'
#' @param product a character string specifying the input data product (default value is 'atl03'). It can be one of 'atl03', 'atl06', 'atl07', 'atl08', 'atl10', 'atl12' or 'atl13'
#' @param client a character string specifying the 'Referring client'. Can be one of 'portal' or 'jupyter' (default is 'portal')
#' @param photonConfidence either NULL or a character vector specifying the 'Confidence levels of the photons (can be one or more). If NULL (default) all photon data will be returned. The available options are 'na', 'noise', 'buffer', 'low', 'medium', 'high'
#' @param sampling a boolean. If TRUE a sampling rate of 1/1000 will be used, otherwise all data will be returned (default is FALSE)
#' @param outputFormat a character string specifying the output format of the downloaded data. One of 'csv', 'json' or 'zip'
#' @param file_path_zip either NULL or a character string specifying a valid path to the output .zip file. This parameter will normally be a valid path if the 'outputFormat' parameter is set to 'zip'. If it's NULL and the 'outputFormat' parameter is 'zip' then the downloaded '.zip' file will be converted and returned as a data.table object
#' @param download_method a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return if the 'file_path_zip' parameter is NULL it returns either a data.table (if outputFormat is 'csv') or a nested list (if outputFormat is 'json') else the file path where the .zip file is saved. In case that the 'outputFormat' is set to 'zip' and the 'file_path_zip' parameter to NULL then a data.table will be returned.
#'
#' @details
#'
#' \describe{
#'  \item{'atl03', \emph{Global Geolocated Photon Data} (Version 4, Requests are limited to 1x1 degree spatial bounding box selection \emph{unless "sampling" is set to true})}{This data set \emph{(ATL03)} contains height above the WGS 84 ellipsoid (ITRF2014 reference frame), latitude, longitude, and time for all photons downlinked by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory. The ATL03 product was designed to be a single source for all photon data and ancillary information needed by higher-level ATLAS/ICESat-2 products. As such, it also includes spacecraft and instrument parameters and ancillary data not explicitly required for ATL03}
#'  \item{'atl06', \emph{Land Ice Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL06)} provides geolocated, land-ice surface heights (above the WGS 84 ellipsoid, ITRF2014 reference frame), plus ancillary parameters that can be used to interpret and assess the quality of the height estimates. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl07', \emph{Sea Ice Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{The data set \emph{(ATL07)} contains along-track heights for sea ice and open water leads (at varying length scales) relative to the WGS84 ellipsoid (ITRF2014 reference frame) after adjustment for geoidal and tidal variations, and inverted barometer effects. Height statistics and apparent reflectance are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl08', \emph{Land and Vegetation Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL08)} contains along-track heights above the WGS84 ellipsoid (ITRF2014 reference frame) for the ground and canopy surfaces. The canopy and ground surfaces are processed in fixed 100 m data segments, which typically contain more than 100 signal photons. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl10', \emph{Sea Ice Freeboard} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL10)} contains estimates of sea ice freeboard, calculated using three different approaches. Sea ice leads used to establish the reference sea surface and descriptive statistics used in the height estimates are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl12', \emph{Ocean Surface Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL12)} contains along-track sea surface heights at variable length scales over cloud-free regions. Estimates of height distributions, surface roughness, surface slope, and apparent reflectance are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl13', \emph{Inland Water Surface Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL13)} contains along-track water surface heights and descriptive statistics for inland water bodies. Water bodies include lakes, reservoirs, bays, and estuaries. Descriptive statistics include along-track surface slope (where data permit), mean and standard deviation, subsurface signal (532 nm) attenuation, wave height, and coarse depth to bottom topography}
#'  }
#'
#' @references
#'
#' https://openaltimetry.earthdatacloud.nasa.gov
#'
#' https://nsidc.org/data/icesat-2
#'
#' @importFrom glue glue
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' #............................................................
#' # observe the available countries and continents using the
#' # 'rnaturalearth' R package to perform a query using map-edit
#' #............................................................
#'
#' cntr_cnt = rnaturalearth::ne_countries(scale = 110,
#'                                        type = 'countries',
#'                                        returnclass = 'sf')
#'
#' cntr_cnt = cntr_cnt[, c('sovereignt', 'continent')]
#'
#' # sort(cntr_cnt$sovereignt)
#' # sort(unique(cntr_cnt$continent))
#'
#'
#' #............................
#' # Select a 'continent' as AOI   (5-degree query)
#' #............................
#'
#' init = select_aoi_global_grid$new(area_of_interest = 'Oceania',
#'                                   verbose = TRUE)
#'
#' init$draw_edit_aoi(degrees = 5.0, square_geoms = TRUE)
#' sf_obj = init$selected_areas_global_grid(plot_data = FALSE)
#' sf_obj
#'
#' #.....................................................
#' # I drew a bounding box close to 'Mount Hagen Papua'
#' # inside one of the 5-degreee cells (after zooming-in)
#' #.....................................................
#'
#' # mapview::mapview(sf_obj, legend = F)
#' # sf::st_as_text(sf::st_geometry(sf_obj))
#'
#' #................................................
#' # to reproduce the results without selecting
#' # an area based on the 'draw_edit_aoi()' function
#' #................................................
#'
#' # plg = "POLYGON ((140 -6.641235, 145 -6.641235, 145 -1.641235, 140 -1.641235, 140 -6.641235))"
#' # sf_obj = sf::st_as_sfc(plg, crs = 4326)
#'
#'
#' #....................................................
#' # first we find available ICESat-2 track-ID's
#' # and Dates (time interval) from a specific RGT cycle
#' #....................................................
#'
#' approx_date_start = "2021-02-01"
#' approx_date_end = "2021-02-15"
#' # 'RGT_cycle_10'
#'
#' res_rgt_many = time_specific_orbits(date_from = approx_date_start,
#'                                     date_to = approx_date_end,
#'                                     RGT_cycle = NULL,
#'                                     download_method = 'curl',
#'                                     threads = parallel::detectCores(),
#'                                     verbose = TRUE)
#' res_rgt_many
#'
#' #.........................................................
#' # then we create the bounding box of the selected area
#' # and proceed to the intersection with the computed RGT's
#' # ( include the bounding box for reproducibility )
#' #.........................................................
#'
#' bbx_aoi = sf::st_bbox(obj = sf_obj)
#' # c(xmin = 140, ymin = -6.641235, xmax = 145, ymax = -1.641235)
#'
#' sf_obj_bbx = sf::st_as_sfc(bbx_aoi)
#'
#' res_inters = sf::st_intersects(x = sf_obj_bbx,
#'                                y = sf::st_geometry(res_rgt_many),
#'                                sparse = TRUE)
#' #.....................
#' # matched (RGT) tracks
#' #.....................
#'
#' df_inters = data.frame(res_inters)
#'
#' if (nrow(df_inters) == 0) {
#'   stop("There is no intersection between the specified AOI and the RGTs!")
#' }
#'
#'
#' rgt_subs = res_rgt_many[df_inters$col.id, , drop = FALSE]
#' rgt_subs
#'
#'
#' #..........................................
#' # find out which of the time specific RGT's
#' # match the OpenAltimetry Track-ID's
#' #..........................................
#'
#' dtbl_rgts = verify_RGTs(nsidc_rgts = rgt_subs,
#'                         bbx_aoi = bbx_aoi,
#'                         verbose = TRUE)
#'
#' #...................................
#' # we will iterate over the available:
#' #    - Dates
#' #    - trackId's
#' #    - product's
#' # to gather the up to 5-degree data
#' #..................................
#'
#' dates_iters = unique(dtbl_rgts$Date_time)
#' RGTs_iters = unique(dtbl_rgts$RGT_NSIDC)
#' prods_5_degrs = c('atl06', 'atl07', 'atl08', 'atl10', 'atl12', 'atl13')
#'
#' dat_out = logs_out = list()
#'
#' for (idx in seq_along(dates_iters)) {
#'
#'   date_i = dates_iters[idx]
#'   rgt_i = RGTs_iters[idx]
#'
#'   for (prod_i in prods_5_degrs) {
#'
#'     name_iter = glue::glue("{date_i}_{rgt_i}_{prod_i}")
#'     cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'  Product: '{prod_i}'"), '\n')
#'
#'     iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
#'                               miny = as.numeric(bbx_aoi['ymin']),
#'                               maxx = as.numeric(bbx_aoi['xmax']),
#'                               maxy = as.numeric(bbx_aoi['ymax']),
#'                               date = date_i,
#'                               trackId = rgt_i,
#'                               product = prod_i,
#'                               client = 'portal',
#'                               outputFormat = 'csv',
#'                               verbose = FALSE)
#'
#'     iter_logs = list(Date = date_i,
#'                      RGT = rgt_i,
#'                      Product = prod_i,
#'                      N_rows = nrow(iter_dat))
#'
#'     logs_out[[name_iter]] = data.table::setDT(iter_logs)
#'     dat_out[[name_iter]] = iter_dat
#'   }
#' }
#'
#' #.........................................
#' # each sublist corresponds to a different
#' # parameter setting (Date, Track, Product)
#' #.........................................
#'
#' dat_out
#'
#' #.....
#' # Logs  (including the number of rows for each parameter setting)
#' #.....
#'
#' dtbl_logs = data.table::rbindlist(logs_out)
#' dtbl_logs = subset(dtbl_logs, N_rows > 0)
#' dtbl_logs = dtbl_logs[order(dtbl_logs$N_rows, decreasing = T), ]
#' dtbl_logs
#'
#' #.............................................
#' # The Products 'atl08' and 'atl13' have data
#' # for the Dates and RGT's of the selected area
#' #.............................................
#'
#' unique(dtbl_logs$Product)
#' # c("atl06" "atl08" "atl13" "atl12")
#'
#' #................
#' # RGT's with data
#' #................
#'
#' unique(dtbl_logs$RGT)
#' # c(627, 756, 688, 619, 817)
#'
#' #................
#' # Dates with Data
#' #................
#'
#' unique(dtbl_logs$Date)
#' # c("2021-02-03", "2021-02-11", "2021-02-07", "2021-02-02", "2021-02-15")
#'
#'
#' #============
#' # ATL03 Data
#' #============
#'
#' ................................................
#' # the default timeout is 60 and we set it to 600
#' #...............................................
#' 
#' options(timeout = 600)
#' print(getOption('timeout'))
#' 
#' 
#' # we skip the interactive selection of the 1 degree grid
#' plg = "POLYGON ((142 -34.64124, 143 -34.64124, 143 -33.64124, 142 -33.64124, 142 -34.64124))"
#' sf_obj_atl03 = sf::st_as_sfc(plg, crs = 4326)
#' 
#' # we update the bounding box based on the AOI
#' bbx_aoi = sf::st_bbox(obj = sf_obj_atl03)
#' # c(xmin = 142, ymin = -34.64124, xmax = 143, ymax = -33.64124)
#' 
#' sf_obj_bbx = sf::st_as_sfc(bbx_aoi)
#' 
#' res_inters = sf::st_intersects(x = sf_obj_bbx,
#'                                y = sf::st_geometry(res_rgt_many),
#'                                sparse = TRUE)
#' 
#' df_inters = data.frame(res_inters)
#' 
#' if (nrow(df_inters) == 0) {
#'   stop("There is no intersection between the specified AOI and the RGTs!")
#' }
#' 
#' 
#' rgt_subs = res_rgt_many[df_inters$col.id, , drop = FALSE]
#' rgt_subs
#' 
#' dtbl_rgts = verify_RGTs(nsidc_rgts = rgt_subs,
#'                         bbx_aoi = bbx_aoi,
#'                         verbose = TRUE)
#' 
#' comp_cas = complete.cases(dtbl_rgts)
#' dtbl_rgts = dtbl_rgts[comp_cas, , drop = F]
#' 
#' 
#' dates_iters = unique(dtbl_rgts$Date_time)
#' RGTs_iters = unique(dtbl_rgts$RGT_NSIDC)
#' prods_1_degrs = c('atl03')
#' 
#' #...........................
#' # use all beams with 'atl03'
#' #...........................
#' 
#' dat_out = logs_out = list()
#' 
#' for (idx in seq_along(dates_iters)) {
#'   
#'   date_i = dates_iters[idx]
#'   rgt_i = RGTs_iters[idx]
#'   
#'   name_iter = glue::glue("{date_i}_{rgt_i}")
#'   cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'"), '\n')
#'   
#'   iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
#'                             miny = as.numeric(bbx_aoi['ymin']),
#'                             maxx = as.numeric(bbx_aoi['xmax']),
#'                             maxy = as.numeric(bbx_aoi['ymax']),
#'                             date = date_i,
#'                             beamName = NULL,
#'                             trackId = rgt_i,
#'                             product = prods_1_degrs,
#'                             client = 'portal',
#'                             outputFormat = 'csv',
#'                             verbose = FALSE)
#'   
#'   iter_logs = list(Date = date_i,
#'                    RGT = rgt_i,
#'                    Product = prods_1_degrs,
#'                    N_rows = nrow(iter_dat))
#'   
#'   logs_out[[name_iter]] = data.table::setDT(iter_logs)
#'   dat_out[[name_iter]] = iter_dat
#' }
#' 
#' dat_out
#' unique(dtbl_logs$Product)
#' unique(dat_out[[1]]$beam)
#' unique(dat_out[[1]]$`confidence code`)
#' 
#' #..................................................
#' # use 2 beams with 'atl03' (a vector of characters) 
#' #..................................................
#' 
#' dat_out = logs_out = list()
#' 
#' for (idx in seq_along(dates_iters)) {
#'   
#'   date_i = dates_iters[idx]
#'   rgt_i = RGTs_iters[idx]
#'   
#'   name_iter = glue::glue("{date_i}_{rgt_i}")
#'   cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'"), '\n')
#'   
#'   iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
#'                             miny = as.numeric(bbx_aoi['ymin']),
#'                             maxx = as.numeric(bbx_aoi['xmax']),
#'                             maxy = as.numeric(bbx_aoi['ymax']),
#'                             date = date_i,
#'                             beamName = c("gt1l", "gt1r"),
#'                             trackId = rgt_i,
#'                             product = prods_1_degrs,
#'                             client = 'portal',
#'                             outputFormat = 'csv',
#'                             verbose = FALSE)
#'   
#'   iter_logs = list(Date = date_i,
#'                    RGT = rgt_i,
#'                    Product = prods_1_degrs,
#'                    N_rows = nrow(iter_dat))
#'   
#'   logs_out[[name_iter]] = data.table::setDT(iter_logs)
#'   dat_out[[name_iter]] = iter_dat
#' }
#' 
#' dat_out
#' unique(dtbl_logs$Product)
#' unique(dat_out[[1]]$beam)
#' 
#' 
#' #.............................................
#' # use 1 beam with 'atl03' (a character string) 
#' #.............................................
#' 
#' dat_out = logs_out = list()
#' 
#' for (idx in seq_along(dates_iters)) {
#'   
#'   date_i = dates_iters[idx]
#'   rgt_i = RGTs_iters[idx]
#'   
#'   name_iter = glue::glue("{date_i}_{rgt_i}")
#'   cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'"), '\n')
#'   
#'   iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
#'                             miny = as.numeric(bbx_aoi['ymin']),
#'                             maxx = as.numeric(bbx_aoi['xmax']),
#'                             maxy = as.numeric(bbx_aoi['ymax']),
#'                             date = date_i,
#'                             beamName = "gt1l",
#'                             trackId = rgt_i,
#'                             product = prods_1_degrs,
#'                             client = 'portal',
#'                             outputFormat = 'csv',
#'                             verbose = FALSE)
#'   
#'   iter_logs = list(Date = date_i,
#'                    RGT = rgt_i,
#'                    Product = prods_1_degrs,
#'                    N_rows = nrow(iter_dat))
#'   
#'   logs_out[[name_iter]] = data.table::setDT(iter_logs)
#'   dat_out[[name_iter]] = iter_dat
#' }
#' 
#' dat_out
#' unique(dtbl_logs$Product)
#' unique(dat_out[[1]]$beam)
#' 
#' 
#' #..............................................
#' # use all beams with 'atl03' & photonConfidence
#' #..............................................
#' 
#' dat_out = logs_out = list()
#' 
#' for (idx in seq_along(dates_iters)) {
#'   
#'   date_i = dates_iters[idx]
#'   rgt_i = RGTs_iters[idx]
#'   
#'   name_iter = glue::glue("{date_i}_{rgt_i}")
#'   cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'"), '\n')
#'   
#'   iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
#'                             miny = as.numeric(bbx_aoi['ymin']),
#'                             maxx = as.numeric(bbx_aoi['xmax']),
#'                             maxy = as.numeric(bbx_aoi['ymax']),
#'                             date = date_i,
#'                             beamName = NULL,
#'                             photonConfidence = c('buffer', 'low'),
#'                             trackId = rgt_i,
#'                             product = prods_1_degrs,
#'                             client = 'portal',
#'                             outputFormat = 'csv',
#'                             verbose = FALSE)
#'   
#'   iter_logs = list(Date = date_i,
#'                    RGT = rgt_i,
#'                    Product = prods_1_degrs,
#'                    N_rows = nrow(iter_dat))
#'   
#'   logs_out[[name_iter]] = data.table::setDT(iter_logs)
#'   dat_out[[name_iter]] = iter_dat
#' }
#' 
#' dat_out
#' unique(dtbl_logs$Product)
#' unique(dat_out[[1]]$beam)
#' unique(dat_out[[1]]$`confidence code`)
#' 
#' }


get_atlas_data = function(minx,
                          miny,
                          maxx,
                          maxy,
                          date,
                          trackId,
                          beamName = NULL,
                          product = 'atl03',
                          client = 'portal',
                          photonConfidence = NULL,
                          sampling = FALSE,
                          outputFormat = 'csv',
                          file_path_zip = NULL,
                          download_method = 'curl',
                          verbose = FALSE) {

  if (!product %in% c('atl03', 'atl06', 'atl07', 'atl08', 'atl10', 'atl12', 'atl13')) stop("The 'product' parameter must be one of 'atl03', 'atl06', 'atl07', 'atl08', 'atl10', 'atl12' or 'atl13'!", call. = F)
  if (!client %in% c('portal', 'jupyter')) stop("The 'client' parameter must be one of 'portal' or 'jupyter'!", call. = F)
  if (!outputFormat %in% c('csv', 'json', 'zip')) stop("The 'outputFormat' parameter must be one of 'csv', 'json' or 'zip'!", call. = F)

  # The following is the default URL used in all products
  URL = glue::glue("https://openaltimetry.earthdatacloud.nasa.gov/data/api/icesat2/{product}?minx={minx}&miny={miny}&maxx={maxx}&maxy={maxy}&trackId={trackId}&outputFormat={outputFormat}&date={date}&client={client}")
  
  if (product == 'atl03') {
    
    URL = glue::glue("{URL}&sampling={tolower(sampling)}")
    
    # for 'atl03' we can also include the 'photonConfidence'
    if (!is.null(photonConfidence)) {
      
      make_url_ph = ''
      
      for (ph_i in photonConfidence) {                                                    # verify that the Beams are valid
        if (!ph_i %in% c('na', 'noise', 'buffer', 'low', 'medium', 'high')) {
          stop(glue::glue("The specified photonConfidence '{ph_i}' is not one of the available ones, i.e. 'na', 'noise', 'buffer', 'low', 'medium' or 'high'!"), call. = F)
        }
        
        make_url_ph = glue::glue("{make_url_ph}&photonConfidence={ph_i}")
      }
      
      URL = glue::glue("{URL}{make_url_ph}")
    }
  }
  
  # the 'beamName' applies to all 'products'
  if (!is.null(beamName)) {
    
    make_url = ''
    
    for (beam_i in beamName) {                                                    # verify that the Beams are valid
      if (!beam_i %in% c('gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l', 'gt3r')) {
        stop(glue::glue("The specified beamName '{beam_i}' is not one of the available ones, i.e. 'gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l' or 'gt3r'!"), call. = F)
      }
      
      make_url = glue::glue("{make_url}&beamNames={beam_i}")
    }
    
    URL = glue::glue("{URL}{make_url}")
  }

  res_out = get_URL_data(URL = URL,
                         outputFormat = outputFormat,
                         download_method = download_method,
                         file_path_zip = file_path_zip,
                         verbose = verbose)
  return(res_out)
}



#' Get IceSat-2 ATLAS 'Level-3A' data for a time interval (up to 1 year)
#'
#' This function allows the user to download IceSat-2 ATLAS 'Level-3A' data for a specific time interval, bounding box, track and beam.
#'
#' @param minx the 'minx' parameter of the bounding box
#' @param maxx the 'maxx' parameter of the bounding box
#' @param miny the 'miny' parameter of the bounding box
#' @param maxy the 'maxy' parameter of the bounding box
#' @param startDate a character string specifying the Data collection of the \emph{start} Date in the format 'yyyy-MM-dd' (such as '2020-01-01')
#' @param endDate a character string specifying the Data collection of the \emph{end} Date in the format 'yyyy-MM-dd' (such as '2020-01-01')
#' @param trackId an integer specifying the 'Reference ground track ID' (see the examples section on how to come to the 'trackId' and to a bounding box of 1 or 5 degrees globally)
#' @param beamName either NULL (data for all 6 beams will be returned) or a character vector specifying the Beam Name(s), It can be one or more of the 'gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l' or 'gt3r'
#' @param product a character string specifying the input data product (default value is 'atl08'). It can be one of 'atl06', 'atl07', 'atl08', 'atl10', 'atl12' or 'atl13'
#' @param client a character string specifying the 'Referring client'. Can be one of 'portal' or 'jupyter' (default is 'portal')
#' @param outputFormat a character string specifying the output format of the downloaded data. One of 'csv', 'json' or 'zip'
#' @param file_path_zip either NULL or a character string specifying a valid path to the output .zip file. This parameter will normally be a valid path if the 'outputFormat' parameter is set to 'zip'. If it's NULL and the 'outputFormat' parameter is 'zip' then the downloaded '.zip' file will be converted and returned as a data.table object
#' @param download_method a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return if the 'file_path_zip' parameter is NULL it returns either a data.table (if outputFormat is 'csv') or a nested list (if outputFormat is 'json') else the file path where the .zip file is saved. In case that the 'outputFormat' is set to 'zip' and the 'file_path_zip' parameter to NULL then a data.table will be returned.
#'
#' @details
#'
#' Up to 1 year worth of ICESat-2 \emph{Level-3A} product data can be downloaded. \emph{Note:} requests are limited to 5x5 degree spatial bounding box selection
#'
#' \describe{
#'  \item{'atl06', \emph{Land Ice Height} (Version 4)}{This data set \emph{(ATL06)} provides geolocated, land-ice surface heights (above the WGS 84 ellipsoid, ITRF2014 reference frame), plus ancillary parameters that can be used to interpret and assess the quality of the height estimates. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl07', \emph{Sea Ice Height} (Version 4)}{The data set \emph{(ATL07)} contains along-track heights for sea ice and open water leads (at varying length scales) relative to the WGS84 ellipsoid (ITRF2014 reference frame) after adjustment for geoidal and tidal variations, and inverted barometer effects. Height statistics and apparent reflectance are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl08', \emph{Land and Vegetation Height} (Version 4)}{This data set \emph{(ATL08)} contains along-track heights above the WGS84 ellipsoid (ITRF2014 reference frame) for the ground and canopy surfaces. The canopy and ground surfaces are processed in fixed 100 m data segments, which typically contain more than 100 signal photons. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl10', \emph{Sea Ice Freeboard} (Version 4)}{This data set \emph{(ATL10)} contains estimates of sea ice freeboard, calculated using three different approaches. Sea ice leads used to establish the reference sea surface and descriptive statistics used in the height estimates are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl12', \emph{Ocean Surface Height} (Version 4)}{This data set \emph{(ATL12)} contains along-track sea surface heights at variable length scales over cloud-free regions. Estimates of height distributions, surface roughness, surface slope, and apparent reflectance are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
#'  \item{'atl13', \emph{Inland Water Surface Height} (Version 4)}{This data set \emph{(ATL13)} contains along-track water surface heights and descriptive statistics for inland water bodies. Water bodies include lakes, reservoirs, bays, and estuaries. Descriptive statistics include along-track surface slope (where data permit), mean and standard deviation, subsurface signal (532 nm) attenuation, wave height, and coarse depth to bottom topography}
#'  }
#'
#' @references
#'
#' https://openaltimetry.earthdatacloud.nasa.gov
#'
#' https://nsidc.org/data/icesat-2
#'
#' @importFrom glue glue
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' #.......................................................
#' # parameter setting based on the output results from the
#' # 'examples' section of the 'get_atlas_data()' function
#' # so that we can iterate over the RGT's and Products for
#' # the specified time interval
#' #.......................................................
#'
#' bbx = c(xmin = 140, ymin = -6.641235, xmax = 145, ymax = -1.641235)
#' start_date = "2021-02-03"
#' end_date = "2021-02-15"
#' RGTs = c(627, 756, 688, 619, 817)
#' Products = c('atl08', 'atl13')
#'
#' #........................................................
#' # loop over the specified parameters and save the results
#' #........................................................
#'
#' dat_out = logs_out = list()
#'
#' for (prod_i in Products) {
#'   for (track_i in RGTs) {
#'
#'     name_iter = glue::glue("{track_i}_{prod_i}")
#'     cat(glue::glue("RGT: '{track_i}'  Product: '{prod_i}'"), '\n')
#'
#'     iter_dat = get_level3a_data(minx = as.numeric(bbx['xmin']),
#'                                 miny = as.numeric(bbx['ymin']),
#'                                 maxx = as.numeric(bbx['xmax']),
#'                                 maxy = as.numeric(bbx['ymax']),
#'                                 startDate = start_date,
#'                                 endDate = end_date,
#'                                 trackId = track_i,
#'                                 beamName = NULL,        # return data of all 6 beams
#'                                 product = prod_i,
#'                                 client = 'portal',
#'                                 outputFormat = 'csv',
#'                                 verbose = FALSE)
#'
#'     iter_logs = list(RGT = track_i,
#'                      Product = prod_i,
#'                      N_rows = nrow(iter_dat))
#'
#'     logs_out[[name_iter]] = data.table::setDT(iter_logs)
#'     dat_out[[name_iter]] = iter_dat
#'   }
#' }
#'
#'
#' #.........................................
#' # each sublist corresponds to a different
#' # parameter setting (Track, Product)
#' #.........................................
#'
#' dat_out
#'
#' #.....
#' # Logs  (including the number of rows for each parameter setting)
#' #.....
#'
#' dtbl_logs = data.table::rbindlist(logs_out)
#' dtbl_logs = subset(dtbl_logs, N_rows > 0)
#' dtbl_logs = dtbl_logs[order(dtbl_logs$N_rows, decreasing = T), ]
#' dtbl_logs
#'
#' }


get_level3a_data = function(minx,
                            miny,
                            maxx,
                            maxy,
                            startDate,
                            endDate,
                            trackId,
                            beamName = NULL,
                            product = 'atl08',
                            client = 'portal',
                            outputFormat = 'csv',
                            file_path_zip = NULL,
                            download_method = 'curl',
                            verbose = FALSE) {

  if (!product %in% c('atl06', 'atl07', 'atl08', 'atl10', 'atl12', 'atl13')) stop("The 'level3a product' parameter must be one of 'atl06', 'atl07', 'atl08', 'atl10', 'atl12' or 'atl13'!", call. = F)
  if (!client %in% c('portal', 'jupyter')) stop("The 'client' parameter must be one of 'portal' or 'jupyter'!", call. = F)
  if (!outputFormat %in% c('csv', 'json', 'zip')) stop("The 'outputFormat' parameter must be one of 'csv', 'json' or 'zip'!", call. = F)
  if (as.Date(startDate) > as.Date(endDate)) stop(glue::glue("The start Date '{startDate}' has to be equal or smaller to the end Date '{endDate}'!"), call. = F)
  
  URL = glue::glue("https://openaltimetry.earthdatacloud.nasa.gov/data/api/icesat2/level3a?product={product}&minx={minx}&miny={miny}&maxx={maxx}&maxy={maxy}&trackId={trackId}&outputFormat={outputFormat}&startDate={startDate}&endDate={endDate}&client={client}")
  
  if (!is.null(beamName)) {
    
    make_url = ''
    
    for (beam_i in beamName) {                                                    # verify that the Beams are valid
      if (!beam_i %in% c('gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l', 'gt3r')) {
        stop(glue::glue("The specified beamName '{beam_i}' is not one of the available ones, i.e. 'gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l' or 'gt3r'!"), call. = F)
      }
      
      make_url = glue::glue("{make_url}&beamNames={beam_i}")
    }

    URL = glue::glue("{URL}{make_url}")
  }

  res_out = get_URL_data(URL = URL,
                         outputFormat = outputFormat,
                         download_method = download_method,
                         file_path_zip = file_path_zip,
                         verbose = verbose)
  return(res_out)
}

# ............................
# "get_atlas_data()" function
# ............................


testthat::test_that("the function 'get_atlas_data()' works for 'atl03' with 'beamName' = NULL", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  res <- get_atlas_data(
    minx = -144.67439, miny = 59.22850, maxx = -137.88048, maxy = 61.69038,
    date = "2020-01-01", trackId = 1290, beamName = NULL, product = "atl03"
  )
  testthat::expect_true(is.null(res) || is.data.frame(res))
})


testthat::test_that("the function 'get_atlas_data()' gives an error if the 'beamName' parameter is invalid!", {
  testthat::expect_error(get_atlas_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    date = "2020-01-01",
    trackId = 1290,
    beamName = "INVALID",
    product = "atl06"
  ))
})


testthat::test_that("the function 'get_atlas_data()' gives an error if the 'product' parameter is not one of 'atl03', 'atl06', 'atl07', 'atl08', 'atl10', 'atl12', 'atl13'!", {
  testthat::expect_error(get_atlas_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    date = "2020-01-01",
    trackId = 1290,
    beamName = "gt1l",
    product = "INVALID"
  ))
})


testthat::test_that("the function 'get_atlas_data()' gives an error if the 'client' parameter is not one of 'portal', 'jupyter'!", {
  testthat::expect_error(get_atlas_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    date = "2020-01-01",
    trackId = 1290,
    beamName = "gt1l",
    product = "atl06",
    client = "INVALID"
  ))
})


testthat::test_that("the function 'get_atlas_data()' gives an error if the 'outputFormat' parameter is not one of 'csv', 'json', 'zip'!", {
  testthat::expect_error(get_atlas_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    date = "2020-01-01",
    trackId = 1290,
    beamName = "gt1l",
    product = "atl06",
    client = "portal",
    outputFormat = "INVALID"
  ))
})


testthat::test_that("the function 'get_atlas_data()' gives an error if the 'product' parameter is 'atl03' and the 'photonConfidence' is other than NULL, 'na', 'noise', 'buffer', 'low', 'medium' or 'high'!", {
  testthat::expect_error(get_atlas_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    date = "2020-01-01",
    trackId = 1290,
    beamName = "gt1l",
    product = "atl03",
    photonConfidence = "INVALID",
    client = "portal",
    outputFormat = "INVALID"
  ))
})


testthat::test_that("the function 'get_atlas_data()' returns the correct output for a specific input parameter setting!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail

  iter_dat <- get_atlas_data(
    minx = minx,
    miny = miny,
    maxx = maxx,
    maxy = maxy,
    date = "2021-02-15",
    trackId = 817,
    product = "atl13",
    client = "portal",
    outputFormat = "csv",
    verbose = FALSE
  )

  col_nams <- c(
    "segment_id_beg", "segment_id_end", "segment_lon", "segment_lat",
    "ht_ortho", "qf_iwp", "track_id", "beam", "file_name"
  )

  testthat::skip_if(nrow(iter_dat) == 0, "API returned no data for the test date/region; skipping value check")
  testthat::expect_true(nrow(iter_dat) > 0 & ncol(iter_dat) == 9 & all(colnames(iter_dat) %in% col_nams) & length(unique(iter_dat$beam)) == 3)
})


# ..............................
# "get_level3a_data()" function
# ..............................


testthat::test_that("the function 'get_level3a_data()' gives an error in case the product is not one of 'atl06', 'atl07', 'atl08', 'atl10', 'atl12', 'atl13'!", {
  testthat::expect_error(get_level3a_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    startDate = "2020-01-01",
    endDate = "2020-01-01",
    trackId = 1290,
    beamName = NULL,
    product = "atl03"
  ))
})


testthat::test_that("the function 'get_level3a_data()' gives an error if the 'client' parameter is not one of 'portal', 'jupyter'!", {
  testthat::expect_error(get_level3a_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    startDate = "2020-01-01",
    endDate = "2020-01-03",
    trackId = 1290,
    beamName = NULL,
    product = "atl06",
    client = "INVALID"
  ))
})


testthat::test_that("the function 'get_level3a_data()' gives an error if the 'outputFormat' parameter is not one of 'csv', 'json', 'zip'!", {
  testthat::expect_error(get_level3a_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    startDate = "2020-01-01",
    endDate = "2020-01-03",
    trackId = 1290,
    beamName = NULL,
    product = "atl06",
    client = "portal",
    outputFormat = "INVALID"
  ))
})


testthat::test_that("the function 'get_level3a_data()' gives an error if the 'startDate' parameter is not equal or smaller to the 'endDate' parameter!", {
  testthat::expect_error(get_level3a_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    startDate = "2020-01-02",
    endDate = "2020-01-01",
    trackId = 1290,
    beamName = NULL,
    product = "atl06",
    client = "portal",
    outputFormat = "csv"
  ))
})


testthat::test_that("the function 'get_level3a_data()' gives an error if the 'beamName' parameter is invalid!", {
  testthat::expect_error(get_level3a_data(
    minx = -144.67439,
    miny = 59.22850,
    maxx = -137.88048,
    maxy = 61.69038,
    startDate = "2020-01-01",
    endDate = "2020-01-03",
    trackId = 1290,
    beamName = "INVALID",
    product = "atl06",
    client = "portal",
    outputFormat = "csv"
  ))
})


testthat::test_that("the function 'get_level3a_data()' returns the correct output for a specific input parameter setting!", {
  testthat::skip_on_cran() # skip on CRAN due to time limits and might fail
  testthat::skip_if_offline()

  iter_dat <- get_level3a_data(
    minx = minx,
    miny = miny,
    maxx = maxx,
    maxy = maxy,
    startDate = "2021-02-15",
    endDate = "2021-02-15",
    trackId = 817,
    beamName = NULL,
    product = "atl13",
    client = "portal",
    outputFormat = "csv"
  )

  # Accept NULL (graceful API failure) or 0-row data.frame as valid empty results
  testthat::expect_true(is.null(iter_dat) || nrow(iter_dat) == 0L)
  # testthat::expect_true(nrow(iter_dat) > 0 & ncol(iter_dat) == 10 & length(unique(iter_dat$beam)) == 3)
})

#' The "IceSat2R" package
#'
#' An important aspect of the "IceSat2R" (https://cran.r-project.org/web/packages/IceSat2R/index.html) package is that it includes the code, documentation, and
#' examples so that users can retrieve, process, and analyze data based on specific workflows.
#'
#' @details
#'
#' For instance,
#'
#' \itemize{
#'   \item A user can select an "area of interest" (AOI) either programmatically or interactively
#'   \item If the "Reference Ground Track" (RGT) is not known, the user has the option to utilize either
#'     \enumerate{
#'       \item one of the "overall_mission_orbits()" or "time_specific_orbits()" to compute the RGT(s) for a pre-specified global area or for a time period, or
#'       \item one of the "vsi_nominal_orbits_wkt()" or "vsi_time_specific_orbits_wkt()" to compute the RGT(s) for a specific AOI
#'     }
#'   \item Once the RGT is computed it can be verified with the "getTracks()" function of the "OpenAltimetry Web API" (https://openaltimetry.earthdatacloud.nasa.gov)
#'   \item Finally the user can utilize one of the "get_atlas_data()" or "get_level3a_data()" functions to retrieve the data for specific product(s), Date(s) and Beam(s)
#' }
#'
#' This work-flow is illustrated also in a \href{https://raw.githubusercontent.com/mlampros/IceSat2R/master/inst/joss_paper/figures/icesat_2_diagram.png}{diagram}
#'
#' @name IceSat2R
#'
#' @import lwgeom
#' @importFrom utils globalVariables
NULL

# ..............................................................
# reference: https://github.com/trafficonese/leaflet.extras
#
# "leaflet.extras" was archived at 2026-02-19 and is no
#  longer available on CRAN, thus I had to include the
#  required code snippets (I used previously) in this package
# ..............................................................

#' @importFrom htmlwidgets JS
#' @noRd
defIconFunction <-
  htmlwidgets::JS("function(icon){
        if (!$.isEmptyObject(icon)) {
          return L.icon(icon);
        } else {
          return L.icon();
        }
     }")

#' @importFrom htmlwidgets JS
#' @noRd
awesomeIconFunction <-
  htmlwidgets::JS("function(icon){
        if (!$.isEmptyObject(icon)) {
          if (!icon.prefix) {
            icon.prefix = icon.library;
          }
          return L.AwesomeMarkers.icon(icon);
        } else {
          return L.AwesomeMarkers.icon();
        }
     }")

#' inner function of 'compute_elapsed_time'
#'
#' @param secs a numeric value specifying the seconds
#' @param estimated a boolean. If TRUE then the output label becomes the 'Estimated time'
#' @return a character string showing the estimated or elapsed time
#'
#' @keywords internal

inner_elapsed_time = function(secs, estimated = FALSE) {
  tmp_hours = as.integer((secs / 60) / 60)
  tmp_hours_minutes = (secs / 60) %% 60
  tmp_seconds = secs %% 60
  est_verb = ifelse(estimated, "Estimated time: ", "Elapsed time: ")
  res_out = paste(c(est_verb, tmp_hours, " hours and ", as.integer(tmp_hours_minutes), " minutes and ", as.integer(tmp_seconds), " seconds."), collapse = "")
  return(res_out)
}


#' elapsed time in hours & minutes & seconds
#'
#' @param time_start a numeric value specifying the start time
#' @return It does not return a value but only prints the time in form of a character string in the R session
#'
#' @keywords internal

compute_elapsed_time = function(time_start) {
  t_end = proc.time()
  time_total = as.numeric((t_end - time_start)['elapsed'])
  time_ = inner_elapsed_time(time_total)
  cat(time_, "\n")
}

#' Natural Earth 10m Glaciated Areas (1:10 million scale)
#'
#' Polygons derived from DCW (Digital Chart of the World), except for Antarctica derived from MOA. Includes name attributes for major polar glaciers.
#' Filtering has aggregated some minor glaciers and eliminated others from the original DCW data.
#'
#' @references
#'
#' https://www.naturalearthdata.com/downloads/10m-physical-vectors/10m-glaciated-areas/
#'
#' @details
#'
#' Issues: Needs scale rank attributes
#'
#' Note: The original DCW data was collected decades ago. Since then many mountain glaciers have either disappeared or
#' diminished in size. This data theme is deliberately called "glaciated areas" instead of "glaciers" to reflect the
#' changeable character of glacier extents.
#'
#' @usage
#'
#' data(ne_10m_glaciated_areas)
#'
#' @keywords datasets
#'
#' @examples
#'
#' require(IceSat2R)
#' require(sf)
#'
#' data(ne_10m_glaciated_areas)
"ne_10m_glaciated_areas"



#' Reference Ground Tracks (RGTs) for IceSat-2 Cycle 14
#'
#' The data includes the following columns: "longitude", "latitude", "day_of_year", "Date", "hour", "minute", "second" and "RGT". The "RGT" column consists of
#' 1387 Reference Ground Tracks (RGTs) for the IceSat-2 Cycle 14 (from 'December 22, 2021' to 'March 23, 2022')
#'
#' @references
#'
#' https://icesat-2.gsfc.nasa.gov/science/specs
#'
#' @details
#'
#' The following code snippet shows how to come to the "RGT_cycle_14" data. The same can be done with any of the available RGT Cycles. For the following code
#' I utilized 8 threads to speed up the pre-processing of the downloaded .kml files (the code takes approximately 15 minutes on my Linux Personal Computer),
#'
#' \code{require(IceSat2R)} \cr
#' \code{require(magrittr)} \cr
#' \code{require(sf)} \cr
#'
#' \code{avail_cycles = available_RGTs(only_cycle_names = TRUE)} \cr
#' \code{avail_cycles} \cr
#'
#' \code{idx_cycle = 14} \cr
#'
#' \code{choose_cycle = avail_cycles[idx_cycle]} \cr
#' \code{choose_cycle} \cr
#'
#' \code{res_rgt_many = time_specific_orbits(RGT_cycle = choose_cycle, download_method = 'curl', threads = parallel::detectCores(), verbose = TRUE)} \cr
#'
#' \code{RGT_cycle_14 = sf::st_coordinates(res_rgt_many)} \cr
#' \code{colnames(RGT_cycle_14) = c('longitude', 'latitude')} \cr
#' \code{RGT_cycle_14 = data.table::data.table(RGT_cycle_14)} \cr
#' \code{RGT_cycle_14$day_of_year = res_rgt_many$day_of_year} \cr
#' \code{RGT_cycle_14$Date = as.Date(res_rgt_many$Date_time)} \cr
#' \code{RGT_cycle_14$hour = lubridate::hour(res_rgt_many$Date_time)} \cr
#' \code{RGT_cycle_14$minute = lubridate::minute(res_rgt_many$Date_time)} \cr
#' \code{RGT_cycle_14$second = lubridate::second(res_rgt_many$Date_time)} \cr
#' \code{RGT_cycle_14$RGT = res_rgt_many$RGT} \cr
#' \code{RGT_cycle_14} \cr
#'
#' @usage
#'
#' data(RGT_cycle_14)
#'
#' @keywords datasets
#'
#' @examples
#'
#' require(IceSat2R)
#' require(data.table)
#'
#' data(RGT_cycle_14)
"RGT_cycle_14"

# ..............................................................
# reference: https://github.com/trafficonese/leaflet.extras
#
# "leaflet.extras" was archived at 2026-02-19 and is no
#  longer available on CRAN, thus I had to include the
#  required code snippets (I used previously) in this package
# ..............................................................

html_dependency <- function(name, version, script, folder, ...) {
  htmltools::htmlDependency(
    name,
    version = version,
    system.file(folder, package = "IceSat2R"),
    script = script,
    ...
  )
}

# match the npm version
html_dep_prod <- function(name, version, has_style = FALSE, has_binding = FALSE, ..., stylesheet = NULL) {
  if (isTRUE(has_style)) {
    if (missing(stylesheet)) {
      stylesheet <- paste0(name, "-prod.css")
    }
  }

  script <- paste0(name, "-prod.js")
  if (isTRUE(has_binding)) {
    script <- c(script, paste0(name, "-bindings.js"))
  }
  html_dependency(
    name, version,
    script,
    file.path("htmlwidgets", "build", name),
    all_files = TRUE,
    ...,
    stylesheet = stylesheet
  )
}

# should only be used if there is not prod file
# keep the version at the lastest release version where the bindings were updated
html_dep_binding <- function(name, version, ...) {
  html_dependency(
    name, version,
    paste0(name, "-bindings.js"),
    file.path("htmlwidgets", "build", name),
    all_files = FALSE,
    ...
  )
}

#' a leaflet base map
#'
#' @param sf_obj a simple features object which must include the columns 'date' and 'size_MB' if the popup parameter is set to TRUE
#' @param leaflet_provider_base a leaflet provider object
#' @param leaflet_provider_secondary a leaflet provider object
#' @param overlay_group a character string specifying the overlay group
#' @param overlay_polylines a character string specifying the Polylines group
#' @param width a numeric value specifying the width of the leaflet map
#' @param height a numeric value specifying the height of the leaflet map
#' @param popup a boolean specifying if the popup should be shown or not
#' @param collapse_layer_control a boolean specifying if the layer-control should be collapsed or not
#'
#' @return a leaflet object (map)
#'
#' @import magrittr
#' @importFrom glue glue
#' @importFrom sf st_cast st_geometry
#' @importFrom leaflet leaflet addProviderTiles addLayersControl layersControlOptions
#' @importFrom leafgl addGlPolylines addGlPolygons
#'
#' @keywords internal

leaflet_base_map = function(sf_obj = NULL,
                            leaflet_provider_base,
                            leaflet_provider_secondary,
                            overlay_group = 'IceSat2 1-degree',
                            overlay_polylines = 'Polylines',
                            width = NULL,
                            height = NULL,
                            popup = NULL,
                            collapse_layer_control = TRUE) {

  lft = leaflet::leaflet(width = width, height = height) %>%

    leaflet::addProviderTiles(provider = leaflet_provider_base,
                                  group = leaflet_provider_base) %>%

    leaflet::addProviderTiles(provider = leaflet_provider_secondary,
                                  group = leaflet_provider_secondary)

  overlay_groups = character(0)

  if (!is.null(sf_obj)) {

    if (!is.null(popup)) popup = glue::glue("area (km^2): {round(x = as.numeric(sf_obj[[popup]]), digits = 2)}")

    #.............................................................................................
    # Using a 'LINESTRING' is a work-around to display the boundaries of the polygons using leafGL
    # see: https://github.com/r-spatial/leafgl/issues/3#issuecomment-635375213
    #      https://github.com/r-spatial/leafgl/issues/68#issuecomment-1005605056
    #.............................................................................................
    pl_ln_dat = sf::st_cast(sf::st_geometry(sf_obj), 'LINESTRING')

    lft = leafgl::addGlPolylines(map = lft,
                                 data = pl_ln_dat,
                                 color = 'blue',
                                 opacity = 1.0,
                                 popup = NULL,
                                 group = overlay_polylines) %>%

      leafgl::addGlPolygons(data = sf_obj,
                            fillColor = 'green',
                            fillOpacity = 0.0,
                            popup = popup,
                            group = overlay_group)

    overlay_groups = c(overlay_polylines, overlay_group)
  }

  lft = leaflet::addLayersControl(map = lft,
                                  baseGroups = c(leaflet_provider_base, leaflet_provider_secondary),
                                  overlayGroups = overlay_groups,
                                  options = leaflet::layersControlOptions(collapsed = collapse_layer_control),
                                  position = "topleft")
  return(lft)
}


#' R6 Class to Select an Area of Interest (AOI) from a Global Grid
#'
#' @description
#' The 'select_aoi_global_grid' class allows the user to select an Area of Interest (AOI) (see the examples section for all available options)
#'
#' @importFrom sf st_as_sfc st_intersects sf_use_s2 st_bbox st_crs
#' @importFrom miniUI miniPage miniContentPanel gadgetTitleBar miniTitleBarButton
#' @importFrom shiny observeEvent stopApp runGadget dialogViewer isTruthy
#' @importFrom rnaturalearth ne_countries
#' @importFrom leaflet providers
#' @importFrom leafsync sync
#' @importFrom glue glue
#'
#' @export
#' @docType class
#' @usage # init <- select_aoi_global_grid$new()
#' @examples
#'
#' \dontrun{
#'
#' require(IceSat2R)
#' require(magrittr)
#'
#' #........................................
#' # 1st. Option: Select an AOI from the map
#' #........................................
#'
#' init = select_aoi_global_grid$new(area_of_interest = NULL,
#'                                   verbose = TRUE)
#' init$draw_edit_aoi()
#' sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
#' sf_obj
#'
#'
#' #...............................................
#' # observe the available countries and continents
#' # using the 'rnaturalearth' R package
#' #...............................................
#'
#' cntr_cnt = rnaturalearth::ne_countries(scale = 110,
#'                                        type = 'countries',
#'                                        returnclass = 'sf')
#'
#' cntr_cnt = cntr_cnt[, c('sovereignt', 'continent')]
#'
#' # sort(cntr_cnt$sovereignt)
#' # sort(unique(cntr_cnt$continent))
#'
#'
#' #.......................................
#' # 2nd. Option: Select a 'country' as AOI   (5-degrees query)
#' #.......................................
#'
#' init = select_aoi_global_grid$new(area_of_interest = 'Antarctica',
#'                                   verbose = TRUE)
#'
#' init$draw_edit_aoi(degrees = 5.0, square_geoms = TRUE)
#' sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
#' sf_obj
#'
#'
#' #.........................................
#' # 3rd. Option: Select a 'continent' as AOI   (1-degree query)
#' #.........................................
#'
#' init = select_aoi_global_grid$new(area_of_interest = 'North America',
#'                                   verbose = TRUE)
#'
#' init$draw_edit_aoi(degrees = 1.0, square_geoms = TRUE)
#' sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
#' sf_obj
#'
#'
#' #.........................................
#' # 4th. Option: Use a bounding box as input  ('Greenland Ice Sheet')
#' #.........................................
#'
#' data(ne_10m_glaciated_areas)
#'
#' dat_bbx = ne_10m_glaciated_areas %>%
#'   subset(!is.na(name)) %>%
#'   subset(name == "Greenland Ice Sheet") %>%
#'   sf::st_bbox(crs = 4326)
#'
#' dat_bbx
#'
#' init = select_aoi_global_grid$new(area_of_interest = dat_bbx,
#'                                   verbose = TRUE)
#'
#' init$draw_edit_aoi(degrees = 1.0, square_geoms = TRUE)
#' sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
#' sf_obj
#'
#'
#' #......................................................
#' # 5th. Option: Create a global hexagonal 5-degrees grid
#' #......................................................
#'
#' bbx_global = c(xmin = -180, xmax = 180, ymin = -90, ymax = 90)
#'
#' init = select_aoi_global_grid$new(area_of_interest = bbx_global,
#'                                   verbose = TRUE)
#'
#' init$draw_edit_aoi(degrees = 5.0, square_geoms = FALSE)
#' sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
#' sf_obj
#'
#' }


select_aoi_global_grid <- R6::R6Class("select_aoi_global_grid",

                                     public = list(

                                       #' @field area_of_interest an R object (character string, vector)
                                       #' @field leaflet_provider_base a leaflet provider object
                                       #' @field leaflet_provider_secondary a leaflet provider object
                                       #' @field crs_value a numeric value
                                       #' @field use_s2 a boolean
                                       #' @field verbose a boolean
                                       area_of_interest = NULL,
                                       leaflet_provider_base = NULL,
                                       leaflet_provider_secondary = NULL,
                                       crs_value = NULL,
                                       use_s2 = FALSE,
                                       verbose = FALSE,

                                       #' @description
                                       #' Initialization method for the 'select_aoi_global_grid' R6 class
                                       #' @param area_of_interest either NULL (which allows the user to draw the area of interest on the map) or a character string (i.e. a 'country' or a 'continent') or a named bounding box vector (such as c(xmin = 16.1, xmax = 16.6, ymax = 48.6, ymin = 47.9)). The 'countries' and 'continents' can be extracted from the "rnaturalearth::ne_countries(scale = 110, type = 'countries', returnclass = 'sf')" function and specifically the columns: 'sovereignt' and 'continent'
                                       #' @param leaflet_provider_base a leaflet provider object
                                       #' @param leaflet_provider_secondary a leaflet provider object
                                       #' @param crs_value a value. The coordinate reference system. The default value of the crs is 4326
                                       #' @param use_s2 a boolean. If TRUE, use the s2 spherical geometry package for geographical coordinate operations (see the documentation of the 'sf::sf_use_s2()' function for more information)
                                       #' @param verbose a boolean. If TRUE then information will be printed in the console

                                       initialize = function(area_of_interest = NULL,
                                                             leaflet_provider_base = leaflet::providers$CartoDB.Positron,
                                                             leaflet_provider_secondary = leaflet::providers$Esri.WorldImagery,
                                                             crs_value = 4326,
                                                             use_s2 = FALSE,
                                                             verbose = FALSE) {

                                         sf::sf_use_s2(use_s2 = use_s2)             # do not use S2 (by default), because for EPSG:4326 can give an error especially when computing the 'intersection()' of 'sf' objects

                                         self$area_of_interest <- area_of_interest
                                         self$leaflet_provider_base <- leaflet_provider_base
                                         self$leaflet_provider_secondary <- leaflet_provider_secondary
                                         self$crs_value <- crs_value
                                         self$verbose <- verbose

                                         if (!is.null(self$area_of_interest)) {
                                           if (inherits(self$area_of_interest, 'character') & length(self$area_of_interest) == 1) {
                                             private$ne_cntr = rnaturalearth::ne_countries(scale = 110,
                                                                                           type = 'countries',
                                                                                           returnclass = 'sf')
                                             private$ne_cntr = private$ne_cntr[, c('sovereignt', 'continent')]
                                           }
                                         }
                                       },


                                       #' @description
                                       #' Internal Shiny application to visualize the selected area
                                       #'
                                       #' @param lft_map a leaflet map
                                       #'
                                       #' @references
                                       #'
                                       #' https://github.com/r-spatial/mapedit/blob/master/experiments/gadget_draw2.R

                                       draw_edit_internal = function(lft_map) {

                                         ui <- miniUI::miniPage(
                                           miniUI::miniContentPanel(lft_map),
                                           miniUI::gadgetTitleBar("Draw a Polygon to Select Grid Cells", right = miniUI::miniTitleBarButton("done", "Done", primary = TRUE))
                                         )

                                         server <- function(input, output, session) {
                                           draw_obj <- list()
                                           edited <- list()

                                           shiny::observeEvent(input$undefined_draw_new_feature, {                           # 'draw' new feature
                                             draw_obj <<- c(draw_obj, list(input$undefined_draw_new_feature))
                                           })

                                           shiny::observeEvent(input$undefined_draw_edited_features, {                       # 'edit' existing feature
                                             edited <<- input$undefined_draw_edited_features
                                             ids <- unlist(lapply(draw_obj, function(x) {
                                               x$properties$`_leaflet_id`
                                             }))

                                             lapply(edited$features, function(x){
                                               loc <- match(x$properties$`_leaflet_id`, ids)
                                               draw_obj[loc] <<- list(x)
                                             })
                                           })

                                           shiny::observeEvent(input$undefined_draw_deleted_features, {                       # 'delete' all existing features (and return or re-draw)

                                             if (shiny::isTruthy(input$undefined_draw_deleted_features)) {
                                               draw_obj <<- list()
                                             }
                                           })

                                           shiny::observeEvent(input$done, {
                                             shiny::stopApp(returnValue = draw_obj)
                                           })
                                                  # 'returnValue' is the value that should be returned from the 'shiny::runGadget'
                                           shiny::observeEvent(input$cancel, {
                                             shiny::stopApp (returnValue = NULL)
                                           })

                                           # session$onSessionEnded(function() {
                                           #   shiny::stopApp()
                                           # })
                                         }

                                         shiny::runGadget(
                                           ui,
                                           server,
                                           viewer =  shiny::dialogViewer("Draw and Edit the Global Grid of Polygons"),
                                           stopOnCancel = FALSE
                                         )
                                       },


                                       #' @description
                                       #' Allows to view the Global Grid on the map including a popup that shows the Area of each grid rectangle (or hexagon). The user can then select an Area of Interest (AOI)
                                       #'
                                       #' @param degrees a numeric value. This can be either 1.0 or 5.0 to allow queries to the 'OpenAltimetry' API
                                       #' @param square_geoms a boolean. If FALSE then a hexagonal grid will be created

                                       draw_edit_aoi = function(degrees = 1.0, square_geoms = TRUE) {

                                         # if (!degrees %in% c(1.0, 5.0)) stop("The 'OpenAltimetry' API allows up to 1-degree bounding box queries for 'atl03' and up to 5-degree bounding box queries for the 'atl06', 'atl07', 'atl08', 'atl10', 'atl12' and 'atl13' products!", call. = F)      # throwing an error is too strict, because it is possible that the user might want to create a bigger-sized grid
                                         if (!degrees %in% c(1.0, 5.0)) message("The 'OpenAltimetry' API allows up to 1-degree bounding box queries for 'atl03' and up to 5-degree bounding box queries for the 'atl06', 'atl07', 'atl08', 'atl10', 'atl12' and 'atl13' products!")

                                         if (!is.null(self$area_of_interest)) {
                                           if (inherits(self$area_of_interest, 'character') & length(self$area_of_interest) == 1) {

                                             flag_invalid = TRUE
                                             if (self$area_of_interest %in% private$ne_cntr$sovereignt) {
                                               dat_aoi = subset(private$ne_cntr, sovereignt == self$area_of_interest)
                                               dat_bbx = sf::st_bbox(dat_aoi)
                                               flag_invalid = FALSE
                                             }
                                             else {
                                               message(glue::glue("The 'area_of_interest' parameter [ '{self$area_of_interest}' ] is not one of the 'rnaturalearth' countries!"))
                                             }

                                             if (self$area_of_interest %in% private$ne_cntr$continent) {
                                               dat_aoi = subset(private$ne_cntr, continent == self$area_of_interest)
                                               dat_bbx = sf::st_bbox(dat_aoi)
                                               flag_invalid = FALSE
                                             }
                                             else {
                                               message(glue::glue("The 'area_of_interest' parameter [ '{self$area_of_interest}' ] is not one of the 'rnaturalearth' continents!"))
                                             }

                                             if (flag_invalid) {
                                               message("An invalid 'country' or 'continent' was used as input! The default leaflet map will be displayed!")
                                               self$area_of_interest <- NULL
                                             }
                                           }
                                           else if ((inherits(self$area_of_interest, 'numeric') & all(names(self$area_of_interest) %in% c("xmin", "xmax", "ymax", "ymin"))) | inherits(self$area_of_interest, "bbox")) {
                                             dat_bbx = sf::st_bbox(obj = self$area_of_interest, crs = sf::st_crs(self$crs_value))
                                             dat_aoi = sf::st_as_sfc(dat_bbx)
                                           }
                                           else {
                                             stop("Input to the 'area_of_interest' parameter must be either a character string (country, continent) or a numeric named vector (bounding box consisting of 'xmin', 'xmax', 'ymin', 'ymax')!", call. = F)
                                           }
                                         }
                                         else {
                                           dat_bbx = NULL
                                           dat_aoi = NULL
                                         }

                                         if (!is.null(dat_bbx) & !is.null(dat_aoi)) {

                                           if (self$verbose) cat("The global grid map will be computed ...\n")
                                           dat_aoi_grid = degrees_to_global_grid(minx = as.numeric(dat_bbx['xmin']),
                                                                                 maxx = as.numeric(dat_bbx['xmax']),
                                                                                 maxy = as.numeric(dat_bbx['ymax']),
                                                                                 miny = as.numeric(dat_bbx['ymin']),
                                                                                 degrees = degrees,
                                                                                 square_geoms = square_geoms,
                                                                                 crs_value = self$crs_value,
                                                                                 verbose = self$verbose)

                                           idx_inters = suppressMessages(sf::st_intersects(x = dat_aoi, y = dat_aoi_grid, sparse = T))
                                           idx_inters = data.frame(idx_inters)
                                           if (nrow(idx_inters) == 0) {
                                             stop(glue::glue("The bounding box ('xmin', 'xmax', 'ymin', 'ymax') does not intersect with the computed global grid of '{degrees}' degrees!"), call. = F)
                                           }
                                           else {
                                             private$dat_aoi = dat_aoi_grid[idx_inters$col.id, , drop = FALSE]
                                           }
                                         }
                                         else {
                                           private$dat_aoi = NULL
                                         }

                                         if (self$verbose) cat("The base leaflet map will be created ...\n")
                                         lft = leaflet_base_map(sf_obj = private$dat_aoi,
                                                                leaflet_provider_base = self$leaflet_provider_base,
                                                                leaflet_provider_secondary = self$leaflet_provider_secondary,
                                                                width = 570,                                                          # by default use 570 pixels in width and height
                                                                height = 570,
                                                                popup = NULL)                                                         # !! don't use a pop-up because I have to select an area on the map and this will also open the pop-up window

                                         if (self$verbose) cat("Add the draw toolbar ...\n")
                                         lft = addDrawToolbar(map = lft,
                                                                              position = 'topleft',
                                                                              polylineOptions = FALSE,                                # disable everything except for the Polygons
                                                                              circleOptions = FALSE,
                                                                              markerOptions = FALSE,
                                                                              circleMarkerOptions = FALSE,
                                                                              editOptions = editToolbarOptions())

                                         if (self$verbose) cat("The draw & edit toolbar will be opened ...\n")
                                         private$draw_obj = list()
                                         private$draw_obj = tryCatch(select_aoi_global_grid$public_methods$draw_edit_internal(lft_map = lft), error = function(e) e)

                                         if (inherits(private$draw_obj, 'error')) stop(glue::glue("The 'draw_edit_aoi()' function raised the following error: '{private$draw_obj$message}' !"), call. = F)
                                       },


                                       #' @description
                                       #' Takes the selected Area(s) of Interest (AOI) from the draw_edit_aoi() method and returns a simple features object with the corresponding n-degree grid cells
                                       #'
                                       #' @param plot_data a boolean specifying if the selected from the user AOI's and the corresponding grid cells should be plotted side by side. If FALSE then only the simple features object will be returned. If TRUE and the initial 'area_of_interest' parameter is NULL then a single plot will be displayed.
                                       #'
                                       #' @return either an 'sfc' object (if the initial 'area_of_interest' parameter is NULL) or an 'sf' object consisting of the n-degree grid cells

                                       selected_areas_global_grid = function(plot_data = FALSE) {

                                         if (inherits(private$draw_obj, 'list')) {
                                           if (length(private$draw_obj) == 0) {                       # if I 'delete' the existing geometries I expect this to return an empty list
                                             msg = "The edited map did not return any geometries!"
                                             if (self$verbose) message(msg)
                                             return(msg)
                                           }
                                           else {

                                             if (self$verbose) cat("The Polygons & MultiPolygons will be extracted ...\n")
                                             draw_obj_sf = unlist(lapply(private$draw_obj, function(x) {
                                               paste(c("POLYGON ((", paste(unlist(lapply(x$geometry$coordinates[[1]], function(y) paste(y, collapse = ' '))), collapse = ', '), "))"), collapse = "")
                                             }))

                                             if (self$verbose) cat("Conversion to Simple Features ...\n")
                                             private$aoi_sf = sf::st_as_sfc(draw_obj_sf, crs = self$crs_value)

                                             if (plot_data) {
                                               if (self$verbose) cat("The Leaflet Map of the 'selected areas' will be computed ...\n")
                                               lft_first = leaflet_base_map(sf_obj = private$aoi_sf,
                                                                            leaflet_provider_base = self$leaflet_provider_base,
                                                                            leaflet_provider_secondary = self$leaflet_provider_secondary,
                                                                            overlay_group = NULL,                                            # !! set this group to NULL for the 'sync' map to work
                                                                            overlay_polylines = 'User selected AOI',
                                                                            width = 750,
                                                                            height = 750,
                                                                            popup = NULL,                           # !! don't use a pop-up because I don't have the 'area' column
                                                                            collapse_layer_control = FALSE)
                                             }

                                             if (is.null(private$dat_aoi)) {
                                               if (self$verbose) cat("The simple map-edit functionality was used! Grid cell geometries won't be returned!\n")
                                               if (plot_data) print(lft_first)
                                               return(private$aoi_sf)
                                             }
                                             else {

                                               if (self$verbose) cat("Computation of the intersection between the selected areas and the global Grid ...\n")
                                               inters = sf::st_intersects(private$aoi_sf, private$dat_aoi, sparse = TRUE)
                                               inters = data.frame(inters)
                                               if (nrow(inters) == 0) stop("I expect that there is an intersection between the selected area and the global grid!", call. = F)

                                               if (self$verbose) cat("The relevant polygons will be extracted ...\n")
                                               dat_products = private$dat_aoi[inters$col.id, , drop = FALSE]

                                               if (self$verbose) cat(glue::glue("Number of grid cells: {nrow(dat_products)}  Total km^2 area (all grid cells): {round(x = sum(dat_products$area, na.rm = T), digits = 2)}"), '\n')

                                               if (plot_data) {

                                                 if (self$verbose) cat("The Leaflet Map of the 'global grid cells' will be computed ...\n")
                                                 lft_second = leaflet_base_map(sf_obj = dat_products,
                                                                               leaflet_provider_base = self$leaflet_provider_base,
                                                                               leaflet_provider_secondary = self$leaflet_provider_secondary,
                                                                               overlay_group = NULL,                                           # !! set this group to NULL for the 'sync' map to work
                                                                               overlay_polylines = 'Grid Cell',
                                                                               width = 750,
                                                                               height = 750,
                                                                               popup = 'area',                                # use the 'area' column to display the area per grid cell
                                                                               collapse_layer_control = FALSE)

                                                 lft_sync = leafsync::sync(lft_first, lft_second)
                                                 print(lft_sync)
                                               }

                                               return(dat_products)
                                             }
                                           }
                                         }
                                         else {
                                           stop("I expect that the 'private$draw_obj' is of type list!", call. = F)
                                         }
                                       },


                                       #' @description
                                       #' Returns the selected area of interest (AOI) by the user in form of an 'sfc' object
                                       #'
                                       #' @return an 'sfc' object

                                       selected_aoi_sf = function() {
                                         if (is.null(private$aoi_sf)) stop("Please run the 'selected_areas_global_grid()' method first!", call. = F)
                                         return(private$aoi_sf)
                                       }

                                     ),

                                     private = list(
                                       dat_aoi = NULL,
                                       draw_obj = NULL,
                                       aoi_sf = NULL,
                                       ne_cntr = NULL
                                     )
)

testthat::test_that("the 'select_aoi_global_grid()' R6 class and especially the 'draw_edit_aoi()' method gives an error if the 'area_of_interest' parameter is neither a character string nor an input object of class 'bbox'!", {

  testthat::skip_if_not(interactive())        # skip if not interactive because it might give the following NOTE on CRAN: "Check: for detritus in the temp directory Found the following files/directories: 'Crashpad'" (see also: https://stackoverflow.com/a/62456524)

  init = select_aoi_global_grid$new(area_of_interest = list(), verbose = FALSE)

  testthat::expect_error(init$draw_edit_aoi(degrees = 1.0))
})

#..............................
# Parameter setting (and Url's)
#..............................

minx = 140
miny = -6.641235
maxx = 145
maxy = -1.641235

date = "2021-02-11"
date_start = '2020-01-01'
date_end = '2020-01-01'
trackId = 756
beamName = 'gt1l'
product = 'atl08'
outputFormat = 'csv'
client = 'portal'
sampling = FALSE
sample_rgts = c("24", "32")

INVALID_URL = "https://openaltimetry.earthdatacloud.nasa.gov/INVALID/"
VALID_URL_a3 = glue::glue("https://openaltimetry.earthdatacloud.nasa.gov/data/api/icesat2/{product}?minx={minx}&miny={miny}&maxx={maxx}&maxy={maxy}&trackId={trackId}&beamNames={beamName}&outputFormat={outputFormat}&date={date}&client={client}&sampling={tolower(sampling)}")
VALID_URL_remaining = glue::glue("https://openaltimetry.earthdatacloud.nasa.gov/data/api/icesat2/{product}?minx={minx}&miny={miny}&maxx={maxx}&maxy={maxy}&trackId={trackId}&outputFormat={outputFormat}&date={date}&client={client}&beamNames={beamName}")
ORBITS_URL = 'https://icesat-2.gsfc.nasa.gov/sites/default/files/page_files/arcticallorbits.zip'
WKT = 'POLYGON ((-14.765 18.979, -11.25 18.979, -11.25 21.943, -14.765 21.943, -14.765 18.979))'

rgts = data.table::setDT(list(RGT = c(1251L, 1252L, 1260L, 1267L, 1275L),
                              Date_time = c("2020-12-15", "2020-12-15",
                                            "2020-12-15", "2020-12-16", "2020-12-16")))
bbx = c(xmin = -53.108876, ymin = 60.119614, xmax = -19.203521, ymax = 80.793117)

utils::globalVariables(c('km'))                        # for the conversion from meters to kilometers when using the 'units::set_units()' function


#' Create a global grid based on degrees
#'
#' This function allows the user to create a degrees grid based on an input bounding box
#'
#' @param minx the 'minx' parameter of the bounding box
#' @param maxx the 'maxx' parameter of the bounding box
#' @param miny the 'miny' parameter of the bounding box
#' @param maxy the 'maxy' parameter of the bounding box
#' @param degrees a numeric value specifying the degrees. It defaults to 1.0
#' @param square_geoms a boolean. If FALSE then a hexagonal grid will be created
#' @param crs_value a value. The coordinate reference system of the output grid. The crs must correspond to the EPSG of the input 'minx', 'maxx', 'miny' and 'maxy'. Moreover, the 'EPSG:4326' returns a degrees grid as output.
#' @param verbose a boolean. If TRUE then information will be printed out in the console
#'
#' @return a simple features (sf) object
#'
#' @details
#'
#' The default global input 'minx', 'maxx', 'miny' and 'maxy' correspond to a WKT of "POLYGON ((-180 -90, 180 -90, 180 90, -180 90, -180 -90))"
#' When 'minx', 'maxx', 'miny' and 'maxy' EPSG is lat,long (or 'EPSG:4326') I expect the output grid to be in 'degrees' unit. Using an EPSG other than 'EPSG:4326' might
#' return a different output unit (such as meters). See also the following 'Stackoverflow' thread for more information, https://stackoverflow.com/a/64903836/8302386
#'
#' Based on 'Approximate Metric Equivalents' 1 degree is approximately 111 km  (or 60 nautical miles), Reference: https://www.usna.edu/Users/oceano/pguth/md_help/html/approx_equivalents.htm
#'
#' @importFrom glue glue
#' @importFrom sf st_bbox st_as_sfc st_make_grid st_crs st_as_sf st_geometry st_area
#' @importFrom units set_units
#'
#' @export
#'
#' @examples
#'
#' \dontrun{
#'
#' require(IceSat2R)
#'
#' #...........................................
#' # 'OpenAltimetry' allows 1x1 degree bounding
#' # box selection for the 'atl03' Product
#' #...........................................
#'
#' gl_grid_1_d = degrees_to_global_grid(degrees = 1.0, verbose = TRUE)
#' gl_grid_1_d
#' # summary(gl_grid_1_d$area)
#'
#' #...........................................................................
#' # 'OpenAltimetry' allows 5x5 degree bounding box selection for the following
#' # Products: 'atl06', 'atl07', 'atl08', 'atl10', 'atl12', 'atl13', 'level3a'
#' #...........................................................................
#'
#' gl_grid_5_d = degrees_to_global_grid(degrees = 5.0, verbose = TRUE)
#' gl_grid_5_d
#'
#' }

degrees_to_global_grid = function(minx = -180,
                                  maxx = 180,
                                  maxy = 90,
                                  miny = -90,
                                  degrees = 1.0,
                                  square_geoms = TRUE,
                                  crs_value = 4326,
                                  verbose = FALSE) {
  if (verbose) {
    t_start = proc.time()
    cat('Create the bounding box and the sf object from the input minx, maxx, miny, maxy ...\n')
  }

  named_vec = c(xmin = minx, xmax = maxx, ymax = maxy, ymin = miny)
  glob_bounds_bbx = sf::st_bbox(obj = named_vec, crs = sf::st_crs(crs_value))
  glob_bounds = sf::st_as_sfc(glob_bounds_bbx)

  if (verbose) cat('Create the grid using the "sf::st_make_grid()" function ...\n')
  glob_grid = sf::st_make_grid(glob_bounds,
                               what = "polygons",
                               square = square_geoms,
                               cellsize = c(degrees, degrees),
                               flat_topped = FALSE,
                               crs = crs_value)
  update_name = 'geometry'
  glob_grid = sf::st_as_sf(glob_grid)
  names(glob_grid) = update_name                             # rename the geometry
  sf::st_geometry(glob_grid) = update_name                   # give the name 'geometry'

  if (verbose) cat('Compute the area in km^2 of the grid cells ...\n')
  glob_grid$area = sf::st_area(glob_grid)
  glob_grid$area = units::set_units(glob_grid$area, km^2)

  if (verbose) {
    cat(glue::glue('The output sf object consists of  {nrow(glob_grid)}  grid cells!'), '\n')
    compute_elapsed_time(time_start = t_start)
  }

  return(glob_grid)
}

testthat::test_that("the function 'degrees_to_global_grid()' gives the correct output either for 1- or for a 5-degrees grid!", {

  glob_grid_1_degr = degrees_to_global_grid(minx = minx,
                                            miny = miny,
                                            maxx = maxx,
                                            maxy = maxy,
                                            degrees = 1.0,
                                            square_geoms = TRUE,
                                            crs_value = 4326,
                                            verbose = FALSE)

  glob_grid_5_degr = degrees_to_global_grid(minx = minx,
                                            miny = miny,
                                            maxx = maxx,
                                            maxy = maxy,
                                            degrees = 5.0,
                                            square_geoms = TRUE,
                                            crs_value = 4326,
                                            verbose = FALSE)

  testthat::expect_true(nrow(glob_grid_1_degr) == 25 & ncol(glob_grid_1_degr) == 2 & nrow(glob_grid_5_degr) == 1 & ncol(glob_grid_5_degr) == 2)
})

# ..............................................................
# reference: https://github.com/trafficonese/leaflet.extras
#
# "leaflet.extras" was archived at 2026-02-19 and is no
#  longer available on CRAN, thus I had to include the
#  required code snippets (I used previously) in this package
# ..............................................................

#' Add Bootstrap dependency to a map
#' @param map the map widget
#' @importFrom leaflet leafletDependencies
#' @keywords internal
#' @noRd
addBootstrapDependency <- function(map) {
  map$dependencies <- c(
    map$dependencies,
    leaflet::leafletDependencies$bootstrap()
  )
  map
}

#' Add AwesomeMarkers and related lib dependencies to a map
#' @param map the map widget
#' @param libs char vector with lib names.
#' @importFrom leaflet leafletDependencies
#' @keywords internal
#' @noRd
addAwesomeMarkersDependencies <- function(map, libs) {
  map$dependencies <- c(
    map$dependencies,
    leaflet::leafletDependencies$awesomeMarkers()
  )
  if ("fa" %in% libs) {
    map$dependencies <- c(
      map$dependencies,
      leaflet::leafletDependencies$fontawesome()
    )
  }
  if ("ion" %in% libs) {
    map$dependencies <- c(
      map$dependencies,
      leaflet::leafletDependencies$ionicon()
    )
  }
  if ("glyphicon" %in% libs) {
    map$dependencies <- c(
      map$dependencies,
      leaflet::leafletDependencies$bootstrap()
    )
  }
  map
}

---
title: "ICESat-2 Atlas Products"
output: rmarkdown::pdf_document
always_allow_html: true
vignette: >
  %\VignetteIndexEntry{ICESat-2 Atlas Products}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
urlcolor: blue
---


```{r, include = FALSE}
knitr::opts_chunk$set(
  warning = FALSE,
  message = FALSE,
  fig.pos = "!H",
  echo = TRUE,
  eval = TRUE,
  screenshot.force = FALSE
) # this vignette by default does not evaluate the code snippets
```

<br>

This *second* vignette relies on the computation of *time specific orbits* (as was the case in the *first* vignette too) and explains how to use the 'atl06' *ATLAS OpenAltimetry* product (additional use cases can be found in the *Examples section* of the R package documentation). We'll restrict the Area of Interest (AOI) to one of the *Global glaciated areas* and specifically to the [Greenland Ice Sheet](https://en.wikipedia.org/wiki/Greenland_ice_sheet), which is the second-largest ice body in the world, after the Antarctic ice sheet.

<br>

Based on the OpenAltimetry documentation for the *ATL06* (ATLAS/ICESat-2 L3A Land Ice Height, Version 4) Product, *"This data set (ATL06) provides geolocated, land-ice surface heights (above the WGS 84 ellipsoid, ITRF2014 reference frame), plus ancillary parameters that can be used to interpret and assess the quality of the height estimates."*

<br>

We'll use 2 different time periods of the year (*winter* and *summer*) to observe potential differences on the *East* part of the *'Greenland Ice Sheet'* using the *Land Ice Height (ATL06)* Product and specifically we'll use,

* for the winter (2020, 2021) the **RGT_cycle_9** and **RGT_cycle_10** (from **2020-12-15** to **2021-02-15**)
* for the summer (2021) the **RGT_cycle_11** and **RGT_cycle_12** (from **2021-06-15** to **2021-08-15**)

<br>

First, we'll compute the time *specific orbits* for both periods,

<br>

```{r}
pkgs <- c(
  "IceSat2R", "sf", "data.table", "stargazer", "glue", "utils",
  "reshape2", "plotly", "magrittr", "geodist", "CopernicusDEM", "terra"
)
load_pkgs <- lapply(pkgs, require, character.only = TRUE) # load required R packages

sf::sf_use_s2(use_s2 = FALSE) # disable 's2' in this vignette
if (requireNamespace("mapview", quietly = TRUE)) {
  mapview::mapviewOptions(
    leafletHeight = "600px",
    leafletWidth = "700px"
  ) # applies to all leaflet maps
}
```

<br>

```r

#....................
# winter (2020, 2021)
#....................

start_date_w = "2020-12-15"
end_date_w = "2021-02-15"

rgt_winter = time_specific_orbits(date_from = start_date_w,
                                  date_to = end_date_w,
                                  RGT_cycle = NULL,
                                  download_method = 'curl',
                                  threads = parallel::detectCores(),
                                  verbose = TRUE)

# ICESAT-2 orbits: 'Earliest-Date' is '2018-10-13'  'Latest-Date' is '2022-06-21' 
# -----------------------------------------------------
# The .zip file of 'RGT_cycle_9' will be downloaded ... 
# -----------------------------------------------------
#   % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
#                                  Dload  Upload   Total   Spent    Left  Speed
# 100  140M  100  140M    0     0  3662k      0  0:00:39  0:00:39 --:--:-- 2904k
# The downloaded .zip file will be extracted in the '/tmp/RtmpPleeLI/RGT_cycle_9' directory . 
# Download and unzip the RGT_cycle_9 .zip file: Elapsed time: 0 hours and 0 minutes and 41 sec 
# 138 .kml files will be processed ... 
# Parallel processing of 138 .kml files using  8  threads starts ... 
# The 'description' column of the output data will be processed ...
# The temproary files will be removed ...
# Processing of cycle 'RGT_cycle_9': Elapsed time: 0 hours and 1 minutes and 9 seconds. 
# -----------------------------------------------------
# The .zip file of 'RGT_cycle_10' will be downloaded ... 
# -----------------------------------------------------
#   % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
#                                  Dload  Upload   Total   Spent    Left  Speed
# 100  140M  100  140M    0     0  3795k      0  0:00:37  0:00:37 --:--:-- 3841k
# The downloaded .zip file will be extracted in the '/tmp/RtmpPleeLI/RGT_cycle_10' directory . 
# Download and unzip the RGT_cycle_10 .zip file: Elapsed time: 0 hours and 0 minutes and 40 sec 
# 824 .kml files will be processed ... 
# Parallel processing of 824 .kml files using  8  threads starts ... 
# The 'description' column of the output data will be processed ...
# The temproary files will be removed ...
# Processing of cycle 'RGT_cycle_10': Elapsed time: 0 hours and 7 minutes and 53 seconds. 
# Total Elapsed time: 0 hours and 10 minutes and 25 seconds. 

```

<br>

```r

rgt_winter

# Simple feature collection with 91390 features and 14 fields
# Geometry type: POINT
# Dimension:     XY
# Bounding box:  xmin: -179.9029 ymin: -87.66478 xmax: 179.9718 ymax: 87.32805
# CRS:           4326
# First 10 features:
#    Name timestamp begin  end  altitudeMode tessellate extrude visibility drawOrder icon  RGT 
# 1            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 2            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 3            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 4            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 5            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 6            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 7            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 8            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 9            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 
# 10           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1250 

```

<br>

for the winter period, it took approximately 10 min. to download and process the 962 .kml files utilizing 8 threads and then return an 'sf' (simple features) object. We'll do the same for the summer period,

<br>

```r

#..............
# summer (2021)
#..............

start_date_s = "2021-06-15"
end_date_s = "2021-08-15"

rgt_summer = time_specific_orbits(date_from = start_date_s,
                                  date_to = end_date_s,
                                  RGT_cycle = NULL,
                                  download_method = 'curl',
                                  threads = parallel::detectCores(),
                                  verbose = TRUE)

# ICESAT-2 orbits: 'Earliest-Date' is '2018-10-13'  'Latest-Date' is '2022-06-21' 
# -----------------------------------------------------
# The .zip file of 'RGT_cycle_11' will be downloaded ... 
# -----------------------------------------------------
#   % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
#                                  Dload  Upload   Total   Spent    Left  Speed
# 100  140M  100  140M    0     0  3440k      0  0:00:41  0:00:41 --:--:-- 4227k
# The downloaded .zip file will be extracted in the '/tmp/RtmpPleeLI/RGT_cycle_11' directory . 
# Download and unzip the RGT_cycle_11 .zip file: Elapsed time: 0 hours and 0 minutes and 44 sec 
# 142 .kml files will be processed ... 
# Parallel processing of 142 .kml files using  8  threads starts ... 
# The 'description' column of the output data will be processed ...
# The temproary files will be removed ...
# Processing of cycle 'RGT_cycle_11': Elapsed time: 0 hours and 1 minutes and 30 seconds. 
# -----------------------------------------------------
# The .zip file of 'RGT_cycle_12' will be downloaded ... 
# -----------------------------------------------------
#   % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
#                                  Dload  Upload   Total   Spent    Left  Speed
# 100  140M  100  140M    0     0  3204k      0  0:00:44  0:00:44 --:--:-- 2333k
# The downloaded .zip file will be extracted in the '/tmp/RtmpPleeLI/RGT_cycle_12' directory . 
# Download and unzip the RGT_cycle_12 .zip file: Elapsed time: 0 hours and 0 minutes and 47 sec 
# 820 .kml files will be processed ... 
# Parallel processing of 820 .kml files using  8  threads starts ... 
# The 'description' column of the output data will be processed ...
# The temproary files will be removed ...
# Processing of cycle 'RGT_cycle_12': Elapsed time: 0 hours and 8 minutes and 20 seconds. 
# Total Elapsed time: 0 hours and 11 minutes and 22 seconds. 

```

<br>

```r

rgt_summer

# Simple feature collection with 89965 features and 14 fields
# Geometry type: POINT
# Dimension:     XY
# Bounding box:  xmin: -179.9861 ymin: -87.66478 xmax: 179.9393 ymax: 87.32805
# CRS:           4326
# First 10 features:
#    Name timestamp begin  end  altitudeMode tessellate extrude visibility drawOrder icon  RGT
# 1            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256 
# 2            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256 
# 3            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256 
# 4            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256 
# 5            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256 
# 6            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256 
# 7            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256 
# 8            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256 
# 9            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256
# 10           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1256

```

<br>

for the summer it took approximately the same time as for winter to process the 962 .kml files of the 2 Reference Ground Track (RGT) cycles. We'll proceed to find the intersection of these 2 time periods (winter, summer) with the area of the *East 'Greenland Ice Sheet'*,

<br>

```r

#...............................
# load the 'Greenland Ice Sheet' 
#...............................

data(ne_10m_glaciated_areas)

greenl_sh = subset(ne_10m_glaciated_areas, name == "Greenland Ice Sheet")
greenl_sh

```

<br>

We'll continue with one of the 2 Greenland Ice Sheet parts ('East')

<br>

```r

greenl_sh_east = greenl_sh[2, ]
# mapview::mapview(greenl_sh_east, legend = F)

#.....................................................
# create the bounding of the selected area because  
# it's required for the 'OpenAltimetry' functions this
#  will increase the size of the initial east area
#.....................................................

bbx_greenl_sh_east = sf::st_bbox(obj = greenl_sh_east)
sfc_bbx_greenl_sh_east = sf::st_as_sfc(bbx_greenl_sh_east)
# mapview::mapview(sfc_bbx_greenl_sh_east, legend = F)

```


```r

#..............................................
# intersection with the computed "winter" RGT's
#..............................................

inters_winter = sf::st_intersects(x = sfc_bbx_greenl_sh_east,
                                  y = sf::st_geometry(rgt_winter),
                                  sparse = TRUE)

#.....................
# matched (RGT) tracks
#.....................

df_inters_winter = data.frame(inters_winter)
rgt_subs_winter = rgt_winter[df_inters_winter$col.id, , drop = FALSE]
rgt_subs_winter

# Simple feature collection with 1079 features and 14 fields
# Geometry type: POINT
# Dimension:     XY
# Bounding box:  xmin: -53.06014 ymin: 61.26295 xmax: -19.23143 ymax: 80.40264
# CRS:           4326
# First 10 features:
#     Name timestamp begin  end  altitudeMode tessellate extrude visibility drawOrder icon  RGT
# 117           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1251 
# 207           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1252
# 208           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1252
# 209           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1252
# 210           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1252
# 211           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1252
# 212           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1252
# 977           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260
# 978           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260
# 979           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260

```

<br>

From the initial 91390 coordinate points, 1079 of the winter period intersect with the bounding box of our Area of Interest (AOI). We'll do the same for the summer period,

<br>

```r

#..............................................
# intersection with the computed "summer" RGT's
#..............................................

inters_summer = sf::st_intersects(x = sfc_bbx_greenl_sh_east,
                                  y = sf::st_geometry(rgt_summer),
                                  sparse = TRUE)

#.....................
# matched (RGT) tracks
#.....................

df_inters_summer = data.frame(inters_summer)
rgt_subs_summer = rgt_summer[df_inters_summer$col.id, , drop = FALSE]
rgt_subs_summer

# Simple feature collection with 1066 features and 14 fields
# Geometry type: POINT
# Dimension:     XY
# Bounding box:  xmin: -53.06014 ymin: 61.26295 xmax: -19.23143 ymax: 80.40264
# CRS:           4326
# First 10 features:
#      Name timestamp begin  end  altitudeMode tessellate extrude visibility drawOrder icon  RGT
# 407            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260
# 408            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260 
# 409            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260 
# 410            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260 
# 411            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260 
# 412            <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1260 
# 1062           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1267 
# 1063           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1267
# 1064           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1267
# 1065           <NA>  <NA> <NA> clampToGround         -1       0          1        NA <NA> 1267

```


<br>

for the summer period, the intersected points are 1066. Based on these 2 intersected objects of winter and summer we'll also have to find the common RGT's that we'll use for comparison purposes,

<br>

```r

#...............................................
# compute the unique RGT's for summer and winter
#...............................................

unq_rgt_winter = unique(rgt_subs_winter$RGT)
unq_rgt_summer = unique(rgt_subs_summer$RGT)
dif_rgt = setdiff(unique(rgt_subs_winter$RGT), unique(rgt_subs_summer$RGT))

cat(glue::glue("Number of RGT winter: {length(unq_rgt_winter)}"), '\n')
# Number of RGT winter: 230 

cat(glue::glue("Number of RGT summer: {length(unq_rgt_summer)}"), '\n') 
# Number of RGT summer: 227 

cat(glue::glue("Difference in RGT: {length(dif_rgt)}"), '\n') 
# Difference in RGT: 3 


#......................
# find the intersection
#......................

inters_rgts = intersect(unq_rgt_winter, unq_rgt_summer)


#...................................................
# create the subset data RGT's for summer and winter
#...................................................

idx_w = which(rgt_subs_winter$RGT %in% inters_rgts)
subs_rgt_winter = rgt_subs_winter[idx_w, , drop = F]

idx_s = which(rgt_subs_summer$RGT %in% inters_rgts)
subs_rgt_summer = rgt_subs_summer[idx_s, , drop = F]

```

<br>

Then we have to verify that the intersected *time specific orbit RGT's* match the *OpenAltimetry Tracks* for both winter and summer,

<br>

```r

#.........................................................
# we keep the relevant columns and remove duplicated 
# Dates and RGTs to iterate over each pair of observations
#.........................................................

#.......
# winter
#.......

subs_rgt_w_trc = subs_rgt_winter[, c('RGT', 'Date_time')]
subs_rgt_w_trc$Date_time = as.character(as.Date(subs_rgt_w_trc$Date_time))
dups_w = which(duplicated(sf::st_drop_geometry(subs_rgt_w_trc)))
subs_rgt_w_trc = subs_rgt_w_trc[-dups_w, ]

ver_trc_winter = verify_RGTs(nsidc_rgts = subs_rgt_w_trc, 
                             bbx_aoi = bbx_greenl_sh_east,
                             verbose = TRUE)

# split(ver_trc_winter, by = 'Date_time')

colnames(ver_trc_winter) = c('date_winter', 'RGT_OpenAlt', 'RGT_NSIDC')

#.......
# summer
#.......

subs_rgt_s_trc = subs_rgt_summer[, c('RGT', 'Date_time')]
subs_rgt_s_trc$Date_time = as.character(as.Date(subs_rgt_s_trc$Date_time))
dups_s = which(duplicated(sf::st_drop_geometry(subs_rgt_s_trc)))
subs_rgt_s_trc = subs_rgt_s_trc[-dups_s, ]

ver_trc_summer = verify_RGTs(nsidc_rgts = subs_rgt_s_trc, 
                             bbx_aoi = bbx_greenl_sh_east,
                             verbose = TRUE)

colnames(ver_trc_summer) = c('date_summer', 'RGT_OpenAlt', 'RGT_NSIDC')

#.............
# merge by RGT
#.............

rgts_ws = merge(ver_trc_winter[, 1:2], ver_trc_summer[, 1:2], by = 'RGT_OpenAlt')
colnames(rgts_ws) = c('RGT', 'date_winter', 'date_summer')
rgts_ws

#       RGT date_winter date_summer
#   1:    2  2020-12-24  2021-06-23
#   2:   10  2020-12-24  2021-06-24
#   3:   11  2020-12-24  2021-06-24
#   4:   17  2020-12-25  2021-06-24
#   5:   18  2020-12-25  2021-06-24
#  ---                             
# 226: 1366  2020-12-22  2021-06-22
# 227: 1367  2020-12-22  2021-06-22
# 228: 1373  2020-12-23  2021-06-22
# 229: 1374  2020-12-23  2021-06-22
# 230: 1382  2020-12-23  2021-06-23

```


```{r, echo = F}
# .........................................................................................
# For reproducibility purposes save only the 'date' and 'RGT' matches of winter and summer
# .........................................................................................

# data.table::fwrite(x = rgts_ws, file = system.file('data_files', 'vignette_data', 'winter_summer_inters.csv', package = "IceSat2R"), row.names = F)
```

<br>

Now that we have the available RGT's for the winter and summer Dates we have to create the *5-degree grid of the Greenland Ice sheet bounding box*, because the *'atl06' OpenAltimetry* product allows queries up to 5x5 degrees. Rather than a 5-degree grid, we will create a 4.5-degree to avoid any *'over limit' OpenAltimetry API* errors,

<br>

```r

greenl_grid = degrees_to_global_grid(minx = as.numeric(bbx_greenl_sh_east['xmin']),
                                     maxx = as.numeric(bbx_greenl_sh_east['xmax']),
                                     maxy = as.numeric(bbx_greenl_sh_east['ymax']),
                                     miny = as.numeric(bbx_greenl_sh_east['ymin']),
                                     degrees = 4.5,
                                     square_geoms = TRUE,
                                     crs_value = 4326,
                                     verbose = TRUE)

```

<br>

In the beginning, we created the bounding box of the *East 'Greenland Ice Sheet'* (the *bbx_greenl_sh_east* object), which automatically increased the dimensions of the Area of Interest (AOI). Now, that we have the up to 5x5 degree grid we can keep the grid cells that intersect with our initial area,

<br>

```r

inters_init = sf::st_intersects(sf::st_geometry(greenl_sh_east), greenl_grid)
inters_init = data.frame(inters_init)
inters_init = inters_init$col.id

greenl_grid_subs = greenl_grid[inters_init, , drop = F]

# Simple feature collection with 27 features and 1 field
# Geometry type: POLYGON
# Dimension:     XY
# Bounding box:  xmin: -53.10888 ymin: 60.11961 xmax: -17.10888 ymax: 82.61961
# CRS:           EPSG:4326
# First 10 features:
#                          geometry             area
# 1  POLYGON ((-53.10888 60.1196... 116709.30 [km^2]
# 2  POLYGON ((-48.60888 60.1196... 116709.30 [km^2]
# 3  POLYGON ((-44.10888 60.1196... 116709.30 [km^2]
# 9  POLYGON ((-53.10888 64.6196...  98928.06 [km^2]
# 10 POLYGON ((-48.60888 64.6196...  98928.06 [km^2]
# 11 POLYGON ((-44.10888 64.6196...  98928.06 [km^2]
# 12 POLYGON ((-39.60888 64.6196...  98928.06 [km^2]
# 13 POLYGON ((-35.10888 64.6196...  98928.06 [km^2]
# 14 POLYGON ((-30.60888 64.6196...  98928.06 [km^2]
# 15 POLYGON ((-26.10888 64.6196...  98928.06 [km^2]

```

<br>

We also have to make sure that the *winter* and *summer* data intersect with the up to 5x5 degree grid,

<br>

```r

#............
# winter join
#............

subs_join_w = sf::st_join(x = greenl_grid_subs,
                          y = subs_rgt_w_trc, 
                          join = sf::st_intersects, 
                          left = FALSE)

subs_join_w = sf::st_as_sfc(unique(sf::st_geometry(subs_join_w)), crs = 4326)
subs_join_w

# Geometry set for 9 features 
# Geometry type: POLYGON
# Dimension:     XY
# Bounding box:  xmin: -53.10888 ymin: 60.11961 xmax: -17.10888 ymax: 82.61961
# CRS:           EPSG:4326
# First 5 geometries:
# POLYGON ((-53.10888 60.11961, -48.60888 60.1196...
# POLYGON ((-48.60888 60.11961, -44.10888 60.1196...
# POLYGON ((-44.10888 60.11961, -39.60888 60.1196...
# POLYGON ((-26.10888 73.61961, -21.60888 73.6196...
# POLYGON ((-21.60888 73.61961, -17.10888 73.6196...

```


```r
#............
# summer join
#............

subs_join_s = sf::st_join(x = greenl_grid_subs,
                          y = subs_rgt_s_trc, 
                          join = sf::st_intersects, 
                          left = FALSE)

subs_join_s = sf::st_as_sfc(unique(sf::st_geometry(subs_join_s)), crs = 4326)
subs_join_s

# Geometry set for 9 features 
# Geometry type: POLYGON
# Dimension:     XY
# Bounding box:  xmin: -53.10888 ymin: 60.11961 xmax: -17.10888 ymax: 82.61961
# CRS:           EPSG:4326
# First 5 geometries:
# POLYGON ((-53.10888 60.11961, -48.60888 60.1196...
# POLYGON ((-48.60888 60.11961, -44.10888 60.1196...
# POLYGON ((-44.10888 60.11961, -39.60888 60.1196...
# POLYGON ((-26.10888 73.61961, -21.60888 73.6196...
# POLYGON ((-21.60888 73.61961, -17.10888 73.6196...

```

```{r, echo = FALSE}
subs_join_s <- readRDS(file = system.file("data_files", "vignette_data", "Greenland_grid_cells.RDS", package = "IceSat2R"))
```

<br>

Since the *winter* and *summer* intersected spatial data are identical,

<br>

```r

identical(subs_join_w, subs_join_s)
# [1] TRUE

```

<br>

we'll iterate only over one of the two 'sfc' objects. 

<br>

We can also visualize the available areas after the *sf-join* between the *winter and summer spatial data* and the *East 'Greenland Ice Sheet'*,

<br>

```{r}
if (requireNamespace("mapview", quietly = TRUE)) {
  mapview::mapview(subs_join_s, legend = F)
}
```

<br>

We can proceed that way and download the available **'atl06' Land Ice Height** data using the **get_level3a_data()** function that takes a time interval and a bounding box as input (the bounding box will consist of up to a 5x5 degree grid cells). To reduce the computation time in this vignette we'll restrict the for-loop to the first 5 Greenland Grid Cells,

<br>

```{r}
join_geoms <- 1:5
subs_join_reduced <- subs_join_s[join_geoms]

if (requireNamespace("mapview", quietly = TRUE)) {
  mapview::mapview(subs_join_reduced, legend = F)
}
```

<br>

and to the following RGTs,

<br>

```{r, echo = F, results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)}
pth_subs_RGTs <- system.file("data_files", "vignette_data", "Greenland_RGTs_grid_cells.csv", package = "IceSat2R")
dtbl_RGTs <- data.table::fread(pth_subs_RGTs, header = T, stringsAsFactors = F)
dtbl_RGTs <- dtbl_RGTs[order(dtbl_RGTs$RGT, decreasing = F), ]

stargazer::stargazer(dtbl_RGTs,
  summary = FALSE,
  rownames = FALSE,
  header = FALSE,
  float = FALSE,
  table.placement = "h",
  title = "Greenland Grid Cells"
)


# #.................................................................
# # I came to these RGT's after processing the next for-loop for
# #   - all 9 Greenland geoms
# #   - all RGT's (of the 'rgts_ws' variable)
# #.................................................................
#
# require(magrittr)
# logs_out_dtbl = data.table::rbindlist(logs_out)         # the output of the 'logs_out' list is required!
# logs_out_dtbl$index = names(dat_out_w)                  # the output of the 'dat_out_w' list is required!
# logs_out_dtbl = logs_out_dtbl[order(logs_out_dtbl$N_rows_winter, decreasing = T), ]
# logs_out_dtbl_subs = subset(logs_out_dtbl, N_rows_winter > 0 & N_rows_summer > 0)
# vec_grid = as.vector(unlist(lapply(strsplit(logs_out_dtbl_subs$index, '_'), function(x) x[3])))
# logs_out_dtbl_subs$greenland_cell = vec_grid
# spl_rgt = split(logs_out_dtbl_subs, by = 'greenland_cell')
# spl_subs = spl_rgt[as.character(1:5)] %>%
#   lapply(function(x) x[1:4, ]) %>%
#   data.table::rbindlist()
```

<br>

```r

#...............................................
# keep a subset of RGTs and Greenland Grid cells
#...............................................

RGTs = c(33, 41, 56, 94, 108, 284, 
         421, 422, 437, 460, 475, 
         521, 544, 658, 681, 787, 
         794, 1290, 1366, 1373)

#......................
# update the input data
#......................

rgts_ws_reduced = subset(rgts_ws, RGT %in% RGTs)
rgts_ws_RGT = rgts_ws_reduced$RGT


#..........................................................
# we'll loop over the RGT's for the Date start and Date end
#..........................................................

# length(unique(rgts_ws_reduced$RGT)) == nrow(rgts_ws_reduced)   # check for duplicated RGT's
rgts_ws_reduced$date_summer = as.Date(rgts_ws_reduced$date_summer)
rgts_ws_reduced$date_winter = as.Date(rgts_ws_reduced$date_winter)
min_max_dates = apply(rgts_ws_reduced[, -1], 2, function(x) c(min(x), max(x)))

start_w = as.character(min_max_dates[1, 'date_winter'])
end_w = as.character(min_max_dates[2, 'date_winter'])

start_s = as.character(min_max_dates[1, 'date_summer'])
end_s = as.character(min_max_dates[2, 'date_summer'])

dat_out_w = dat_out_s = logs_out = list()
LEN = length(subs_join_reduced)
LEN_rgt = length(rgts_ws_RGT)

t_start = proc.time()
for (idx_grid in 1:LEN) {
  
  geom_iter = subs_join_reduced[idx_grid]
  bbx_iter = sf::st_bbox(obj = geom_iter, crs = 4326)
  
  for (j in 1:LEN_rgt) {
    
    message("Greenland Geom: ", 
            idx_grid, "/",
            LEN, "   RGT-index: ",
            j, "/", 
            LEN_rgt, "\r", 
            appendLF = FALSE)
    
    utils::flush.console()
    
    track_i = rgts_ws_RGT[j]
    name_iter = glue::glue("geom_idx_{idx_grid}_RGT_{track_i}")
    
    iter_dat_winter = get_level3a_data(minx = as.numeric(bbx_iter['xmin']),
                                       miny = as.numeric(bbx_iter['ymin']),
                                       maxx = as.numeric(bbx_iter['xmax']),
                                       maxy = as.numeric(bbx_iter['ymax']),
                                       startDate = start_w,
                                       endDate = end_w,
                                       trackId = track_i,
                                       beamName = NULL,        # return data of all 6 beams
                                       product = 'atl06',
                                       client = 'portal',
                                       outputFormat = 'csv',
                                       verbose = FALSE)
    
    iter_dat_summer = get_level3a_data(minx = as.numeric(bbx_iter['xmin']),
                                       miny = as.numeric(bbx_iter['ymin']),
                                       maxx = as.numeric(bbx_iter['xmax']),
                                       maxy = as.numeric(bbx_iter['ymax']),
                                       startDate = start_s,
                                       endDate = end_s,
                                       trackId = track_i,
                                       beamName = NULL,        # return data of all 6 beams
                                       product = 'atl06',
                                       client = 'portal',
                                       outputFormat = 'csv',
                                       verbose = FALSE)
    
    NROW_w = nrow(iter_dat_winter)
    NROW_s = nrow(iter_dat_summer)
    
    if (NROW_s > 0 & NROW_w > 0) {
      iter_logs = list(RGT = track_i,
                       N_rows_winter = NROW_w,
                       N_rows_summer = NROW_s)
      
      logs_out[[name_iter]] = data.table::setDT(iter_logs)
      dat_out_w[[name_iter]] = iter_dat_winter
      dat_out_s[[name_iter]] = iter_dat_summer
    }
  }
}
IceSat2R:::compute_elapsed_time(time_start = t_start)
# Elapsed time: 0 hours and 6 minutes and 15 seconds.
```

<br>

We then sort and observe the output LOGs,

<br>

```r

logs_out_dtbl = data.table::rbindlist(logs_out)
logs_out_dtbl$index = names(dat_out_w)

```

```{r, echo = FALSE}
pth_logs <- system.file("data_files", "vignette_data", "LOGs.csv", package = "IceSat2R")
logs_out_dtbl <- data.table::fread(file = pth_logs, stringsAsFactors = F, header = T)
```

```{r, results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)}
logs_out_dtbl <- logs_out_dtbl[order(logs_out_dtbl$N_rows_winter, decreasing = T), ]

stargazer::stargazer(logs_out_dtbl,
  summary = FALSE,
  rownames = FALSE,
  header = FALSE,
  float = FALSE,
  table.placement = "h",
  title = "LOGs"
)
```

<br>

We'll first process and visualize one of Greenland's geometries and RGT,

<br>

```r

#................................
# we pick one with approx. same 
# rows for both summer and winter
#................................

# names(dat_out_w)
Greenland_Geom_index = 4
RGT = 1290

sublist_name = glue::glue("geom_idx_{Greenland_Geom_index}_RGT_{RGT}")

#...............
# winter sublist
#...............

w_subs = dat_out_w[[sublist_name]]
w_subs


#               date segment_id longitude latitude      h_li atl06_quality_summary track_id beam
#      1: 2020-12-17     566420 -22.43278 78.11961  553.5237                     0     1290 gt1l
#      2: 2020-12-17     566421 -22.43294 78.11943  553.5601                     0     1290 gt1l
#      3: 2020-12-17     566422 -22.43310 78.11926  553.5809                     0     1290 gt1l
#      4: 2020-12-17     566423 -22.43326 78.11908  553.5331                     0     1290 gt1l
#      5: 2020-12-17     566424 -22.43341 78.11891  553.5400                     0     1290 gt1l
#     ---                                                                                                                                         
# 152558: 2020-12-17     591901 -25.70488 73.62039 1640.8195                     0     1290 gt3r
# 152559: 2020-12-17     591902 -25.70496 73.62021 1650.2201                     0     1290 gt3r
# 152560: 2020-12-17     591903 -25.70505 73.62003 1657.0470                     0     1290 gt3r
# 152561: 2020-12-17     591904 -25.70514 73.61985 1665.8877                     0     1290 gt3r
# 152562: 2020-12-17     591905 -25.70523 73.61968 1675.1165                     0     1290 gt3r

```


```r

#...............
# summer sublist
#...............

s_subs = dat_out_s[[sublist_name]]
s_subs

#               date segment_id longitude latitude      h_li atl06_quality_summary track_id beam
#      1: 2021-06-17     566420 -22.43270 78.11961  554.1851                     0     1290 gt1l
#      2: 2021-06-17     566421 -22.43286 78.11943  554.2042                     0     1290 gt1l
#      3: 2021-06-17     566422 -22.43302 78.11925  554.1635                     0     1290 gt1l
#      4: 2021-06-17     566423 -22.43318 78.11908  554.1546                     0     1290 gt1l
#      5: 2021-06-17     566424 -22.43334 78.11890  554.2347                     0     1290 gt1l
#     ---                                                                                                                                         
# 152048: 2021-06-17     591901 -25.70511 73.62039 1638.9667                     0     1290 gt3r
# 152049: 2021-06-17     591902 -25.70521 73.62022 1646.6494                     0     1290 gt3r
# 152050: 2021-06-17     591903 -25.70530 73.62004 1654.2173                     0     1290 gt3r
# 152051: 2021-06-17     591904 -25.70539 73.61986 1663.3259                     0     1290 gt3r
# 152052: 2021-06-17     591905 -25.70548 73.61968 1673.2574                     0     1290 gt3r

```

<br>

The OpenAltimetry Data Dictionary includes the definitions for the column names of the output data.tables (except for the *beam* column which appears in the *'level3a'* product of the OpenAltimetry API),

* **segment_id**: *"Segment number, counting from the equator. Equal to the segment_id for the second of the two 20m ATL03 segments included in the 40m ATL06 segment"*
* **h_li**: *"Standard land-ice segment height determined by land ice algorithm, corrected for first-photon bias, representing the median-based height of the selected 'signal photon events' (PEs)"*
* **atl06_quality_summary**: *"The ATL06_quality_summary parameter indicates the best-quality subset of all ATL06 data. A 0.0 (zero) in this parameter implies that no data-quality tests have found a problem with the segment, a 1.0 (one) implies that some potential problem has been found. Users who select only segments with zero values for this flag can be relatively certain of obtaining high-quality data, but will likely miss a significant fraction of usable data, particularly in cloudy, rough, or low-surface-reflectance conditions."*
* **beam**: *the 6 (six) beams 'gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l' or 'gt3r'*

<br>

The *Date* of the selected sublist for *winter* is '2020-12-17' whereas for *summer* is '2021-06-17'. We'll

* exclude the low-quality observations using the 'atl06_quality_summary' column
* keep specific columns ('date', 'segment_id', 'longitude', 'latitude', 'h_li', 'beam')
* rename the winter and summer columns by adding the '_winter' and '_summer' extension
* merge the remaining observations (winter, summer) based on the 'segment_id' and 'beam' columns
* create an additional column (*'dif_height'*) with the difference in height between the *'h_li_winter'* and *'h_li_summer'*

<br>


```r

cols_keep = c('date', 'segment_id', 'longitude', 'latitude', 'h_li', 'beam')

w_subs_hq = subset(w_subs, atl06_quality_summary == 0)
w_subs_hq = w_subs_hq[, ..cols_keep]
colnames(w_subs_hq) = glue::glue("{cols_keep}_winter")

s_subs_hq = subset(s_subs, atl06_quality_summary == 0)
s_subs_hq = s_subs_hq[, ..cols_keep]
colnames(s_subs_hq) = glue::glue("{cols_keep}_summer")

sw_hq_merg = merge(x = w_subs_hq, 
                   y = s_subs_hq, 
                   by.x = c('segment_id_winter', 'beam_winter'), 
                   by.y = c('segment_id_summer', 'beam_summer'))

sw_hq_merg$dif_height = sw_hq_merg$h_li_winter - sw_hq_merg$h_li_summer

summary(sw_hq_merg$dif_height)

#     Min.  1st Qu.   Median     Mean  3rd Qu.     Max. 
# -42.6563  -0.4222  -0.0854  -0.1567   0.0537  26.4173

```

<br>

the following code snippet will create the visualizations for the beams *"gt1l"* and *"gt1r"* for the selected subset,

<br>

```r

cols_viz = c('segment_id_winter', 'beam_winter', 'h_li_winter', 'h_li_summer')
ws_vis = sw_hq_merg[, ..cols_viz]
```

```{r, echo = FALSE}
ws_vis <- readRDS(file = system.file("data_files", "vignette_data", "plotly_beams.RDS", package = "IceSat2R"))
```

```{r, eval = requireNamespace("reshape2", quietly = TRUE)}
ws_vis_mlt <- reshape2::melt(ws_vis, id.vars = c("segment_id_winter", "beam_winter"))
ws_vis_mlt <- data.table::data.table(ws_vis_mlt, stringsAsFactors = F)
ws_vis_mlt_spl <- split(ws_vis_mlt, by = "beam_winter")
# BEAMS = names(ws_vis_mlt_spl)       # plot all beams
```

```{r}
# ...................................
# function to plot each subplot beam
# ...................................

plotly_beams <- function(spl_data,
                         beam,
                         left_width,
                         left_height,
                         right_width,
                         right_height) {
  subs_iter <- spl_data[[beam]]

  cat(glue::glue("Plot for Beam '{beam}' will be created ..."), "\n")

  # ......................
  # plot for all segments
  # ......................

  fig_lns <- plot_ly(
    data = subs_iter,
    x = ~segment_id_winter,
    y = ~value,
    color = ~variable,
    colors = c("blue", "red"),
    line = list(width = 2),
    text = ~ glue::glue("land-ice-height: {value} Segment-id: {segment_id_winter}"),
    hoverinfo = "text",
    width = left_width,
    height = left_height
  ) %>%
    plotly::layout(
      xaxis = list(gridcolor = "grey", showgrid = T),
      yaxis = list(gridcolor = "grey", showgrid = T)
    ) %>%
    plotly::add_lines()

  # ..............................
  # plot for a subset of segments
  # ..............................

  segm_ids <- 588326:588908 # this subset of segments show a big difference betw. summer and winter
  plt_title <- glue::glue("Beam: '{beam}' ( Segments: from {min(segm_ids)} to {max(segm_ids)} )")
  subs_iter_segm <- subset(subs_iter, segment_id_winter %in% segm_ids)

  fig_spl <- plot_ly(
    data = subs_iter_segm,
    x = ~segment_id_winter,
    y = ~value,
    color = ~variable,
    colors = c("blue", "red"),
    line = list(width = 2),
    text = ~ glue::glue("land-ice-height: {value} Segment-id: {segment_id_winter}"),
    hoverinfo = "text",
    width = right_width,
    height = right_height
  ) %>%
    plotly::layout(
      xaxis = list(gridcolor = "grey", showgrid = T),
      yaxis = list(gridcolor = "grey", showgrid = T)
    ) %>%
    plotly::add_lines(showlegend = FALSE)

  both_plt <- plotly::subplot(list(fig_lns, fig_spl), nrows = 1, margin = 0.03, widths = c(0.7, 0.3)) %>%
    plotly::layout(title = plt_title)
  # plotly::export(p = both_plt, file = glue::glue('{beam}.png'))

  return(both_plt)
}
```

<br>

The output left plot shows all segment-id's for the *"gt1l"* and *"gt1r"* beams, whereas the right plot is restricted only to the segment-id's from 588326 to 588908 to highlight potential differences in land-ice-height between the winter and summer periods,

<br>

```{r, eval = requireNamespace("plotly", quietly = TRUE) && requireNamespace("reshape2", quietly = TRUE)}
plt_gt1l <- plotly_beams(
  spl_data = ws_vis_mlt_spl,
  beam = "gt1l",
  left_width = 1800,
  left_height = 800,
  right_width = 900,
  right_height = 400
)

plt_gt1l
```

<br>

```{r, eval = requireNamespace("plotly", quietly = TRUE) && requireNamespace("reshape2", quietly = TRUE)}
plt_gt1r <- plotly_beams(
  spl_data = ws_vis_mlt_spl,
  beam = "gt1r",
  left_width = 1800,
  left_height = 800,
  right_width = 900,
  right_height = 400
)
plt_gt1r
```

```{r, echo = F}
# #......................................................
# # save all images for all beams in a separate directory
# #......................................................
#
# nams_ws = names(dat_out_w)
# save_summary = save_dat = list()
#
# for (nam_iter in nams_ws) {
#
#   cat("-----------------\n")
#   cat(nam_iter, '\n')
#   cat("-----------------\n")
#
#   w_subs = dat_out_w[[nam_iter]]
#   s_subs = dat_out_s[[nam_iter]]
#
#   cols_keep = c('date', 'segment_id', 'longitude', 'latitude', 'h_li', 'beam')
#
#   w_subs_hq = subset(w_subs, atl06_quality_summary == 0)
#   w_subs_hq = w_subs_hq[, ..cols_keep]
#   colnames(w_subs_hq) = glue::glue("{cols_keep}_winter")
#
#   s_subs_hq = subset(s_subs, atl06_quality_summary == 0)
#   s_subs_hq = s_subs_hq[, ..cols_keep]
#   colnames(s_subs_hq) = glue::glue("{cols_keep}_summer")
#
#   sw_hq_merg = merge(x = w_subs_hq,
#                      y = s_subs_hq,
#                      by.x = c('segment_id_winter', 'beam_winter'),
#                      by.y = c('segment_id_summer', 'beam_summer'))
#
#   if (nrow(sw_hq_merg) > 0) {
#     sw_hq_merg$dif_height = sw_hq_merg$h_li_winter - sw_hq_merg$h_li_summer
#
#     save_dat[[nam_iter]] = sw_hq_merg
#     save_summary[[nam_iter]] = data.table::setDT(list(name_iter = nam_iter,
#                                                       min = min(sw_hq_merg$dif_height),
#                                                       mean = mean(sw_hq_merg$dif_height),
#                                                       median = median(sw_hq_merg$dif_height),
#                                                       max = max(sw_hq_merg$dif_height),
#                                                       N_rows = nrow(sw_hq_merg)))
#     #.......................................
#     # save the plots for visual verification
#     #.......................................
#
#     cols_viz = c('segment_id_winter', 'beam_winter', 'h_li_winter', 'h_li_summer')
#     ws_vis = sw_hq_merg[, ..cols_viz]
#
#     ws_vis_mlt = reshape2::melt(ws_vis, id.vars = c('segment_id_winter', 'beam_winter'))
#     ws_vis_mlt = data.table::data.table(ws_vis_mlt, stringsAsFactors = F)
#     ws_vis_mlt_spl = split(ws_vis_mlt, by = 'beam_winter')
#
#     dir_save = file.path('all_beams_all_RGTs', nam_iter)           # !! create the 'all_beams_all_RGTs' directory first
#     if (!dir.exists(dir_save)) dir.create(dir_save)
#
#     BEAMS = names(ws_vis_mlt_spl)       # plot all beams
#
#     for (beam in BEAMS) {
#
#       subs_iter = ws_vis_mlt_spl[[beam]]
#
#       cat(glue::glue("Plot for Beam '{beam}' will be saved ..."), '\n')
#
#       #......................
#       # plot for all segments
#       #......................
#
#       fig_lines = plotly::plot_ly(data = subs_iter,
#                                   x = ~segment_id_winter,
#                                   y = ~value,
#                                   color = ~variable,
#                                   colors = c("blue", "red"),
#                                   line = list(width = 2),
#                                   text = ~glue::glue("land-ice-height: {value}  Segment-id: {segment_id_winter}"),
#                                   hoverinfo = "text",
#                                   width = 1800,
#                                   height = 1000) %>%
#
#         plotly::layout(xaxis = list(gridcolor = "grey", showgrid = T),
#                        yaxis = list(gridcolor = "grey", showgrid = T)) %>%
#
#         plotly::add_lines()
#
#       plotly::export(p = fig_lines, file = file.path(dir_save, glue::glue('{beam}.png')))
#     }
#   }
#   else {
#    message(glue::glue("Empty data table after merging for idx and RGT: '{nam_iter}'"))
#   }
# }
#
# save_summary = data.table::rbindlist(save_summary)
# save_summary = save_summary[order(save_summary$max, decreasing = T), ]
# save_summary
```

<br>

Finally, we can also add the elevation of the AOI for comparison purposes. We'll choose another Greenland Grid cell, RGT, and beams,

<br>

```r

Greenland_Geom_index = 2
RGT = 33

sublist_name = glue::glue("geom_idx_{Greenland_Geom_index}_RGT_{RGT}")

w_subs = dat_out_w[[sublist_name]]  
s_subs = dat_out_s[[sublist_name]]

cols_keep = c('date', 'segment_id', 'longitude', 'latitude', 'h_li', 'beam')

w_subs_hq = subset(w_subs, atl06_quality_summary == 0)
w_subs_hq = w_subs_hq[, ..cols_keep]
colnames(w_subs_hq) = glue::glue("{cols_keep}_winter")

s_subs_hq = subset(s_subs, atl06_quality_summary == 0)
s_subs_hq = s_subs_hq[, ..cols_keep]
colnames(s_subs_hq) = glue::glue("{cols_keep}_summer")

sw_hq_merg = merge(x = w_subs_hq, 
                   y = s_subs_hq, 
                   by.x = c('segment_id_winter', 'beam_winter'), 
                   by.y = c('segment_id_summer', 'beam_summer'))

```

<br>

After merging the winter and summer data for the specific Grid Cell and RGT, I have to keep only one pair of (latitude, longitude) coordinates for visualization purposes. I'll compute the distance between the winter and summer coordinates (of each row) for the same *segment_id* and *beam* and I'll continue with the beams and observations that have the lowest difference in distance (for a fair comparison),

<br>

```r

#...............................
# compute the pair-wise distance
#...............................

sw_hq_merg$dist_dif = geodist::geodist(x = sw_hq_merg[, c('longitude_winter', 'latitude_winter')], 
                                       y = sw_hq_merg[, c('longitude_summer', 'latitude_summer')], 
                                       paired = TRUE,
                                       measure = 'geodesic')
#..............
# split by beam
#..............

spl_beam = split(sw_hq_merg, by = 'beam_winter')

#...............................................
# compute the summary of the distance to observe
# the beams with the lowest distance difference
#...............................................

sm_stats = lapply(spl_beam, function(x) {
  summary(x$dist_dif)
})

# $gt1l
#     Min.  1st Qu.   Median     Mean  3rd Qu.     Max. 
# 0.006829 1.568755 2.107793 2.083547 2.654482 3.911961 
# $gt1r
#      Min.   1st Qu.    Median      Mean   3rd Qu.      Max. 
# 0.0008427 0.2483260 0.5219864 0.5766648 0.8521821 2.0041663 
# $gt2l
#      Min.   1st Qu.    Median      Mean   3rd Qu.      Max. 
# 0.0002195 0.3001418 0.6112493 0.6753225 0.9860298 2.2545978 
# $gt2r
#    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#   3.170   4.697   5.356   5.300   5.931   7.170 
# $gt3l
#    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#   1.197   2.679   3.115   3.155   3.621   5.069 
# $gt3r
#    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#   3.881   5.066   5.488   5.498   5.949   7.114 

```

<br>

We observe that the beams 'gt1r' and 'gt2l' return the lowest difference in distance (approximately 2.0 and 2.25 meters) for the coordinates between the 'winter' and 'summer' observations, therefore we continue with these 2 beams. Moreover, the 'gt1r' and 'gt2l' beams are separated by approximately 3 kilometers from each other.

<br>


```r

#......................................
# keep only the 'gt1r' and 'gt2l' beams
#......................................

sw_hq_merg_beams = subset(sw_hq_merg, beam_winter %in% c('gt1r', 'gt2l'))


#.......................................................
# keep only a pair of coordinates and rename the columns
#.......................................................

sw_hq_merg_beams = sw_hq_merg_beams[, c('segment_id_winter', 'beam_winter', 'longitude_winter',
                                        'latitude_winter', 'h_li_winter', 'h_li_summer')]
colnames(sw_hq_merg_beams) = c('segment_id', 'beam', 'longitude', 
                               'latitude', 'h_li_winter', 'h_li_summer')

#       segment_id beam longitude latitude h_li_winter h_li_summer
#    1:     351810 gt1r -44.10891 63.26390    2724.035    2725.023
#    2:     351811 gt1r -44.10895 63.26408    2724.004    2725.155
#    3:     351812 gt1r -44.10900 63.26426    2724.445    2725.298
#    4:     351813 gt1r -44.10904 63.26444    2724.530    2725.461
#    5:     351814 gt1r -44.10908 63.26462    2725.084    2725.650
#   ---                                                           
# 8700:     359399 gt1r -44.44637 64.61873    2777.249    2777.390
# 8701:     359400 gt1r -44.44642 64.61891    2777.136    2777.297
# 8702:     359401 gt1r -44.44647 64.61908    2777.081    2777.192
# 8703:     359402 gt1r -44.44651 64.61926    2776.936    2777.076
# 8704:     359403 gt1r -44.44656 64.61944    2776.818    2776.968

```

<br>

Then we'll download the *30-meter raster DEM (Digital Elevation Model)* for the AOI using the [CopernicusDEM](https://CRAN.R-project.org/package=CopernicusDEM) R package,

<br>

```r

sf_aoi = sf::st_as_sf(sw_hq_merg_beams, coords = c('longitude', 'latitude'), crs = 4326)
bbx_aoi = sf::st_bbox(sf_aoi)
sfc_aoi = sf::st_as_sfc(bbx_aoi)

dem_dir = tempdir()
dem_dir

dem30 = CopernicusDEM::aoi_geom_save_tif_matches(sf_or_file = sfc_aoi,
                                                 dir_save_tifs = dem_dir,
                                                 resolution = 30,
                                                 crs_value = 4326,
                                                 threads = parallel::detectCores(),
                                                 verbose = TRUE)

if (nrow(dem30$csv_aoi) > 1) {          # create a .VRT file if I have more than 1 .tif files
  file_out = file.path(dem_dir, 'VRT_mosaic_FILE.vrt')

  vrt_dem30 = CopernicusDEM::create_VRT_from_dir(dir_tifs = dem_dir,
                                                 output_path_VRT = file_out,
                                                 verbose = TRUE)
}

if (nrow(dem30$csv_aoi) == 1) {         # if I have a single .tif file keep the first index
  file_out = list.files(dem_dir, pattern = '.tif', full.names = T)[1]
}


#.......................................................
# crop the raster to the bounding box of the coordinates
#.......................................................

rst_inp = terra::rast(x = file_out)
vec_crop = terra::vect(x = sfc_aoi)
rst_crop = terra::crop(x = rst_inp, 
                       y = vec_crop, 
                       snap = "out")      # snap = "in" gives NA's

pth_egm = file.path(dem_dir, 'copernicus_dem_egm2008.tif')
terra::writeRaster(x = rst_crop, filename = pth_egm)

rst_dem = terra::rast(pth_egm)
rst_dem

CRS_dem = terra::crs(rst_dem, proj = TRUE)
RES_dem = as.character(terra::res(rst_dem))
EXT_dem = as.vector(terra::ext(rst_dem))
EXT_dem = c(EXT_dem['xmin'], EXT_dem['ymin'], EXT_dem['xmax'], EXT_dem['ymax'])
EXT_dem = as.character(EXT_dem)

#...........................................................................................................
# The Copernicus DEM has a "horizontal" CRS (Coordinate Reference System) of WGS84-G1150 (EPSG 4326)
# and a "vertical" CRS of EGM2008 (EPSG 3855) - geoid. Therefore, a transformation of the vertical 
# CRS is required to match the ellipsoid CRS of the ICESat-2 data
# The Table 1 (page 10 of 37) of the following weblink includes more information:
# https://spacedata.copernicus.eu/documents/20126/0/GEO1988-CopernicusDEM-SPE-002_ProductHandbook_I1.00.pdf
# The 2.5 minute geoid grid .gtx file of EGM2008 was downloaded from the following "osgeo" url:
# http://download.osgeo.org/proj/vdatum/egm08_25/
#...........................................................................................................

pth_gtx = file.path(dem_dir, 'egm08_25.gtx')

download.file(url = 'http://download.osgeo.org/proj/vdatum/egm08_25/egm08_25.gtx', 
              destfile = pth_gtx, 
              method = 'curl')

pth_dem_transformed = file.path(dem_dir, 'copernicus_dem_egm2008_transformed.tif')

convrt = sf::gdal_utils(
  util = "warp",
  source = pth_egm,
  destination = pth_dem_transformed,
  options = c("-s_srs", glue::glue("{CRS_dem} +geoidgrids={pth_gtx}"),
              "-t_srs", CRS_dem,
              "-tr", RES_dem,
              "-te", EXT_dem),
  quiet = FALSE)

#....................................
# load the transformed Copernicus DEM
#....................................

rst_crop_transformed = terra::rast(x = pth_dem_transformed)


#...............................................
# we also have to find the closest elevation 
# value to the 'winter' and 'summer' coordinates
# using the raster resolution
#...............................................

ter_dtbl = data.table::as.data.table(x = rst_crop_transformed, xy = TRUE, cells = TRUE)
colnames(ter_dtbl) = c("cell", "lon_dem30", "lat_dem30", "dem30")
# length(unique(ter_dtbl$cell)) == nrow(ter_dtbl)

xy = as.matrix(sw_hq_merg_beams[, c('longitude', 'latitude')])
sw_cells = terra::cellFromXY(object = rst_crop_transformed, xy = xy)

sw_hq_merg_beams$cell = sw_cells
# length(unique(sw_hq_merg_beams$cell)) < nrow(sw_hq_merg_beams)

merg_cells = merge(x = sw_hq_merg_beams, y = ter_dtbl, by = 'cell')


#......................................................................
# compute also the difference in distance between the beam measurements
# and the DEM coordinates (based on the 30-meter resolution cells)
#......................................................................

merg_cells$dem_dif_dist = geodist::geodist(x = merg_cells[, c('longitude', 'latitude')], 
                                           y = merg_cells[, c('lon_dem30', 'lat_dem30')], 
                                           paired = TRUE,
                                           measure = 'geodesic')

summary(merg_cells$dem_dif_dist)

#   Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.2356  8.1619 11.5943 11.1490 14.3201 20.5394

```

<br>

Based on the included 30-meter DEM there is a mean distance of 11.1490 and a maximum distance of 20.5394 meters between the *beam* and *DEM* coordinates. The following 3-dimensional interactive line plot shows,

* in *blue* color the *elevation* based on the *DEM* (compared to the 2 beams as these are separated by a 3-km distance)
* in *orange* color the land-ice-height measurements of the *summer* period (separately for 'gt1r' and 'gt2l')
* in *green* color the land-ice-height measurements of the *winter* period (separately for 'gt1r' and 'gt2l')

<br>

```r

cols_viz_dem = c('beam', 'longitude', 'latitude', 'h_li_winter', 'h_li_summer', 'dem30')
merg_cells_viz = merg_cells[, ..cols_viz_dem]

```

```{r, echo = FALSE}
# merg_cells_viz = readRDS(file = system.file('data_files', 'vignette_data', '3d_plot.RDS', package = "IceSat2R"))
```

```r

merg_cells_viz_mlt = reshape2::melt(merg_cells_viz, id.vars = c('beam', 'longitude', 'latitude'))
merg_cells_viz_mlt = data.table::data.table(merg_cells_viz_mlt, stringsAsFactors = F)
colnames(merg_cells_viz_mlt) = c('beam', 'longitude', 'latitude',  'variable', 'height')

fig_height = plotly::plot_ly(merg_cells_viz_mlt, 
                             x = ~longitude,
                             y = ~latitude,
                             z = ~height, 
                             split = ~beam,           # split by beam
                             type = 'scatter3d',
                             mode = 'lines',
                             color = ~variable,
                             line = list(width = 10),
                             width = 1000,
                             height = 900)
fig_height

```

<br>

![](3d_plotly.png)

<br>

---
title: "ICESat-2 Mission Orbits"
output: rmarkdown::pdf_document
always_allow_html: true
vignette: >
  %\VignetteIndexEntry{ICESat-2 Mission Orbits}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
urlcolor: blue
---


```{r, include = FALSE}
knitr::opts_chunk$set(
  fig.width = 4,
  fig.height = 4,
  fig.align = "center",
  fig.pos = "!H",
  warning = FALSE,
  message = FALSE,
  echo = TRUE,
  eval = TRUE
)
```

<br>

This *first* vignette demonstrates how to download and process *time specific orbits*. We'll use one of the *Reference Ground Track (RGT) cycles* and merge it with other data sources with the purpose to visualize specific areas. 

We'll load one of the latest which is *"RGT_cycle_14"* (from *December 22, 2021* to *March 23, 2022*). The documentation of the *"RGT_cycle_14"* data includes more details on how a user can come to the same data format for any of the RGT Cycles.

<br>

```{r}
pkgs <- c(
  "IceSat2R", "magrittr", "sf", "rnaturalearth",
  "data.table", "DT", "stargazer"
)
load_pkgs <- lapply(pkgs, require, character.only = TRUE) # load required R packages

sf::sf_use_s2(use_s2 = FALSE) # disable 's2' in this vignette
if (requireNamespace("mapview", quietly = TRUE)) {
  mapview::mapviewOptions(
    leafletHeight = "600px",
    leafletWidth = "700px"
  ) # applies to all leaflet maps
}

# .............................
# load the 'RGT_cycle_14' data
# .............................

data(RGT_cycle_14)

res_rgt_many <- sf::st_as_sf(x = RGT_cycle_14, coords = c("longitude", "latitude"), crs = 4326)
res_rgt_many
```

<br>

## ICESat-2 and Countries intersection

<br>

We'll proceed to merge the orbit geometry points with the countries data of the *rnaturalearth* R package (1:110 million scales) and for this purpose, we keep only the *"sovereignt"* and *"sov_a3"* columns,

<br>

```{r}
cntr <- rnaturalearth::ne_countries(scale = 110, type = "countries", returnclass = "sf")
cntr <- cntr[, c("sovereignt", "sov_a3")]
cntr
```

<br>

We then merge the orbit points with the country geometries and specify also *"left = TRUE"* to keep also observations that do not intersect with the *rnaturalearth* countries data,

<br>

```{r}
dat_both <- suppressMessages(sf::st_join(
  x = res_rgt_many,
  y = cntr,
  join = sf::st_intersects,
  left = TRUE
))
dat_both
```

<br>

The unique number of RGT's for *"RGT_cycle_14"* are

<br>

```{r}
length(unique(dat_both$RGT))
```


<br>

We observe that from *December 22, 2021* to *March 23, 2022*,

<br>


```{r}
df_tbl <- data.frame(table(dat_both$sovereignt), stringsAsFactors = F)
colnames(df_tbl) <- c("country", "Num_IceSat2_points")

df_subs <- dat_both[, c("RGT", "sovereignt")]
df_subs$geometry <- NULL
df_subs <- data.table::data.table(df_subs, stringsAsFactors = F)
colnames(df_subs) <- c("RGT", "country")
df_subs <- split(df_subs, by = "country")
df_subs <- lapply(df_subs, function(x) {
  unq_rgt <- sort(unique(x$RGT))
  items <- ifelse(length(unq_rgt) < 5, length(unq_rgt), 5)
  concat <- paste(unq_rgt[1:items], collapse = "-")
  iter_dat <- data.table::setDT(list(
    country = unique(x$country),
    Num_RGTs = length(unq_rgt),
    first_5_RGTs = concat
  ))
  iter_dat
})

df_subs <- data.table::rbindlist(df_subs)

df_tbl <- merge(df_tbl, df_subs, by = "country")
df_tbl <- df_tbl[order(df_tbl$Num_IceSat2_points, decreasing = T), ]
```

<br>

```{r, eval = requireNamespace("DT", quietly = TRUE)}
DT_dtbl <- DT::datatable(df_tbl, rownames = FALSE)
```

<br>

```{r, echo = FALSE, eval = requireNamespace("DT", quietly = TRUE)}
DT_dtbl
```

<br>

all RGT's (1387 in number) intersect with *"Antarctica"* and almost all with *"Russia"*.

<br>

## 'Onshore' and 'Offshore' Points ICESat-2 coverage

<br>

The **onshore** and **offshore** number of ICESat-2 points and percentages for the *"RGT_cycle_14"* equal to

<br>

```{r} 
num_sea <- sum(is.na(dat_both$sovereignt))
num_land <- sum(!is.na(dat_both$sovereignt))

perc_sea <- round(num_sea / nrow(dat_both), digits = 4) * 100.0
perc_land <- round(num_land / nrow(dat_both), digits = 4) * 100.0

dtbl_land_sea <- data.frame(list(
  percentage = c(perc_sea, perc_land),
  Num_Icesat2_points = c(num_sea, num_land)
))

row.names(dtbl_land_sea) <- c("sea", "land")
```

<br>

```{r, results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)}
stargazer::stargazer(dtbl_land_sea,
  summary = FALSE,
  rownames = TRUE,
  header = FALSE,
  float = FALSE,
  table.placement = "!h",
  title = "Land and Sea Proportions"
)
```

<br>

## Global glaciated areas and ICESat-2 coverage

<br>

We can also observe the ICESat-2 *"RGT_cycle_14"* coverage based on the 1 to 10 million large scale [Natural Earth Glaciated Areas](https://www.naturalearthdata.com/downloads/10m-physical-vectors/10m-glaciated-areas/) data,

<br>

```{r}
data(ne_10m_glaciated_areas)
```

<br>

We'll restrict the processing to the major polar glaciers (that have a name included),

<br>

```{r}
ne_obj_subs <- subset(ne_10m_glaciated_areas, !is.na(name))
ne_obj_subs <- sf::st_make_valid(x = ne_obj_subs) # check validity of geometries
ne_obj_subs
```

<br>

and we'll visualize the subset using the *mapview* package,

<br>

```{r}
if (requireNamespace("mapview", quietly = TRUE)) {
  mpv <- mapview::mapview(ne_obj_subs,
    color = "cyan",
    col.regions = "blue",
    alpha.regions = 0.5,
    legend = FALSE
  )
  mpv
}
```

<br>

We will see which orbits of the ICESat-2 *"RGT_cycle_14"* intersect with these major polar glaciers,

<br>

```{r}
res_rgt_many$id_rgt <- 1:nrow(res_rgt_many) # include 'id' for fast subsetting

dat_glac_sf <- suppressMessages(sf::st_join(
  x = ne_obj_subs,
  y = res_rgt_many,
  join = sf::st_intersects
))

dat_glac <- data.table::data.table(sf::st_drop_geometry(dat_glac_sf), stringsAsFactors = F)
dat_glac <- dat_glac[complete.cases(dat_glac), ] # keep non-NA observations
dat_glac
```

<br>

We'll split the merged data by the *'name'* of the glacier,

<br>

```{r}
dat_glac_name <- split(x = dat_glac, by = "name")

sum_stats_glac <- lapply(dat_glac_name, function(x) {
  dtbl_glac <- x[, .(
    name_glacier = unique(name),
    Num_unique_Dates = length(unique(Date)),
    Num_unique_RGTs = length(unique(RGT))
  )]
  dtbl_glac
})

sum_stats_glac <- data.table::rbindlist(sum_stats_glac)
sum_stats_glac <- sum_stats_glac[order(sum_stats_glac$Num_unique_RGTs, decreasing = T), ]
```

<br>

The next table shows the total number of days and RGTs for each one of the major polar glaciers,

<br>

```{r, results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)}
stargazer::stargazer(sum_stats_glac,
  summary = FALSE,
  rownames = FALSE,
  header = FALSE,
  float = FALSE,
  table.placement = "h",
  title = "Days and RGTs"
)
```

<br>

We can restrict to one of the glaciers to visualize the ICESat-2 *"RGT_cycle_14"* coverage over this specific area (*'Southern Patagonian Ice Field'*),

<br>

```{r}
sample_glacier <- "Southern Patagonian Ice Field"
dat_glac_smpl <- dat_glac_name[[sample_glacier]]
```

<br>

```{r, results = 'asis', eval = requireNamespace("stargazer", quietly = TRUE)}
cols_display <- c("name", "day_of_year", "Date", "hour", "minute", "second", "RGT")

stargazer::stargazer(dat_glac_smpl[, ..cols_display],
  summary = FALSE,
  rownames = FALSE,
  header = FALSE,
  float = FALSE,
  table.placement = "h",
  title = "Southern Patagonian Ice Field"
)
```

<br>

and we gather the intersected RGT coordinates points with the selected glacier,

<br>

```{r}
subs_rgts <- subset(res_rgt_many, id_rgt %in% dat_glac_smpl$id_rgt)

set.seed(1)
samp_colrs <- sample(
  x = grDevices::colors(distinct = TRUE),
  size = nrow(subs_rgts)
)
subs_rgts$color <- samp_colrs
```

<br>

```{r}
ne_obj_subs_smpl <- subset(ne_obj_subs, name == sample_glacier)

if (requireNamespace("mapview", quietly = TRUE)) {
  mpv_glacier <- mapview::mapview(ne_obj_subs_smpl,
    color = "cyan",
    col.regions = "blue",
    alpha.regions = 0.5,
    legend = FALSE
  )

  mpv_RGTs <- mapview::mapview(subs_rgts,
    color = subs_rgts$color,
    alpha.regions = 0.0,
    lwd = 6,
    legend = FALSE
  )
}
```

<br>

and visualize both the glacier and the subset of the intersected RGT coordinate points (of the different Days) in the same map. The clickable map and point popups include more information,

<br>


```{r}
if (requireNamespace("mapview", quietly = TRUE)) {
  lft <- mpv_glacier + mpv_RGTs
  lft
}
```

<br>

---
title: "ICESat-2 Virtual File System Orbits"
output: rmarkdown::pdf_document
always_allow_html: true
vignette: >
  %\VignetteIndexEntry{ICESat-2 Virtual File System Orbits}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
urlcolor: blue
---


```{r, include = FALSE}
knitr::opts_chunk$set(fig.width = 4,
                      fig.height = 6, 
                      fig.align = "center",
                      fig.pos = "!H",
                      warning = FALSE, 
                      message = FALSE,
                      echo = TRUE,
                      eval = TRUE)
```

<br>

In this *third* vignette I'll make use of the [GDAL Virtual File Systems](https://gdal.org/en/stable/user/virtual_file_systems.html) to download *nominal* and *time specific* orbit data (from the [ICESat-2 Technical Specs website](https://icesat-2.gsfc.nasa.gov/science/specs)) for an Area of Interest (AOI). This will allow me to reduce the download and computation time once I make the requests to the OpenAltimetry API. 

The area of interest is the **Himalayas mountain range** which has some of the *highest peaks* in the world, including *mount Everest*. The following map shows the bounding box area that I'll use in this vignette,

<br>

```{r himalayas-aoi, fig.pos='h', out.width = "105%", out.height = "100%", fig.align = 'center', fig.cap = "Area of Interest", fig.alt="Display the area of interest in Himalayas", echo = F}

knitr::include_graphics("himalayas_aoi.png")

# mp1 = mapview::mapview(geoms_himal[2, ], legend = F)      # see next code snippet for the "geoms_himal" variable
# mp2 = mapview::mapview(geoms_himal[2, ], legend = F)
# 
# 
# leafsync::sync(mp1, mp2, ncol = 2)
```

<br>

First, we load the data,

<br>

```r

pkgs = c('IceSat2R', 'magrittr', 'sf', 'mapview', 'leaflet')
load_pkgs = lapply(pkgs, require, character.only = TRUE)  # load required R packages

geoms_himal_pth = system.file('data_files', 'vignette_data', 'himalayas.RDS', package = "IceSat2R")
geoms_himal = readRDS(geoms_himal_pth)
geoms_himal

# Simple feature collection with 2 features and 1 field
# Geometry type: POLYGON
# Dimension:     XY
# Bounding box:  xmin: 86.35254 ymin: 26.95635 xmax: 87.29736 ymax: 28.25842
# CRS:           EPSG:4326
#   area_size                       geometry
# 1     small POLYGON ((86.36902 27.66164...
# 2       big POLYGON ((86.35254 26.95635...

```

<br>

Since the *Himalayas mountain range* is located in the *Eastern Hemisphere* we'll pick this as an area when calling the *IceSat2R::vsi_nominal_orbits_wkt()* function. Moreover, we'll iterate over all *8 available repeats* for the *Eastern Hemisphere* to retrieve the *Reference Ground Tracks (RGTs)* of the AOI based on the *nominal orbits*,

<br>

```r

sf_wkt = sf::st_geometry(subset(geoms_himal, area_size == 'big'))

centr_wkt = sf::st_coordinates(sf::st_centroid(sf_wkt))
dat_wkt = sf::st_as_text(sf_wkt)

lst_out = list()

for (iter in 1:8) {         # iterate over all available repeats
  
  cat(paste0(iter, '.'))
  
  dat_iter = IceSat2R::vsi_nominal_orbits_wkt(orbit_area = 'eastern_hemisphere',
                                              track = 'GT7',
                                              rgt_repeat = iter,
                                              wkt_filter = dat_wkt,
                                              download_method = 'curl',
                                              download_zip = FALSE,
                                              verbose = TRUE)
  lst_out[[iter]] = dat_iter
}

# 1.The available Icesat-2 orbits will be red from 'https://icesat-2.gsfc.nasa.gov/ ... 
# Access the data of the technical specs website ...
# Extract the .zip files and the corresponding titles ...
# Keep the relevant data from the url's and titles ...
# Process the nominal and time specific orbits separately ...
# Adjust the Dates of the time specific orbits ...
# Create the nominal orbits data.table ...
# Create the time specific orbits data.table ...
# Return a single data.table ...
# .............
# 8.The available Icesat-2 orbits will be red from 'https://icesat-2.gsfc.nasa.gov/ ... 
# Access the data of the technical specs website ...
# .............
# Elapsed time: 0 hours and 0 minutes and 2 seconds. 
# Data based on repeat and track will be kept ...
# Data based on repeat and track will be kept ...
# The file 'EasternHem_repeat8_GT7.kmz' will be processed ... 
# Total Elapsed time: 0 hours and 0 minutes and 5 seconds. 

```

<br>

```r

lst_out = unlist(lst_out, recursive = F)
unq_rgts = as.vector(unique(unlist(lapply(lst_out, function(x) x$RGT))))
unq_rgts

 # [1] "96"   "157"  "363"  "538"  "599"  "805"  "866"  "1041" "1308" "1247"

```


<br>

```{r reference-plt-1, echo = F}
# #......................................................... mapview visualization
# 
# # make the sf-objects valid
# nams = names(lst_out)
# lst_out = lapply(lst_out, function(x) sf::st_make_valid(x))
# names(lst_out) <- nams
# 
# sf_wkt = sf::st_make_valid(sf_wkt)
# 
# # Plot the two sf-objects
# RGTs = mapview::mapview(lst_out, legend = F)
# AOI_wkt = mapview::mapview(sf_wkt, legend = F)
# 
# lft = RGTs + AOI_wkt
# 
# require(magrittr)
# 
# lft = lft@map %>% leaflet::setView(lng = as.numeric(centr_wkt[, 'X']),
#                                    lat = as.numeric(centr_wkt[, 'Y']),
#                                    zoom = 7)
# lft
# #.......................................................

```

<br>

For this specific use case we are interested in ICESat-2 data for a specific time period,

* from *'2020-01-01'* to *'2021-01-01'* (1-year's data)

Therefore, we'll make use of the *IceSat2R::vsi_time_specific_orbits_wkt()* function which queries all *15 ICESat-2 RGTs cycles* (as of March 2022) to come to the RGTs intersection for the specified 1-year time interval,

<br>

```r

date_start = '2020-01-01'
date_end = '2021-01-01'

orb_cyc_multi = IceSat2R::vsi_time_specific_orbits_wkt(date_from = date_start,
                                                       date_to = date_end,
                                                       RGTs = unq_rgts,
                                                       wkt_filter = dat_wkt,
                                                       verbose = TRUE)

# The available Icesat-2 orbits will be red from 'https://icesat-2.gsfc.nasa.gov/ ... 
# Access the data of the technical specs website ...
# Extract the .zip files and the corresponding titles ...
# Keep the relevant data from the url's and titles ...
# Process the nominal and time specific orbits separately ...
# Adjust the Dates of the time specific orbits ...
# Create the nominal orbits data.table ...
# Create the time specific orbits data.table ...
# Return a single data.table ...
# Elapsed time: 0 hours and 0 minutes and 0 seconds. 
# In total there are 5 intersected dates for which data will be processed! 
# The RGT cycles from which data will be processed are: 
#            RGT_cycle_6, RGT_cycle_7, RGT_cycle_8, RGT_cycle_9, RGT_cycle_10 
# -------------------------------------------------
# RGTs of cycle 'RGT_cycle_6' will be processed ... 
# -------------------------------------------------
# The 'sf' gdalinfo returned an empty character string! Attempt to read the url using 
#      the OS configured 'gdalinfo' function ...
# The internal type of the .zip file is 'kml' 
# The 'https://icesat-2.gsfc.nasa.gov/sites/default/files/page_files/IS2_RGTs_cycle6_...' 
#      'zip'  file includes  1387  'kml' files. 
# Elapsed time: 0 hours and 0 minutes and 8 seconds. 
# 6 out of 10 sublists were empty and will be removed!
# -------------------------------------------------
# RGTs of cycle 'RGT_cycle_7' will be processed ... 
# -------------------------------------------------
# 
# .................
# 
# -------------------------------------------------
# RGTs of cycle 'RGT_cycle_10' will be processed ... 
# -------------------------------------------------
# The 'sf' gdalinfo returned an empty character string! Attempt to read the url using th ...
#       'gdalinfo' function ...
# The internal type of the .zip file is 'kml' 
# The 'https://icesat-2.gsfc.nasa.gov/sites/default/files/page_files/IS2_RGTs_cycle10_date ... 
#       'zip'  file includes  1387  'kml' files. 
# Elapsed time: 0 hours and 0 minutes and 6 seconds. 
# 6 out of 10 sublists were empty and will be removed!
# In total 5 RGT cycles will be included in the output 'sf' object (RGT_cycle_6, RGT_cycle_7,
#      RGT_cycle_8, RGT_cycle_9, RGT_cycle_10)! 
# output of 'RGT_cycle_6' will be re-formatted ... 
# The 'description' column of the output data will be processed ...
# output of 'RGT_cycle_7' will be re-formatted ... 
# The 'description' column of the output data will be processed ...
# output of 'RGT_cycle_8' will be re-formatted ... 
# The 'description' column of the output data will be processed ...
# output of 'RGT_cycle_9' will be re-formatted ... 
# The 'description' column of the output data will be processed ...
# output of 'RGT_cycle_10' will be re-formatted ... 
# The 'description' column of the output data will be processed ...
# Total Elapsed time: 0 hours and 2 minutes and 37 seconds. 

```

<br>

The query returns 18 different Date-Time matches for our defined 1-year time period,

<br>

```r

orb_cyc_multi

# Simple feature collection with 18 features and 14 fields
# Geometry type: POINT
# Dimension:     XY
# Bounding box:  xmin: 86.45225 ymin: 27.09347 xmax: 87.22874 ymax: 27.11331
# CRS:           EPSG:4326
# First 10 features:
#    .... drawOrder icon  RGT           Date_time day_of_year cycle                  geometry
# 1  ....        NA <NA>   96 2020-01-02 00:37:11           2     6 POINT (86.97015 27.10272)
# 2  ....        NA <NA>  538 2020-01-30 23:13:14          30     6 POINT (87.22874 27.09347)
# 3  ....        NA <NA>  599 2020-02-03 23:04:54          34     6 POINT (86.45225 27.11331)
# 4  ....        NA <NA> 1041 2020-03-03 21:40:57          63     6  POINT (86.71086 27.1045)
# 5  ....        NA <NA>   96 2020-04-01 20:17:02          92     7 POINT (87.09815 27.08729)
# 6  ....        NA <NA>  599 2020-05-04 18:44:45         125     7 POINT (86.58026 27.09789)
# 7  ....        NA <NA> 1041 2020-06-02 17:20:48         154     7 POINT (86.83886 27.08907)
# 8  ....        NA <NA>   96 2020-07-01 15:56:55         183     8 POINT (87.00215 27.09888)
# 9  ....        NA <NA>  538 2020-07-30 14:32:58         212     8 POINT (87.26075 27.08963)
# 10 ....        NA <NA>  599 2020-08-03 14:24:38         216     8 POINT (86.48426 27.10947)


```

<br>

We'll use the *mapview* R package to visualize our AOI bounding box with the intersected time-specific RGTs,

<br>

```r

# make the sf-objects valid
orb_cyc_multi = sf::st_make_valid(orb_cyc_multi)
sf_wkt = sf::st_make_valid(sf_wkt)

# plot the sf-objects
orbit_cy = mapview::mapview(orb_cyc_multi, legend = F)
AOI_wkt = mapview::mapview(sf_wkt, legend = F)

lft = orbit_cy + AOI_wkt

lft = lft@map %>% leaflet::setView(lng = as.numeric(centr_wkt[, 'X']),
                                   lat = as.numeric(centr_wkt[, 'Y']),
                                   zoom = 7)
lft

```

<br>

```{r himalayas-rgts, fig.pos='h', out.width = "80%", out.height = "80%", fig.align = 'center', fig.cap = "Intersected RGTs", fig.alt="Intersected Reference Ground Tracks", echo = F}

knitr::include_graphics("himalayas_rgts.png")

```

\newpage

The output of *'vsi_time_specific_orbits_wkt()'* can be verified with the *OpenAltimetry's 'getTracks()'* function,

<br>

```r

bbx_aoi = sf::st_bbox(obj = sf_wkt)

dtbl_rgts = verify_RGTs(nsidc_rgts = orb_cyc_multi, 
                        bbx_aoi = bbx_aoi, 
                        verbose = TRUE)
dtbl_rgts

#      Date_time RGT_OpenAlt RGT_NSIDC
#  1: 2020-01-02          96        96
#  2: 2020-01-30         538       538
#  3: 2020-02-03         599       599
#  4: 2020-03-03        1041      1041
#  5: 2020-04-01          96        96
#  6: 2020-05-04         599       599
#  7: 2020-06-02        1041      1041
#  8: 2020-07-01          96        96
#  9: 2020-07-30         538       538
# 10: 2020-08-03         599       599
# 11: 2020-09-01        1041      1041
# 12: 2020-09-30          96        96
# 13: 2020-11-02         599       599
# 14: 2020-12-01        1041      1041
# 15: 2020-12-30          96        96
# 16: 2021-01-28         538       538
# 17: 2021-02-01         599       599
# 18: 2021-03-02        1041      1041

```

<br>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_Tracks_data.R
\name{getTracks}
\alias{getTracks}
\title{Get the ICESAT-2 Tracks}
\usage{
getTracks(
  minx,
  miny,
  maxx,
  maxy,
  date,
  outputFormat = "csv",
  download_method = "curl",
  verbose = FALSE
)
}
\arguments{
\item{minx}{the 'minx' parameter of the bounding box}

\item{miny}{the 'miny' parameter of the bounding box}

\item{maxx}{the 'maxx' parameter of the bounding box}

\item{maxy}{the 'maxy' parameter of the bounding box}

\item{date}{a character string specifying the Data collection date in the format 'yyyy-MM-dd' (such as '2020-01-01')}

\item{outputFormat}{a character string specifying the output format of the downloaded data. One of 'csv' or 'json'}

\item{download_method}{a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
either a data.table (if outputFormat is 'csv') or a nested list (if outputFormat is 'json')
}
\description{
Get a list of ICESat-2 tracks using a bounding box as input
}
\examples{

\dontrun{

require(IceSat2R)
require(magrittr)
require(sf)

sf::sf_use_s2(use_s2 = FALSE)

#..........................................................
# In case we are interested to find the ICESat-2 tracks for
#     - a specific area (or bounding box)
#     - a specific time interval (close to September 2021)
#..........................................................

#...............
# Ice sheet area
#...............

data(ne_10m_glaciated_areas)

ice_sheet = 'Kluane Ice Cap'

ice_sheet_geom = ne_10m_glaciated_areas \%>\%
  subset(!is.na(name)) \%>\%
  subset(name == ice_sheet)

ice_sheet_geom

#..............
# time interval
#..............

approx_date_start = "2021-09-05"
approx_date_end = "2021-09-20"

res_rgt_many = time_specific_orbits(date_from = approx_date_start,
                                    date_to = approx_date_end,
                                    RGT_cycle = NULL,
                                    download_method = 'curl',
                                    threads = parallel::detectCores(),
                                    verbose = TRUE)

res_rgt_many

# table(as.Date(res_rgt_many$Date_time))

#..............................................
# create the bounding box of the ice
# sheet so that I use the same area in the
# 'st_intersects()' and 'getTracks()' functions
# ('getTracks()' takes a bounding box as input)
#..............................................

bbx_ice_sheet = sf::st_bbox(obj = ice_sheet_geom)
sf_obj_ice_sheet = sf::st_as_sfc(bbx_ice_sheet)

#.........................................
# intersection of the Ice Sheet with
# the tracks for the specified time period
#.........................................

res_inters = sf::st_intersects(x = sf::st_geometry(sf_obj_ice_sheet),
                               y = sf::st_geometry(res_rgt_many),
                               sparse = TRUE)

#...........................................
# one or more intersected RGT's for the area
#...........................................

df_inters = data.frame(res_inters)
rgt_subs = res_rgt_many[df_inters$col.id, , drop = FALSE]


#.......................................................
# Keep the Date and the bounding box to
# verify with the 'getTracks()' function
# an alternative is to use the "verify_RGTs()" function
#.......................................................

for (item in 1:nrow(rgt_subs)) {

  dat_item = rgt_subs[item, , drop = F]
  Date = as.Date(dat_item$Date_time)

  op_tra = getTracks(minx = as.numeric(bbx_ice_sheet['xmin']),
                               miny = as.numeric(bbx_ice_sheet['ymin']),
                               maxx = as.numeric(bbx_ice_sheet['xmax']),
                               maxy = as.numeric(bbx_ice_sheet['ymax']),
                               date = as.character(Date),
                               outputFormat = 'csv',
                               download_method = 'curl',
                               verbose = FALSE)

  date_obj = dat_item$Date_time
  tim_rgt = glue::glue("Date: {date_obj} Time specific RGT: '{dat_item$RGT}'")

  if (nrow(op_tra) > 0) {
    iter_op_trac = paste(op_tra$track, collapse = ', ')
    cat(glue::glue("{tim_rgt}  OpenAltimetry: '{iter_op_trac}'"), '\n')
  }
  else {
    cat(glue::glue("{tim_rgt} without an OpenAltimetry match!"), '\n')
  }
}

}
}
\references{
https://openaltimetry.earthdatacloud.nasa.gov

https://nsidc.org/data/icesat-2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{overall_mission_orbits}
\alias{overall_mission_orbits}
\title{Overall Mission Orbits}
\usage{
overall_mission_orbits(
  orbit_area,
  download_method = "curl",
  threads = 1,
  verbose = FALSE
)
}
\arguments{
\item{orbit_area}{a character string specifying the nominal mission orbits and beam locations. It can be one of 'antarctic', 'arctic', 'western_hemisphere' or 'eastern_hemisphere'}

\item{download_method}{a character string specifying the download method. Corresponds to the 'method' parameter of the 'utils::download.file()' function. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'}

\item{threads}{an integer that specifies the number of threads to use in parallel when processing the data}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
an 'sf' object of multiple tracks (see the 'LAYER' column of the output object)
}
\description{
This function allows the user to view information of the nominal mission orbits and beam locations: "The processed files have 7 tracks per
orbit: one for each of the six beams of ICESat-2, and the seventh for the Reference Ground Track (RGT). The RGT is an imaginary line
through the six-beam pattern that is handy for getting a sense of where the orbits fall on Earth, and which the mission uses to point the
observatory. However, the six tracks for the six beams are our best estimate of where the beams will actually fall on Earth's surface."
}
\examples{
\dontrun{

require(IceSat2R)

res_orb <- overall_mission_orbits(
  orbit_area = "antarctic",
  download_method = "curl",
  threads = 1,
  verbose = TRUE
)
str(res_orb)
}
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{available_nominal_orbits}
\alias{available_nominal_orbits}
\title{Nominal mission orbits}
\usage{
available_nominal_orbits(
  orbit_area = NULL,
  technical_specs_url = "https://icesat-2.gsfc.nasa.gov/science/specs",
  verbose = FALSE
)
}
\arguments{
\item{orbit_area}{either NULL or a character string specifying the earth partition to use, it can be one of 'antarctic', 'arctic', 'western_hemisphere' and 'eastern_hemisphere'}

\item{technical_specs_url}{a character string specifying the technical specs website}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
a list object with the available nominal orbits
}
\description{
This function allows the user to view the nominal orbits (all or a selection)
}
\examples{

require(IceSat2R)

# .............................
# all available nominal orbits
# .............................

nomin_orb <- available_nominal_orbits(verbose = TRUE)
nomin_orb

# ........................
# specific nominal orbits
# ........................

nomin_orb <- available_nominal_orbits(
  orbit_area = "arctic",
  verbose = TRUE
)
nomin_orb

}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{vsi_kml_from_zip}
\alias{vsi_kml_from_zip}
\title{Utilizing Virtual File Systems (vsi) to extract the .kml from the .zip file}
\usage{
vsi_kml_from_zip(
  icesat_rgt_url,
  download_zip = FALSE,
  download_method = "curl",
  verbose = FALSE
)
}
\arguments{
\item{icesat_rgt_url}{a character string specifying the input .zip URL}

\item{download_zip}{a boolean. If TRUE the .zip file will be first downloaded and then the .kml files will be returned, otherwise the 'gdalinfo' function will be used as input to the R 'system2()' function to read the .kml files without downloading the .zip file. The 'gdalinfo' command requires that the user has configured GDAL properly.}

\item{download_method}{a character string specifying the download method. Corresponds to the 'method' parameter of the 'utils::download.file()' function. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
an one column data.table of the output files
}
\description{
This function returns the '.kml' and '.kmz' files in form of virtual file paths. Moreover, the user has the option to download these files.
}
\examples{
\dontrun{

require(IceSat2R)

URL <- "https://icesat-2.gsfc.nasa.gov/sites/default/files/page_files/arcticallorbits.zip"

# ..................................
# without downloading the .zip file
# ..................................

res_out <- vsi_kml_from_zip(
  icesat_rgt_url = URL,
  download_zip = FALSE,
  download_method = "curl",
  verbose = TRUE
)
str(res_out)


# .............................
# by downloading the .zip file
# .............................

res_out <- vsi_kml_from_zip(
  icesat_rgt_url = URL,
  download_zip = TRUE,
  download_method = "curl",
  verbose = TRUE
)
str(res_out)
}
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs

https://gdal.org/user/virtual_file_systems.html
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{revisit_time_RGTs}
\alias{revisit_time_RGTs}
\title{Revisit Time Reference Ground Tracks and Dates}
\usage{
revisit_time_RGTs(RGT_cycle = NULL, complete_date_sequence = FALSE)
}
\arguments{
\item{RGT_cycle}{either NULL or a character string specifying a single RGT (Reference Ground Track) as determined by the output of the 'available_RGTs(only_cycle_names = TRUE)' function. If NULL then all available Data will be returned.}

\item{complete_date_sequence}{a boolean. If TRUE a complete sequence of Dates will be returned, otherwise only the 'minimum' and 'maximum' Dates.}
}
\value{
a list object with the available orbit files, dates and date sequence lengths
}
\description{
This function shows the information of the 'available_RGTs' function and additionally it returns the .zip (kmz files) of all laser
tracks over each 91-day repeat period (revisit time). Note that the locations and times are estimates, but should be correct to
within a few minutes in time and better than 100m in the predicted locations.
}
\details{
ICESat-2 was in safe-hold from June 26 through July 9, 2019. ATLAS was off during this time, so data was not collected or pointed to the reference ground track.
}
\examples{

require(IceSat2R)

# .......................................................
# receive all orbit files, dates and length of sequences
# .......................................................

rev_all <- revisit_time_RGTs(RGT_cycle = NULL, complete_date_sequence = TRUE)
rev_all

# ...................................................
# observe and choose one of the available RGT-cycles
# ...................................................

avail_cycles <- available_RGTs(
  only_cycle_names = TRUE,
  verbose = TRUE
)
avail_cycles

# .....................................
# receive results for a specific cycle
# .....................................

rev_cycle <- revisit_time_RGTs(RGT_cycle = "RGT_cycle_1", complete_date_sequence = FALSE)
rev_cycle
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny_app.R
\name{leaflet_base_map}
\alias{leaflet_base_map}
\title{a leaflet base map}
\usage{
leaflet_base_map(
  sf_obj = NULL,
  leaflet_provider_base,
  leaflet_provider_secondary,
  overlay_group = "IceSat2 1-degree",
  overlay_polylines = "Polylines",
  width = NULL,
  height = NULL,
  popup = NULL,
  collapse_layer_control = TRUE
)
}
\arguments{
\item{sf_obj}{a simple features object which must include the columns 'date' and 'size_MB' if the popup parameter is set to TRUE}

\item{leaflet_provider_base}{a leaflet provider object}

\item{leaflet_provider_secondary}{a leaflet provider object}

\item{overlay_group}{a character string specifying the overlay group}

\item{overlay_polylines}{a character string specifying the Polylines group}

\item{width}{a numeric value specifying the width of the leaflet map}

\item{height}{a numeric value specifying the height of the leaflet map}

\item{popup}{a boolean specifying if the popup should be shown or not}

\item{collapse_layer_control}{a boolean specifying if the layer-control should be collapsed or not}
}
\value{
a leaflet object (map)
}
\description{
a leaflet base map
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/API_utils.R
\name{switch_abb}
\alias{switch_abb}
\title{Convert an abbreviated month name to a numeric value}
\usage{
switch_abb(month)
}
\arguments{
\item{month}{a character string specifying one of the twelve months, i.e. ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")}
}
\value{
a character string which will correspond to the numeric value of the input month, i.e. ("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12")
}
\description{
Convert an abbreviated month name to a numeric value
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ne_10m_glaciated_areas}
\alias{ne_10m_glaciated_areas}
\title{Natural Earth 10m Glaciated Areas (1:10 million scale)}
\format{
An object of class \code{sf} (inherits from \code{data.frame}) with 68 rows and 6 columns.
}
\usage{
data(ne_10m_glaciated_areas)
}
\description{
Polygons derived from DCW (Digital Chart of the World), except for Antarctica derived from MOA. Includes name attributes for major polar glaciers.
Filtering has aggregated some minor glaciers and eliminated others from the original DCW data.
}
\details{
Issues: Needs scale rank attributes

Note: The original DCW data was collected decades ago. Since then many mountain glaciers have either disappeared or
diminished in size. This data theme is deliberately called "glaciated areas" instead of "glaciers" to reflect the
changeable character of glacier extents.
}
\examples{

require(IceSat2R)
require(sf)

data(ne_10m_glaciated_areas)
}
\references{
https://www.naturalearthdata.com/downloads/10m-physical-vectors/10m-glaciated-areas/
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/API_utils.R
\name{verify_RGTs}
\alias{verify_RGTs}
\title{Verification of the Reference Ground Tracks (RGTs)}
\usage{
verify_RGTs(nsidc_rgts, bbx_aoi, verbose = FALSE, ...)
}
\arguments{
\item{nsidc_rgts}{a data.frame, data.table or tibble object that includes the columns 'Date_time' and 'RGT'}

\item{bbx_aoi}{a named numeric vector or an sf-bbox object with names 'xmin', 'ymin', 'xmax', 'ymax'}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}

\item{...}{further parameters for the getTracks function}
}
\value{
a 'data.table' object where it is possible that the number of the OpenAltimetry RGTs is higher compared to the NSIDC RGTs
}
\description{
This function allows the user to verify the NSIDC extracted RGTs with the corresponding OpenAltimetry using the same Dates
}
\examples{

\dontrun{

require(IceSat2R)

rgts = data.table::setDT(list(RGT = c(1251L, 1252L, 1260L, 1267L, 1275L),
                              Date_time = c("2020-12-15", "2020-12-15",
                              "2020-12-15", "2020-12-16", "2020-12-16")))
bbx = c(xmin = -53.108876, ymin = 60.119614, xmax = -19.203521, ymax = 80.793117)

dtbl = verify_RGTs(nsidc_rgts = rgts, bbx_aoi = bbx, verbose = TRUE)
dtbl

# split by Date to observe RGTs by date

split(dtbl, by = 'Date_time')

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny_app.R
\docType{class}
\name{select_aoi_global_grid}
\alias{select_aoi_global_grid}
\title{R6 Class to Select an Area of Interest (AOI) from a Global Grid}
\usage{
# init <- select_aoi_global_grid$new()
}
\description{
The 'select_aoi_global_grid' class allows the user to select an Area of Interest (AOI) (see the examples section for all available options)
}
\examples{

\dontrun{

require(IceSat2R)
require(magrittr)

#........................................
# 1st. Option: Select an AOI from the map
#........................................

init = select_aoi_global_grid$new(area_of_interest = NULL,
                                  verbose = TRUE)
init$draw_edit_aoi()
sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
sf_obj


#...............................................
# observe the available countries and continents
# using the 'rnaturalearth' R package
#...............................................

cntr_cnt = rnaturalearth::ne_countries(scale = 110,
                                       type = 'countries',
                                       returnclass = 'sf')

cntr_cnt = cntr_cnt[, c('sovereignt', 'continent')]

# sort(cntr_cnt$sovereignt)
# sort(unique(cntr_cnt$continent))


#.......................................
# 2nd. Option: Select a 'country' as AOI   (5-degrees query)
#.......................................

init = select_aoi_global_grid$new(area_of_interest = 'Antarctica',
                                  verbose = TRUE)

init$draw_edit_aoi(degrees = 5.0, square_geoms = TRUE)
sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
sf_obj


#.........................................
# 3rd. Option: Select a 'continent' as AOI   (1-degree query)
#.........................................

init = select_aoi_global_grid$new(area_of_interest = 'North America',
                                  verbose = TRUE)

init$draw_edit_aoi(degrees = 1.0, square_geoms = TRUE)
sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
sf_obj


#.........................................
# 4th. Option: Use a bounding box as input  ('Greenland Ice Sheet')
#.........................................

data(ne_10m_glaciated_areas)

dat_bbx = ne_10m_glaciated_areas \%>\%
  subset(!is.na(name)) \%>\%
  subset(name == "Greenland Ice Sheet") \%>\%
  sf::st_bbox(crs = 4326)

dat_bbx

init = select_aoi_global_grid$new(area_of_interest = dat_bbx,
                                  verbose = TRUE)

init$draw_edit_aoi(degrees = 1.0, square_geoms = TRUE)
sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
sf_obj


#......................................................
# 5th. Option: Create a global hexagonal 5-degrees grid
#......................................................

bbx_global = c(xmin = -180, xmax = 180, ymin = -90, ymax = 90)

init = select_aoi_global_grid$new(area_of_interest = bbx_global,
                                  verbose = TRUE)

init$draw_edit_aoi(degrees = 5.0, square_geoms = FALSE)
sf_obj = init$selected_areas_global_grid(plot_data = TRUE)
sf_obj

}
}
\references{
https://github.com/r-spatial/mapedit/blob/master/experiments/gadget_draw2.R
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{area_of_interest}}{an R object (character string, vector)}

\item{\code{leaflet_provider_base}}{a leaflet provider object}

\item{\code{leaflet_provider_secondary}}{a leaflet provider object}

\item{\code{crs_value}}{a numeric value}

\item{\code{use_s2}}{a boolean}

\item{\code{verbose}}{a boolean}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-select_aoi_global_grid-new}{\code{select_aoi_global_grid$new()}}
\item \href{#method-select_aoi_global_grid-draw_edit_internal}{\code{select_aoi_global_grid$draw_edit_internal()}}
\item \href{#method-select_aoi_global_grid-draw_edit_aoi}{\code{select_aoi_global_grid$draw_edit_aoi()}}
\item \href{#method-select_aoi_global_grid-selected_areas_global_grid}{\code{select_aoi_global_grid$selected_areas_global_grid()}}
\item \href{#method-select_aoi_global_grid-selected_aoi_sf}{\code{select_aoi_global_grid$selected_aoi_sf()}}
\item \href{#method-select_aoi_global_grid-clone}{\code{select_aoi_global_grid$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-select_aoi_global_grid-new"></a>}}
\if{latex}{\out{\hypertarget{method-select_aoi_global_grid-new}{}}}
\subsection{Method \code{new()}}{
Initialization method for the 'select_aoi_global_grid' R6 class
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{select_aoi_global_grid$new(
  area_of_interest = NULL,
  leaflet_provider_base = leaflet::providers$CartoDB.Positron,
  leaflet_provider_secondary = leaflet::providers$Esri.WorldImagery,
  crs_value = 4326,
  use_s2 = FALSE,
  verbose = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{area_of_interest}}{either NULL (which allows the user to draw the area of interest on the map) or a character string (i.e. a 'country' or a 'continent') or a named bounding box vector (such as c(xmin = 16.1, xmax = 16.6, ymax = 48.6, ymin = 47.9)). The 'countries' and 'continents' can be extracted from the "rnaturalearth::ne_countries(scale = 110, type = 'countries', returnclass = 'sf')" function and specifically the columns: 'sovereignt' and 'continent'}

\item{\code{leaflet_provider_base}}{a leaflet provider object}

\item{\code{leaflet_provider_secondary}}{a leaflet provider object}

\item{\code{crs_value}}{a value. The coordinate reference system. The default value of the crs is 4326}

\item{\code{use_s2}}{a boolean. If TRUE, use the s2 spherical geometry package for geographical coordinate operations (see the documentation of the 'sf::sf_use_s2()' function for more information)}

\item{\code{verbose}}{a boolean. If TRUE then information will be printed in the console}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-select_aoi_global_grid-draw_edit_internal"></a>}}
\if{latex}{\out{\hypertarget{method-select_aoi_global_grid-draw_edit_internal}{}}}
\subsection{Method \code{draw_edit_internal()}}{
Internal Shiny application to visualize the selected area
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{select_aoi_global_grid$draw_edit_internal(lft_map)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{lft_map}}{a leaflet map}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-select_aoi_global_grid-draw_edit_aoi"></a>}}
\if{latex}{\out{\hypertarget{method-select_aoi_global_grid-draw_edit_aoi}{}}}
\subsection{Method \code{draw_edit_aoi()}}{
Allows to view the Global Grid on the map including a popup that shows the Area of each grid rectangle (or hexagon). The user can then select an Area of Interest (AOI)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{select_aoi_global_grid$draw_edit_aoi(degrees = 1, square_geoms = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{degrees}}{a numeric value. This can be either 1.0 or 5.0 to allow queries to the 'OpenAltimetry' API}

\item{\code{square_geoms}}{a boolean. If FALSE then a hexagonal grid will be created}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-select_aoi_global_grid-selected_areas_global_grid"></a>}}
\if{latex}{\out{\hypertarget{method-select_aoi_global_grid-selected_areas_global_grid}{}}}
\subsection{Method \code{selected_areas_global_grid()}}{
Takes the selected Area(s) of Interest (AOI) from the draw_edit_aoi() method and returns a simple features object with the corresponding n-degree grid cells
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{select_aoi_global_grid$selected_areas_global_grid(plot_data = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{plot_data}}{a boolean specifying if the selected from the user AOI's and the corresponding grid cells should be plotted side by side. If FALSE then only the simple features object will be returned. If TRUE and the initial 'area_of_interest' parameter is NULL then a single plot will be displayed.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
either an 'sfc' object (if the initial 'area_of_interest' parameter is NULL) or an 'sf' object consisting of the n-degree grid cells
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-select_aoi_global_grid-selected_aoi_sf"></a>}}
\if{latex}{\out{\hypertarget{method-select_aoi_global_grid-selected_aoi_sf}{}}}
\subsection{Method \code{selected_aoi_sf()}}{
Returns the selected area of interest (AOI) by the user in form of an 'sfc' object
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{select_aoi_global_grid$selected_aoi_sf()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an 'sfc' object
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-select_aoi_global_grid-clone"></a>}}
\if{latex}{\out{\hypertarget{method-select_aoi_global_grid-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{select_aoi_global_grid$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_Atlas_data.R
\name{get_atlas_data}
\alias{get_atlas_data}
\title{Get ICESat-2 ATLAS data for a specific Date}
\usage{
get_atlas_data(
  minx,
  miny,
  maxx,
  maxy,
  date,
  trackId,
  beamName = NULL,
  product = "atl03",
  client = "portal",
  photonConfidence = NULL,
  sampling = FALSE,
  outputFormat = "csv",
  file_path_zip = NULL,
  download_method = "curl",
  verbose = FALSE
)
}
\arguments{
\item{minx}{the 'minx' parameter of the bounding box}

\item{miny}{the 'miny' parameter of the bounding box}

\item{maxx}{the 'maxx' parameter of the bounding box}

\item{maxy}{the 'maxy' parameter of the bounding box}

\item{date}{a character string specifying the Data collection date in the format 'yyyy-MM-dd' (such as '2020-01-01')}

\item{trackId}{an integer specifying the 'Reference ground track ID' (see the examples section on how to come to the 'trackId' and to a bounding box of 1 or 5 degrees globally)}

\item{beamName}{either NULL (if the 'product' parameter is not equal to 'atl03' which means data for all 6 beams will be returned) or a character string specifying the Beam Name, which can be one of the 'gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l' or 'gt3r'}

\item{product}{a character string specifying the input data product (default value is 'atl03'). It can be one of 'atl03', 'atl06', 'atl07', 'atl08', 'atl10', 'atl12' or 'atl13'}

\item{client}{a character string specifying the 'Referring client'. Can be one of 'portal' or 'jupyter' (default is 'portal')}

\item{photonConfidence}{either NULL or a character vector specifying the 'Confidence levels of the photons (can be one or more). If NULL (default) all photon data will be returned. The available options are 'na', 'noise', 'buffer', 'low', 'medium', 'high'}

\item{sampling}{a boolean. If TRUE a sampling rate of 1/1000 will be used, otherwise all data will be returned (default is FALSE)}

\item{outputFormat}{a character string specifying the output format of the downloaded data. One of 'csv', 'json' or 'zip'}

\item{file_path_zip}{either NULL or a character string specifying a valid path to the output .zip file. This parameter will normally be a valid path if the 'outputFormat' parameter is set to 'zip'. If it's NULL and the 'outputFormat' parameter is 'zip' then the downloaded '.zip' file will be converted and returned as a data.table object}

\item{download_method}{a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
if the 'file_path_zip' parameter is NULL it returns either a data.table (if outputFormat is 'csv') or a nested list (if outputFormat is 'json') else the file path where the .zip file is saved. In case that the 'outputFormat' is set to 'zip' and the 'file_path_zip' parameter to NULL then a data.table will be returned.
}
\description{
This function allows the user to download ICESat-2 ATLAS Product data for a specific date, bounding box, track and beam.
}
\details{
\describe{
 \item{'atl03', \emph{Global Geolocated Photon Data} (Version 4, Requests are limited to 1x1 degree spatial bounding box selection \emph{unless "sampling" is set to true})}{This data set \emph{(ATL03)} contains height above the WGS 84 ellipsoid (ITRF2014 reference frame), latitude, longitude, and time for all photons downlinked by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory. The ATL03 product was designed to be a single source for all photon data and ancillary information needed by higher-level ATLAS/ICESat-2 products. As such, it also includes spacecraft and instrument parameters and ancillary data not explicitly required for ATL03}
 \item{'atl06', \emph{Land Ice Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL06)} provides geolocated, land-ice surface heights (above the WGS 84 ellipsoid, ITRF2014 reference frame), plus ancillary parameters that can be used to interpret and assess the quality of the height estimates. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl07', \emph{Sea Ice Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{The data set \emph{(ATL07)} contains along-track heights for sea ice and open water leads (at varying length scales) relative to the WGS84 ellipsoid (ITRF2014 reference frame) after adjustment for geoidal and tidal variations, and inverted barometer effects. Height statistics and apparent reflectance are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl08', \emph{Land and Vegetation Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL08)} contains along-track heights above the WGS84 ellipsoid (ITRF2014 reference frame) for the ground and canopy surfaces. The canopy and ground surfaces are processed in fixed 100 m data segments, which typically contain more than 100 signal photons. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl10', \emph{Sea Ice Freeboard} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL10)} contains estimates of sea ice freeboard, calculated using three different approaches. Sea ice leads used to establish the reference sea surface and descriptive statistics used in the height estimates are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl12', \emph{Ocean Surface Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL12)} contains along-track sea surface heights at variable length scales over cloud-free regions. Estimates of height distributions, surface roughness, surface slope, and apparent reflectance are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl13', \emph{Inland Water Surface Height} (Version 4, Requests are limited to 5x5 degree spatial bounding box selection)}{This data set \emph{(ATL13)} contains along-track water surface heights and descriptive statistics for inland water bodies. Water bodies include lakes, reservoirs, bays, and estuaries. Descriptive statistics include along-track surface slope (where data permit), mean and standard deviation, subsurface signal (532 nm) attenuation, wave height, and coarse depth to bottom topography}
 }
}
\examples{

\dontrun{

require(IceSat2R)

#............................................................
# observe the available countries and continents using the
# 'rnaturalearth' R package to perform a query using map-edit
#............................................................

cntr_cnt = rnaturalearth::ne_countries(scale = 110,
                                       type = 'countries',
                                       returnclass = 'sf')

cntr_cnt = cntr_cnt[, c('sovereignt', 'continent')]

# sort(cntr_cnt$sovereignt)
# sort(unique(cntr_cnt$continent))


#............................
# Select a 'continent' as AOI   (5-degree query)
#............................

init = select_aoi_global_grid$new(area_of_interest = 'Oceania',
                                  verbose = TRUE)

init$draw_edit_aoi(degrees = 5.0, square_geoms = TRUE)
sf_obj = init$selected_areas_global_grid(plot_data = FALSE)
sf_obj

#.....................................................
# I drew a bounding box close to 'Mount Hagen Papua'
# inside one of the 5-degreee cells (after zooming-in)
#.....................................................

# mapview::mapview(sf_obj, legend = F)
# sf::st_as_text(sf::st_geometry(sf_obj))

#................................................
# to reproduce the results without selecting
# an area based on the 'draw_edit_aoi()' function
#................................................

# plg = "POLYGON ((140 -6.641235, 145 -6.641235, 145 -1.641235, 140 -1.641235, 140 -6.641235))"
# sf_obj = sf::st_as_sfc(plg, crs = 4326)


#....................................................
# first we find available ICESat-2 track-ID's
# and Dates (time interval) from a specific RGT cycle
#....................................................

approx_date_start = "2021-02-01"
approx_date_end = "2021-02-15"
# 'RGT_cycle_10'

res_rgt_many = time_specific_orbits(date_from = approx_date_start,
                                    date_to = approx_date_end,
                                    RGT_cycle = NULL,
                                    download_method = 'curl',
                                    threads = parallel::detectCores(),
                                    verbose = TRUE)
res_rgt_many

#.........................................................
# then we create the bounding box of the selected area
# and proceed to the intersection with the computed RGT's
# ( include the bounding box for reproducibility )
#.........................................................

bbx_aoi = sf::st_bbox(obj = sf_obj)
# c(xmin = 140, ymin = -6.641235, xmax = 145, ymax = -1.641235)

sf_obj_bbx = sf::st_as_sfc(bbx_aoi)

res_inters = sf::st_intersects(x = sf_obj_bbx,
                               y = sf::st_geometry(res_rgt_many),
                               sparse = TRUE)
#.....................
# matched (RGT) tracks
#.....................

df_inters = data.frame(res_inters)

if (nrow(df_inters) == 0) {
  stop("There is no intersection between the specified AOI and the RGTs!")
}


rgt_subs = res_rgt_many[df_inters$col.id, , drop = FALSE]
rgt_subs


#..........................................
# find out which of the time specific RGT's
# match the OpenAltimetry Track-ID's
#..........................................

dtbl_rgts = verify_RGTs(nsidc_rgts = rgt_subs,
                        bbx_aoi = bbx_aoi,
                        verbose = TRUE)

#...................................
# we will iterate over the available:
#    - Dates
#    - trackId's
#    - product's
# to gather the up to 5-degree data
#..................................

dates_iters = unique(dtbl_rgts$Date_time)
RGTs_iters = unique(dtbl_rgts$RGT_NSIDC)
prods_5_degrs = c('atl06', 'atl07', 'atl08', 'atl10', 'atl12', 'atl13')

dat_out = logs_out = list()

for (idx in seq_along(dates_iters)) {

  date_i = dates_iters[idx]
  rgt_i = RGTs_iters[idx]

  for (prod_i in prods_5_degrs) {

    name_iter = glue::glue("{date_i}_{rgt_i}_{prod_i}")
    cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'  Product: '{prod_i}'"), '\n')

    iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
                              miny = as.numeric(bbx_aoi['ymin']),
                              maxx = as.numeric(bbx_aoi['xmax']),
                              maxy = as.numeric(bbx_aoi['ymax']),
                              date = date_i,
                              trackId = rgt_i,
                              product = prod_i,
                              client = 'portal',
                              outputFormat = 'csv',
                              verbose = FALSE)

    iter_logs = list(Date = date_i,
                     RGT = rgt_i,
                     Product = prod_i,
                     N_rows = nrow(iter_dat))

    logs_out[[name_iter]] = data.table::setDT(iter_logs)
    dat_out[[name_iter]] = iter_dat
  }
}

#.........................................
# each sublist corresponds to a different
# parameter setting (Date, Track, Product)
#.........................................

dat_out

#.....
# Logs  (including the number of rows for each parameter setting)
#.....

dtbl_logs = data.table::rbindlist(logs_out)
dtbl_logs = subset(dtbl_logs, N_rows > 0)
dtbl_logs = dtbl_logs[order(dtbl_logs$N_rows, decreasing = T), ]
dtbl_logs

#.............................................
# The Products 'atl08' and 'atl13' have data
# for the Dates and RGT's of the selected area
#.............................................

unique(dtbl_logs$Product)
# c("atl06" "atl08" "atl13" "atl12")

#................
# RGT's with data
#................

unique(dtbl_logs$RGT)
# c(627, 756, 688, 619, 817)

#................
# Dates with Data
#................

unique(dtbl_logs$Date)
# c("2021-02-03", "2021-02-11", "2021-02-07", "2021-02-02", "2021-02-15")


#============
# ATL03 Data
#============

................................................
# the default timeout is 60 and we set it to 600
#...............................................

options(timeout = 600)
print(getOption('timeout'))


# we skip the interactive selection of the 1 degree grid
plg = "POLYGON ((142 -34.64124, 143 -34.64124, 143 -33.64124, 142 -33.64124, 142 -34.64124))"
sf_obj_atl03 = sf::st_as_sfc(plg, crs = 4326)

# we update the bounding box based on the AOI
bbx_aoi = sf::st_bbox(obj = sf_obj_atl03)
# c(xmin = 142, ymin = -34.64124, xmax = 143, ymax = -33.64124)

sf_obj_bbx = sf::st_as_sfc(bbx_aoi)

res_inters = sf::st_intersects(x = sf_obj_bbx,
                               y = sf::st_geometry(res_rgt_many),
                               sparse = TRUE)

df_inters = data.frame(res_inters)

if (nrow(df_inters) == 0) {
  stop("There is no intersection between the specified AOI and the RGTs!")
}


rgt_subs = res_rgt_many[df_inters$col.id, , drop = FALSE]
rgt_subs

dtbl_rgts = verify_RGTs(nsidc_rgts = rgt_subs,
                        bbx_aoi = bbx_aoi,
                        verbose = TRUE)

comp_cas = complete.cases(dtbl_rgts)
dtbl_rgts = dtbl_rgts[comp_cas, , drop = F]


dates_iters = unique(dtbl_rgts$Date_time)
RGTs_iters = unique(dtbl_rgts$RGT_NSIDC)
prods_1_degrs = c('atl03')

#...........................
# use all beams with 'atl03'
#...........................

dat_out = logs_out = list()

for (idx in seq_along(dates_iters)) {
  
  date_i = dates_iters[idx]
  rgt_i = RGTs_iters[idx]
  
  name_iter = glue::glue("{date_i}_{rgt_i}")
  cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'"), '\n')
  
  iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
                            miny = as.numeric(bbx_aoi['ymin']),
                            maxx = as.numeric(bbx_aoi['xmax']),
                            maxy = as.numeric(bbx_aoi['ymax']),
                            date = date_i,
                            beamName = NULL,
                            trackId = rgt_i,
                            product = prods_1_degrs,
                            client = 'portal',
                            outputFormat = 'csv',
                            verbose = FALSE)
  
  iter_logs = list(Date = date_i,
                   RGT = rgt_i,
                   Product = prods_1_degrs,
                   N_rows = nrow(iter_dat))
  
  logs_out[[name_iter]] = data.table::setDT(iter_logs)
  dat_out[[name_iter]] = iter_dat
}

dat_out
unique(dtbl_logs$Product)
unique(dat_out[[1]]$beam)
unique(dat_out[[1]]$`confidence code`)

#..................................................
# use 2 beams with 'atl03' (a vector of characters) 
#..................................................

dat_out = logs_out = list()

for (idx in seq_along(dates_iters)) {
  
  date_i = dates_iters[idx]
  rgt_i = RGTs_iters[idx]
  
  name_iter = glue::glue("{date_i}_{rgt_i}")
  cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'"), '\n')
  
  iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
                            miny = as.numeric(bbx_aoi['ymin']),
                            maxx = as.numeric(bbx_aoi['xmax']),
                            maxy = as.numeric(bbx_aoi['ymax']),
                            date = date_i,
                            beamName = c("gt1l", "gt1r"),
                            trackId = rgt_i,
                            product = prods_1_degrs,
                            client = 'portal',
                            outputFormat = 'csv',
                            verbose = FALSE)
  
  iter_logs = list(Date = date_i,
                   RGT = rgt_i,
                   Product = prods_1_degrs,
                   N_rows = nrow(iter_dat))
  
  logs_out[[name_iter]] = data.table::setDT(iter_logs)
  dat_out[[name_iter]] = iter_dat
}

dat_out
unique(dtbl_logs$Product)
unique(dat_out[[1]]$beam)


#.............................................
# use 1 beam with 'atl03' (a character string) 
#.............................................

dat_out = logs_out = list()

for (idx in seq_along(dates_iters)) {
  
  date_i = dates_iters[idx]
  rgt_i = RGTs_iters[idx]
  
  name_iter = glue::glue("{date_i}_{rgt_i}")
  cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'"), '\n')
  
  iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
                            miny = as.numeric(bbx_aoi['ymin']),
                            maxx = as.numeric(bbx_aoi['xmax']),
                            maxy = as.numeric(bbx_aoi['ymax']),
                            date = date_i,
                            beamName = "gt1l",
                            trackId = rgt_i,
                            product = prods_1_degrs,
                            client = 'portal',
                            outputFormat = 'csv',
                            verbose = FALSE)
  
  iter_logs = list(Date = date_i,
                   RGT = rgt_i,
                   Product = prods_1_degrs,
                   N_rows = nrow(iter_dat))
  
  logs_out[[name_iter]] = data.table::setDT(iter_logs)
  dat_out[[name_iter]] = iter_dat
}

dat_out
unique(dtbl_logs$Product)
unique(dat_out[[1]]$beam)


#..............................................
# use all beams with 'atl03' & photonConfidence
#..............................................

dat_out = logs_out = list()

for (idx in seq_along(dates_iters)) {
  
  date_i = dates_iters[idx]
  rgt_i = RGTs_iters[idx]
  
  name_iter = glue::glue("{date_i}_{rgt_i}")
  cat(glue::glue("Date: '{date_i}'  RGT: '{rgt_i}'"), '\n')
  
  iter_dat = get_atlas_data(minx = as.numeric(bbx_aoi['xmin']),
                            miny = as.numeric(bbx_aoi['ymin']),
                            maxx = as.numeric(bbx_aoi['xmax']),
                            maxy = as.numeric(bbx_aoi['ymax']),
                            date = date_i,
                            beamName = NULL,
                            photonConfidence = c('buffer', 'low'),
                            trackId = rgt_i,
                            product = prods_1_degrs,
                            client = 'portal',
                            outputFormat = 'csv',
                            verbose = FALSE)
  
  iter_logs = list(Date = date_i,
                   RGT = rgt_i,
                   Product = prods_1_degrs,
                   N_rows = nrow(iter_dat))
  
  logs_out[[name_iter]] = data.table::setDT(iter_logs)
  dat_out[[name_iter]] = iter_dat
}

dat_out
unique(dtbl_logs$Product)
unique(dat_out[[1]]$beam)
unique(dat_out[[1]]$`confidence code`)

}
}
\references{
https://openaltimetry.earthdatacloud.nasa.gov

https://nsidc.org/data/icesat-2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/elapsed_time.R
\name{inner_elapsed_time}
\alias{inner_elapsed_time}
\title{inner function of 'compute_elapsed_time'}
\usage{
inner_elapsed_time(secs, estimated = FALSE)
}
\arguments{
\item{secs}{a numeric value specifying the seconds}

\item{estimated}{a boolean. If TRUE then the output label becomes the 'Estimated time'}
}
\value{
a character string showing the estimated or elapsed time
}
\description{
inner function of 'compute_elapsed_time'
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/API_utils.R
\name{get_URL_data}
\alias{get_URL_data}
\title{Get the data based on the API URL}
\usage{
get_URL_data(
  URL,
  outputFormat = "csv",
  download_method = "curl",
  file_path_zip = NULL,
  verbose = FALSE
)
}
\arguments{
\item{URL}{a character string specifying the API command}

\item{outputFormat}{a character string specifying the output format of the downloaded data. One of 'csv', 'json' or 'zip'}

\item{download_method}{a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function}

\item{file_path_zip}{either NULL or a character string specifying a valid path to the output .zip file. This parameter will normally be a valid path if the 'outputFormat' parameter is set to 'zip'. If it's NULL and the 'outputFormat' parameter is 'zip' then the downloaded '.zip' file will be converted and returned as a data.table object}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
if the 'file_path_zip' parameter is NULL it returns either a data.table (if outputFormat is 'csv') or a nested list (if outputFormat is 'json') else the file path where the .zip file is saved. In case that the 'outputFormat' is set to 'zip' and the 'file_path_zip' parameter to NULL then a data.table will be returned.
}
\description{
Get the data based on the API URL
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{latest_orbits}
\alias{latest_orbits}
\title{Extraction of the url from the Technical Specification Website}
\usage{
latest_orbits(
  technical_specs_url = "https://icesat-2.gsfc.nasa.gov/science/specs",
  verbose = FALSE
)
}
\arguments{
\item{technical_specs_url}{a character string specifying the technical specs website}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
a 'data.table' object
}
\description{
This function allows the user to view the latest 'Nominal' and 'Time Specfic' orbit metadata (Url, Reference Ground Track Names, Dates and Types)
}
\examples{
\dontrun{

require(IceSat2R)

orbs <- latest_orbits(verbose = TRUE)
orbs
}
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{RGT_cycle_14}
\alias{RGT_cycle_14}
\title{Reference Ground Tracks (RGTs) for IceSat-2 Cycle 14}
\format{
An object of class \code{data.table} (inherits from \code{data.frame}) with 131765 rows and 8 columns.
}
\usage{
data(RGT_cycle_14)
}
\description{
The data includes the following columns: "longitude", "latitude", "day_of_year", "Date", "hour", "minute", "second" and "RGT". The "RGT" column consists of
1387 Reference Ground Tracks (RGTs) for the IceSat-2 Cycle 14 (from 'December 22, 2021' to 'March 23, 2022')
}
\details{
The following code snippet shows how to come to the "RGT_cycle_14" data. The same can be done with any of the available RGT Cycles. For the following code
I utilized 8 threads to speed up the pre-processing of the downloaded .kml files (the code takes approximately 15 minutes on my Linux Personal Computer),

\code{require(IceSat2R)} \cr
\code{require(magrittr)} \cr
\code{require(sf)} \cr

\code{avail_cycles = available_RGTs(only_cycle_names = TRUE)} \cr
\code{avail_cycles} \cr

\code{idx_cycle = 14} \cr

\code{choose_cycle = avail_cycles[idx_cycle]} \cr
\code{choose_cycle} \cr

\code{res_rgt_many = time_specific_orbits(RGT_cycle = choose_cycle, download_method = 'curl', threads = parallel::detectCores(), verbose = TRUE)} \cr

\code{RGT_cycle_14 = sf::st_coordinates(res_rgt_many)} \cr
\code{colnames(RGT_cycle_14) = c('longitude', 'latitude')} \cr
\code{RGT_cycle_14 = data.table::data.table(RGT_cycle_14)} \cr
\code{RGT_cycle_14$day_of_year = res_rgt_many$day_of_year} \cr
\code{RGT_cycle_14$Date = as.Date(res_rgt_many$Date_time)} \cr
\code{RGT_cycle_14$hour = lubridate::hour(res_rgt_many$Date_time)} \cr
\code{RGT_cycle_14$minute = lubridate::minute(res_rgt_many$Date_time)} \cr
\code{RGT_cycle_14$second = lubridate::second(res_rgt_many$Date_time)} \cr
\code{RGT_cycle_14$RGT = res_rgt_many$RGT} \cr
\code{RGT_cycle_14} \cr
}
\examples{

require(IceSat2R)
require(data.table)

data(RGT_cycle_14)
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/API_utils.R
\name{switch_full}
\alias{switch_full}
\title{Convert a full month name to a numeric value}
\usage{
switch_full(month)
}
\arguments{
\item{month}{a character string specifying one of the twelve months, i.e. ("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December")}
}
\value{
a character string which will correspond to the numeric value of the input month, i.e. ("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12")
}
\description{
Convert a full month name to a numeric value
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{available_RGTs}
\alias{available_RGTs}
\title{Reference Ground Tracks (RGTs)}
\usage{
available_RGTs(
  only_cycle_names = FALSE,
  technical_specs_url = "https://icesat-2.gsfc.nasa.gov/science/specs",
  verbose = FALSE
)
}
\arguments{
\item{only_cycle_names}{a boolean. If TRUE then only the RGT (Reference Ground Track) cycle names will be returned. Otherwise all orbit files, dates and cycle names.}

\item{technical_specs_url}{a character string specifying the technical specs website}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
a list object with the available orbit files, dates and cycle names
}
\description{
This function returns the url's of all Reference Ground Track (RGT) cycles. Moreover, it returns the dates that belong to each RGT cycle and the names of the RGT cycles.
}
\examples{
\dontrun{

require(IceSat2R)

# .................................................
# all available orbit files, dates and cycle names
# .................................................

avail_dat <- available_RGTs(
  only_cycle_names = FALSE,
  verbose = TRUE
)
avail_dat

# .............................
# receive only the cycle names
# .............................

avail_cycles <- available_RGTs(
  only_cycle_names = TRUE,
  verbose = TRUE
)
avail_cycles
}
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{time_specific_orbits}
\alias{time_specific_orbits}
\title{Time Specific Orbits}
\usage{
time_specific_orbits(
  date_from = NULL,
  date_to = NULL,
  RGT_cycle = NULL,
  download_method = "curl",
  threads = 1,
  verbose = FALSE
)
}
\arguments{
\item{date_from}{either NULL or a character string specifying the start date in the format 'yyyy-MM-dd' (such as '2020-01-01'). If this parameter is NULL then the 'RGT_cycle' parameter must be specified}

\item{date_to}{either NULL or a character string specifying the end date in the format 'yyyy-MM-dd' (such as '2020-01-01'). If this parameter is NULL then the 'RGT_cycle' parameter must be specified}

\item{RGT_cycle}{a character vector specifying the RGT (Reference Ground Track) cycle(s) (the specific revisit times of ICESAT-2). This parameter can be greater or equal to 1 with a maximum 'RGT-cycle' names as determined by the output of the 'available_RGTs(only_cycle_names = TRUE)' function. The computation time of a single 'RGT-cycle' might take approximately 15 minutes utilizing 8 threads (in parallel) and require approximately 2 GB of memory.}

\item{download_method}{a character string specifying the download method. Corresponds to the 'method' parameter of the 'utils::download.file()' function. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'}

\item{threads}{an integer that specifies the number of threads to use in parallel when processing the data}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
an 'sf' object that will include one or more Reference Ground Tracks  (see the 'RGT' column of the output object)
}
\description{
This function shows the reference ground track time and locations for specific date ranges. "Updated KML files have been posted to the 'tech-specs'
website (see the 'references' section for more details) containing individual files for each Reference Ground Track (RGT) with a date and time stamp
posted every 420 kilometers along-track (roughly 1 minute of flight time in between each point). The first RGT is 234; this is where the time series
begins. The date of each RGT is in the file name, so the user can easily ascertain where and when ICESat-2 will be on a particular day."
}
\examples{
\dontrun{

require(IceSat2R)

# ................................................
# RGTs (Reference Ground Tracks) for a single day
# ................................................

res_rgt_one <- time_specific_orbits(
  date_from = "2019-06-01",
  date_to = "2019-06-01",
  download_method = "curl",
  threads = 1,
  verbose = TRUE
)
str(res_rgt_one)

# ..........................................................
# RGTs (Reference Ground Tracks) for a specific time period
# ..........................................................

res_rgt_many <- time_specific_orbits(
  date_from = "2019-06-01",
  date_to = "2019-06-03",
  download_method = "curl",
  threads = 1,
  verbose = TRUE
)
str(res_rgt_many)


# .........................................................
# processing more than one RGTs for a specified date range
# .........................................................

res_rgt_inters <- time_specific_orbits(
  date_from = "2021-03-23",
  date_to = "2021-03-26",
  download_method = "curl",
  threads = 1,
  verbose = TRUE
)
str(res_rgt_inters)

table(res_rgt_inters$cycle)
table(res_rgt_inters$day_of_year)
table(res_rgt_inters$RGT)

# ...............................................................
# RGTs (Reference Ground Tracks) for a selected 'cycle'
# Observe the available RGT-cycles and use all available threads
# ...............................................................

avail_cycles <- available_RGTs(
  only_cycle_names = TRUE,
  verbose = TRUE
)
avail_cycles

choose_cycle <- avail_cycles[3]

res_rgt_many <- time_specific_orbits(
  RGT_cycle = choose_cycle,
  download_method = "curl",
  threads = parallel::detectCores(),
  verbose = TRUE
)
}
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{vsi_nominal_orbits_wkt}
\alias{vsi_nominal_orbits_wkt}
\title{Utilizing Virtual File Systems (vsi) and Well Known Text (WKT) to access the 'nominal orbits'}
\usage{
vsi_nominal_orbits_wkt(
  orbit_area,
  track = "GT7",
  rgt_repeat = 1,
  wkt_filter = NULL,
  download_method = "curl",
  download_zip = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{orbit_area}{a character string specifying the earth partition to use, it can be one of 'antarctic', 'arctic', 'western_hemisphere' and 'eastern_hemisphere'}

\item{track}{a character string specifying the orbit track. Can be one of 'GT1L','GT1R','GT2L','GT2R','GT3L','GT3R' or 'GT7'}

\item{rgt_repeat}{an integer specifying the orbit repeat. This parameter defaults to 1 and it is relevant if a user chooses one of the 'western_hemisphere' or 'eastern_hemisphere' where there are 8 orbit repeats, whereas for the 'antarctic' and 'arctic' there is only 1 repeat}

\item{wkt_filter}{either NULL, or a Well Known Text (WKT) character string to allow a user to restrict to an area of interest rather than processing all data (this parameter will be used as input to the 'sf::st_read()' function)}

\item{download_method}{a character string specifying the download method. Corresponds to the 'method' parameter of the 'utils::download.file()' function. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'}

\item{download_zip}{a boolean. If TRUE the .zip file will be first downloaded and then the .kml files will be returned, otherwise the 'gdalinfo' function will be used as input to the R 'system2()' function to read the .kml files without downloading the .zip file. The 'gdalinfo' command requires that the user has configured GDAL properly. Set the parameter 'download_zip' to TRUE if GDAL is not (properly) installed.}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
an 'sf' object
}
\description{
Utilizing Virtual File Systems (vsi) and Well Known Text (WKT) to access the 'nominal orbits'
}
\examples{
\dontrun{

require(IceSat2R)
require(magrittr)

# ......................................
# processing all data of the orbit area
# ......................................

dat_rgt <- vsi_nominal_orbits_wkt(
  orbit_area = "eastern_hemisphere",
  track = "GT7",
  rgt_repeat = 1,
  wkt_filter = NULL,
  download_method = "curl",
  download_zip = FALSE,
  verbose = TRUE
)
str(dat_rgt)


# ...........................................
# extracting nominal orbits only for the WKT
# ...........................................

WKT <- "POLYGON ((-14.765 18.979, -11.25 18.979, -11.25 21.943, -14.765 21.943, -14.765 18.979))"

dat_rgt <- vsi_nominal_orbits_wkt(
  orbit_area = "western_hemisphere",
  track = "GT3R",
  rgt_repeat = 8,
  wkt_filter = WKT,
  download_method = "curl",
  download_zip = FALSE,
  verbose = TRUE
)
str(dat_rgt)
dat_rgt[[1]]$RGT # Reference Ground Tracks of input WKT

# .............................
# Visualize the results
# (first compute the centroid)
# .............................

wkt_sf <- sf::st_as_sfc(WKT, crs = 4326)
centr_wkt <- sf::st_coordinates(sf::st_centroid(wkt_sf))

RGTs <- mapview::mapview(dat_rgt, legend = F)
AOI_wkt <- mapview::mapview(wkt_sf, legend = F)

lft <- RGTs + AOI_wkt
lft@map \%>\% leaflet::setView(
  lng = as.numeric(centr_wkt[, "X"]),
  lat = as.numeric(centr_wkt[, "Y"]),
  zoom = 7
)
}
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs

https://gdal.org/user/virtual_file_systems.html
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/API_utils.R
\name{download_file}
\alias{download_file}
\title{Customized function to download files}
\usage{
download_file(url, destfile, download_method, verbose = FALSE)
}
\arguments{
\item{url}{a character string specifying a valid url}

\item{destfile}{a character string specifying a valid path where the output file will be saved}

\item{download_method}{a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function}

\item{verbose}{a boolean. If TRUE then information will appear in the console}
}
\description{
Customized function to download files
}
\examples{

\dontrun{

require(IceSat2R)

# the default timeout is 60 and we set it to 600
options(timeout = 600)

# we specify a URL and a temporary file
default_url = "https://icesat-2.gsfc.nasa.gov/sites/default/"
url_downl = glue::glue('{default_url}files/page_files/IS2RGTscycle12datetime.zip')
tmp_f = tempfile(fileext = '.zip')

# then we download the file
downl_f = download_file(url = url_downl,
                        destfile = tmp_f, 
                        download_method = 'curl',
                        verbose = TRUE)
}
}
\references{
https://github.com/mlverse/torchdatasets/blob/master/R/utils.R#L20
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_Atlas_data.R
\name{get_level3a_data}
\alias{get_level3a_data}
\title{Get IceSat-2 ATLAS 'Level-3A' data for a time interval (up to 1 year)}
\usage{
get_level3a_data(
  minx,
  miny,
  maxx,
  maxy,
  startDate,
  endDate,
  trackId,
  beamName = NULL,
  product = "atl08",
  client = "portal",
  outputFormat = "csv",
  file_path_zip = NULL,
  download_method = "curl",
  verbose = FALSE
)
}
\arguments{
\item{minx}{the 'minx' parameter of the bounding box}

\item{miny}{the 'miny' parameter of the bounding box}

\item{maxx}{the 'maxx' parameter of the bounding box}

\item{maxy}{the 'maxy' parameter of the bounding box}

\item{startDate}{a character string specifying the Data collection of the \emph{start} Date in the format 'yyyy-MM-dd' (such as '2020-01-01')}

\item{endDate}{a character string specifying the Data collection of the \emph{end} Date in the format 'yyyy-MM-dd' (such as '2020-01-01')}

\item{trackId}{an integer specifying the 'Reference ground track ID' (see the examples section on how to come to the 'trackId' and to a bounding box of 1 or 5 degrees globally)}

\item{beamName}{either NULL (data for all 6 beams will be returned) or a character vector specifying the Beam Name(s), It can be one or more of the 'gt1l', 'gt1r', 'gt2l', 'gt2r', 'gt3l' or 'gt3r'}

\item{product}{a character string specifying the input data product (default value is 'atl08'). It can be one of 'atl06', 'atl07', 'atl08', 'atl10', 'atl12' or 'atl13'}

\item{client}{a character string specifying the 'Referring client'. Can be one of 'portal' or 'jupyter' (default is 'portal')}

\item{outputFormat}{a character string specifying the output format of the downloaded data. One of 'csv', 'json' or 'zip'}

\item{file_path_zip}{either NULL or a character string specifying a valid path to the output .zip file. This parameter will normally be a valid path if the 'outputFormat' parameter is set to 'zip'. If it's NULL and the 'outputFormat' parameter is 'zip' then the downloaded '.zip' file will be converted and returned as a data.table object}

\item{download_method}{a character string specifying the download method to use. Can be one of 'internal', 'wininet' (Windows only), 'libcurl', 'wget', 'curl' or 'auto'. For more information see the documentation of the 'utils::download.file()' function}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
if the 'file_path_zip' parameter is NULL it returns either a data.table (if outputFormat is 'csv') or a nested list (if outputFormat is 'json') else the file path where the .zip file is saved. In case that the 'outputFormat' is set to 'zip' and the 'file_path_zip' parameter to NULL then a data.table will be returned.
}
\description{
This function allows the user to download IceSat-2 ATLAS 'Level-3A' data for a specific time interval, bounding box, track and beam.
}
\details{
Up to 1 year worth of ICESat-2 \emph{Level-3A} product data can be downloaded. \emph{Note:} requests are limited to 5x5 degree spatial bounding box selection

\describe{
 \item{'atl06', \emph{Land Ice Height} (Version 4)}{This data set \emph{(ATL06)} provides geolocated, land-ice surface heights (above the WGS 84 ellipsoid, ITRF2014 reference frame), plus ancillary parameters that can be used to interpret and assess the quality of the height estimates. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl07', \emph{Sea Ice Height} (Version 4)}{The data set \emph{(ATL07)} contains along-track heights for sea ice and open water leads (at varying length scales) relative to the WGS84 ellipsoid (ITRF2014 reference frame) after adjustment for geoidal and tidal variations, and inverted barometer effects. Height statistics and apparent reflectance are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl08', \emph{Land and Vegetation Height} (Version 4)}{This data set \emph{(ATL08)} contains along-track heights above the WGS84 ellipsoid (ITRF2014 reference frame) for the ground and canopy surfaces. The canopy and ground surfaces are processed in fixed 100 m data segments, which typically contain more than 100 signal photons. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl10', \emph{Sea Ice Freeboard} (Version 4)}{This data set \emph{(ATL10)} contains estimates of sea ice freeboard, calculated using three different approaches. Sea ice leads used to establish the reference sea surface and descriptive statistics used in the height estimates are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl12', \emph{Ocean Surface Height} (Version 4)}{This data set \emph{(ATL12)} contains along-track sea surface heights at variable length scales over cloud-free regions. Estimates of height distributions, surface roughness, surface slope, and apparent reflectance are also provided. The data were acquired by the Advanced Topographic Laser Altimeter System (ATLAS) instrument on board the Ice, Cloud and land Elevation Satellite-2 (ICESat-2) observatory}
 \item{'atl13', \emph{Inland Water Surface Height} (Version 4)}{This data set \emph{(ATL13)} contains along-track water surface heights and descriptive statistics for inland water bodies. Water bodies include lakes, reservoirs, bays, and estuaries. Descriptive statistics include along-track surface slope (where data permit), mean and standard deviation, subsurface signal (532 nm) attenuation, wave height, and coarse depth to bottom topography}
 }
}
\examples{

\dontrun{

require(IceSat2R)

#.......................................................
# parameter setting based on the output results from the
# 'examples' section of the 'get_atlas_data()' function
# so that we can iterate over the RGT's and Products for
# the specified time interval
#.......................................................

bbx = c(xmin = 140, ymin = -6.641235, xmax = 145, ymax = -1.641235)
start_date = "2021-02-03"
end_date = "2021-02-15"
RGTs = c(627, 756, 688, 619, 817)
Products = c('atl08', 'atl13')

#........................................................
# loop over the specified parameters and save the results
#........................................................

dat_out = logs_out = list()

for (prod_i in Products) {
  for (track_i in RGTs) {

    name_iter = glue::glue("{track_i}_{prod_i}")
    cat(glue::glue("RGT: '{track_i}'  Product: '{prod_i}'"), '\n')

    iter_dat = get_level3a_data(minx = as.numeric(bbx['xmin']),
                                miny = as.numeric(bbx['ymin']),
                                maxx = as.numeric(bbx['xmax']),
                                maxy = as.numeric(bbx['ymax']),
                                startDate = start_date,
                                endDate = end_date,
                                trackId = track_i,
                                beamName = NULL,        # return data of all 6 beams
                                product = prod_i,
                                client = 'portal',
                                outputFormat = 'csv',
                                verbose = FALSE)

    iter_logs = list(RGT = track_i,
                     Product = prod_i,
                     N_rows = nrow(iter_dat))

    logs_out[[name_iter]] = data.table::setDT(iter_logs)
    dat_out[[name_iter]] = iter_dat
  }
}


#.........................................
# each sublist corresponds to a different
# parameter setting (Track, Product)
#.........................................

dat_out

#.....
# Logs  (including the number of rows for each parameter setting)
#.....

dtbl_logs = data.table::rbindlist(logs_out)
dtbl_logs = subset(dtbl_logs, N_rows > 0)
dtbl_logs = dtbl_logs[order(dtbl_logs$N_rows, decreasing = T), ]
dtbl_logs

}
}
\references{
https://openaltimetry.earthdatacloud.nasa.gov

https://nsidc.org/data/icesat-2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/elapsed_time.R
\name{compute_elapsed_time}
\alias{compute_elapsed_time}
\title{elapsed time in hours & minutes & seconds}
\usage{
compute_elapsed_time(time_start)
}
\arguments{
\item{time_start}{a numeric value specifying the start time}
}
\value{
It does not return a value but only prints the time in form of a character string in the R session
}
\description{
elapsed time in hours & minutes & seconds
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IceSat2R.R
\name{IceSat2R}
\alias{IceSat2R}
\title{The "IceSat2R" package}
\description{
An important aspect of the "IceSat2R" (https://cran.r-project.org/web/packages/IceSat2R/index.html) package is that it includes the code, documentation, and
examples so that users can retrieve, process, and analyze data based on specific workflows.
}
\details{
For instance,

\itemize{
  \item A user can select an "area of interest" (AOI) either programmatically or interactively
  \item If the "Reference Ground Track" (RGT) is not known, the user has the option to utilize either
    \enumerate{
      \item one of the "overall_mission_orbits()" or "time_specific_orbits()" to compute the RGT(s) for a pre-specified global area or for a time period, or
      \item one of the "vsi_nominal_orbits_wkt()" or "vsi_time_specific_orbits_wkt()" to compute the RGT(s) for a specific AOI
    }
  \item Once the RGT is computed it can be verified with the "getTracks()" function of the "OpenAltimetry Web API" (https://openaltimetry.earthdatacloud.nasa.gov)
  \item Finally the user can utilize one of the "get_atlas_data()" or "get_level3a_data()" functions to retrieve the data for specific product(s), Date(s) and Beam(s)
}

This work-flow is illustrated also in a \href{https://raw.githubusercontent.com/mlampros/IceSat2R/master/inst/joss_paper/figures/icesat_2_diagram.png}{diagram}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_grid.R
\name{degrees_to_global_grid}
\alias{degrees_to_global_grid}
\title{Create a global grid based on degrees}
\usage{
degrees_to_global_grid(
  minx = -180,
  maxx = 180,
  maxy = 90,
  miny = -90,
  degrees = 1,
  square_geoms = TRUE,
  crs_value = 4326,
  verbose = FALSE
)
}
\arguments{
\item{minx}{the 'minx' parameter of the bounding box}

\item{maxx}{the 'maxx' parameter of the bounding box}

\item{maxy}{the 'maxy' parameter of the bounding box}

\item{miny}{the 'miny' parameter of the bounding box}

\item{degrees}{a numeric value specifying the degrees. It defaults to 1.0}

\item{square_geoms}{a boolean. If FALSE then a hexagonal grid will be created}

\item{crs_value}{a value. The coordinate reference system of the output grid. The crs must correspond to the EPSG of the input 'minx', 'maxx', 'miny' and 'maxy'. Moreover, the 'EPSG:4326' returns a degrees grid as output.}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
a simple features (sf) object
}
\description{
This function allows the user to create a degrees grid based on an input bounding box
}
\details{
The default global input 'minx', 'maxx', 'miny' and 'maxy' correspond to a WKT of "POLYGON ((-180 -90, 180 -90, 180 90, -180 90, -180 -90))"
When 'minx', 'maxx', 'miny' and 'maxy' EPSG is lat,long (or 'EPSG:4326') I expect the output grid to be in 'degrees' unit. Using an EPSG other than 'EPSG:4326' might
return a different output unit (such as meters). See also the following 'Stackoverflow' thread for more information, https://stackoverflow.com/a/64903836/8302386

Based on 'Approximate Metric Equivalents' 1 degree is approximately 111 km  (or 60 nautical miles), Reference: https://www.usna.edu/Users/oceano/pguth/md_help/html/approx_equivalents.htm
}
\examples{

\dontrun{

require(IceSat2R)

#...........................................
# 'OpenAltimetry' allows 1x1 degree bounding
# box selection for the 'atl03' Product
#...........................................

gl_grid_1_d = degrees_to_global_grid(degrees = 1.0, verbose = TRUE)
gl_grid_1_d
# summary(gl_grid_1_d$area)

#...........................................................................
# 'OpenAltimetry' allows 5x5 degree bounding box selection for the following
# Products: 'atl06', 'atl07', 'atl08', 'atl10', 'atl12', 'atl13', 'level3a'
#...........................................................................

gl_grid_5_d = degrees_to_global_grid(degrees = 5.0, verbose = TRUE)
gl_grid_5_d

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mission_orbits.R
\name{vsi_time_specific_orbits_wkt}
\alias{vsi_time_specific_orbits_wkt}
\title{Utilizing Virtual File Systems (vsi) and Well Known Text (WKT) to access the 'time specific orbits'}
\usage{
vsi_time_specific_orbits_wkt(
  date_from,
  date_to,
  RGTs,
  wkt_filter = NULL,
  download_zip = FALSE,
  verbose = FALSE
)
}
\arguments{
\item{date_from}{a character string specifying the 'start' date in the format 'yyyy-MM-dd' (such as '2020-01-01')}

\item{date_to}{a character string specifying the 'end' date in the format 'yyyy-MM-dd' (such as '2020-01-01')}

\item{RGTs}{a character vector (consisting of one or more) Reference Ground Track (RGT). See the Examples section on how to come to these RGTs based on the "vsi_nominal_orbits_wkt()" function}

\item{wkt_filter}{either NULL, or a Well Known Text (WKT) character string to allow a user to restrict to an area of interest rather than processing all data. It is possible that the WKT won't intersect with any of the available time specific orbits due to the sparsity of the coordinates (the output in that case will be an empty list)}

\item{download_zip}{a boolean. If TRUE the .zip file will be first downloaded and then the .kml files will be returned, otherwise the 'gdalinfo' function will be used as input to the R 'system2()' function to read the .kml files without downloading the .zip file. The 'gdalinfo' command requires that the user has configured GDAL properly. Set the parameter 'download_zip' to TRUE if GDAL is not (properly) installed.}

\item{verbose}{a boolean. If TRUE then information will be printed out in the console}
}
\value{
a list of 'sf' objects where each sublist will represent a different RGT cycle
}
\description{
Utilizing Virtual File Systems (vsi) and Well Known Text (WKT) to access the 'time specific orbits'
}
\details{
In case that this function does not return any results (empty list object) for a specified 'wkt_filter' parameter, then use a bigger Well Known Text (WKT) area. This is required because the 'time specific orbits' (points) are quite sparse.

Moreover, set the parameter 'download_zip' to TRUE if the 'gdalinfo' function returns internally an empty character string. In that case also a warning will be shown in the R session.
}
\examples{
\dontrun{

require(IceSat2R)
require(magrittr)

# ...........................................
# extracting nominal orbits only for the WKT
# ...........................................

WKT <- "POLYGON ((-14.765 18.979, -11.25 18.979, -11.25 21.943, -14.765 21.943, -14.765 18.979))"

dat_rgt <- vsi_nominal_orbits_wkt(
  orbit_area = "western_hemisphere",
  track = "GT3R",
  rgt_repeat = 8,
  wkt_filter = WKT,
  download_method = "curl",
  download_zip = FALSE,
  verbose = TRUE
)
str(dat_rgt)

out_rgt <- dat_rgt[[1]]$RGT

# .........................................
# time specific RGTs (for a time interval)
# request using a single RGT cycle
# .........................................

date_start <- "2020-01-01"
date_end <- "2020-02-01"

orb_cyc_single <- vsi_time_specific_orbits_wkt(
  date_from = date_start,
  date_to = date_end,
  RGTs = out_rgt,
  wkt_filter = WKT,
  verbose = TRUE
)
str(orb_cyc_single)

# .........................................
# time specific RGTs (for a time interval)
# request using more than one RGT cycles
# .........................................

date_start <- "2019-11-01"
date_end <- "2020-01-01"

orb_cyc_multi <- vsi_time_specific_orbits_wkt(
  date_from = date_start,
  date_to = date_end,
  RGTs = out_rgt,
  wkt_filter = WKT,
  verbose = TRUE
)
str(orb_cyc_multi)
table(orb_cyc_multi$cycle)


# .......................................................
# visualization of the output cycles (including the WKT)
# .......................................................

orb_cyc_multi$cycle <- as.factor(orb_cyc_multi$cycle)
mp_orb <- mapview::mapview(orb_cyc_multi, legend = TRUE, zcol = "cycle")

sf_aoi <- sf::st_as_sfc(WKT, crs = 4326)
mp_aoi <- mapview::mapview(sf_aoi, alpha.regions = 0.3, legend = F)

mp_orb + mp_aoi
}
}
\references{
https://icesat-2.gsfc.nasa.gov/science/specs

https://gdal.org/user/virtual_file_systems.html
}
